/***************************************************************************
                            PackageSelect.cpp
                             -------------------                                         
    begin                : Fri Oct 29 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/



#include <alloca.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <newt.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qfont.h>
#include "log.h"
#include "windows.h"
#include "smp.h"
#include <fstream.h>

#include "PackageSelect.h"
#include "mountpoint.h"
#include "packagelistitem.h"

PackageSelect::PackageSelect(	QWidget* parent, InstallSelect *a_install, const char* name ):
	PackageSelectData( parent, name ), install( a_install ), initialized( false )
{
	QFont font( QFont::defaultFont() );
	font.setPointSize( 11 );
	font.setFamily( "times" );
	viewPackages->setFont( font );
}

PackageSelect::~PackageSelect()
{
}

void PackageSelect::showEvent( QShowEvent * )
{
 	qApp->setOverrideCursor( Qt::waitCursor );
	if( !initialized ){
  	install->loadData();
  	insertItems();
  	initialized = true;
	}else{
		initializeSelection();
	}
	displayFreeSpace();
 	qApp->restoreOverrideCursor();
}

void PackageSelect::displayFreeSpace()
{
	MountPointList mps = MountPoints::allDirectories();

	MountPoint *mp;
	int iFreeSpace = 0;
	for( mp = mps.first(); mp != NULL; mp = mps.next() ){
    if( mp->name().ascii()[0] == '/' )// some good one
			iFreeSpace += (int)(mp->free() / 0x100000);
	}
  lcdFreeSpace->display( iFreeSpace );
}

void PackageSelect::slotSelectionChanged(class QListViewItem *a_item )
{
	int type, count;
	char *buf;
	descriptionTextArea->clear();
	lboxRequires->clear();
	if( ((PackageListItem*)a_item)->package ){
		QString str;
		if( headerGetEntry( ((PackageListItem*)a_item)->package->h, RPMTAG_VERSION, &type,
			( void **)&buf, &count ) )
			str = tr("Version: ") + QString::fromLatin1( buf );
		if(	headerGetEntry( ((PackageListItem*)a_item)->package->h, RPMTAG_DESCRIPTION, &type,
			( void **)&buf, &count ) )
			str = str +"\n"+ QString::fromLatin1( buf );
		descriptionTextArea->setText( str );

		char **dataBufferName;
		int rpmItemTypeName, itemCountName, tmp;
  	int statusBadName = headerGetEntry(((PackageListItem*)a_item)->package->h,RPMTAG_REQUIRENAME, &rpmItemTypeName,
  		( void **) &dataBufferName,  &itemCountName);
  	if( statusBadName && statusBadName != 1 )	return;
		char **dataBufferVersion;
		int rpmItemTypeVersion;
  	int statusBadVersion = headerGetEntry(((PackageListItem*)a_item)->package->h,RPMTAG_REQUIREVERSION, &rpmItemTypeVersion,
  		( void **) &dataBufferVersion, &tmp );
  	if( statusBadVersion && statusBadVersion != 1 )	return;
   	for( int ipt = 0; ipt < itemCountName ; ipt ++ ){
			if( rpmItemTypeName == RPM_STRING_ARRAY_TYPE )
			  if (dataBufferName)
			    if (dataBufferName[ipt]){
			      str = QString::fromLatin1( dataBufferName[ipt] );
					}else
  					continue;
			if( rpmItemTypeVersion == RPM_STRING_ARRAY_TYPE )
   		  if (dataBufferVersion)
   		    if (dataBufferVersion[ipt])
					  str += " "+ QString::fromLatin1( dataBufferVersion[ipt] );
			lboxRequires->insertItem( str );
		}
	}
}

void PackageSelect::slotCurrentChanged(class QListViewItem *)
{
}

void PackageSelect::slotTotalChanged( int a_size )
{
	int newsize = a_size;
//printf( "size: %d\n", newsize );fflush( stdout );
	int i = ((unsigned int)newsize) >> 20;
	if( i < 0 ) i = 0;
	lcdNeededSpace->display( i );
}

void PackageSelect::slotToggleSelectionOfPackage(class QListViewItem * a_item )
{
	((PackageListItem *)a_item)->setPackageSelected();
}

void PackageSelect::updateDependencies()
{
	if( !initialized ) return;
 	qApp->setOverrideCursor( Qt::waitCursor );
	install->verifyDependencies();
	initializeSelection();
 	qApp->restoreOverrideCursor();
}

void PackageSelect::insertItems()
{
	viewPackages->clear();
	viewPackages->setSorting( -1 );
	root = new PackageListItem( viewPackages, &(install->ps) );
	connect( root, SIGNAL( totalChanged(int) ), this, SLOT( slotTotalChanged(int) ) );

	QListViewItem* tmpItem;
	component *comps = install->cs.comps;
	for( int i = 0; i < install->cs.numComponents; i++ ){
		if( comps[i].inmenu ){
    	QString group( QString::fromLatin1( comps[i].name ) );
  		PackageListItem *gr = findAppropriateGroup( group );
			if( !gr ) gr = root;
    	for( int k = 0; k < comps[i].ps.numPackages; k++ ){
    		packageInfo *package = comps[i].ps.packages[k];
//if( group == "Base" )
//  printf( "Group: %s, name: %s, selected: %d\n", group.ascii(), package->name, package->selected );fflush( stdout );
    		tmpItem = new PackageListItem(
    			gr,	QString::fromLatin1( package->name ),	package	);
    	}
		}
	}
	PackageListItem *gr = NULL;
	QString others( "Others" );
	for( int i = 0; i < install->ps.numPackages; i++ ){
		if( !install->ps.packages[i]->inmenu ){
  		if( !gr )
				gr = findAppropriateGroup( others );
  		tmpItem = new PackageListItem(
  			gr,	QString::fromLatin1( install->ps.packages[i]->name ),	install->ps.packages[i]	);
		}
	}
	viewPackages->setSorting( 0 );
	initializeSelection();
	root->setOpen( true );
}

PackageListItem * PackageSelect::findAppropriateGroup( QString& a_group )
{
	PackageListItem *tmpItem = root;
	if( tmpItem->text(0) == a_group )
		return root;
	int pos = 0;
	int prevpos = 0;
	PackageListItem *prevItem;
	do{
		pos = a_group.find( "/", pos );
		QString cur = a_group.mid( prevpos, (pos!=-1?pos:a_group.length()) - prevpos );
		prevpos = ++pos;
//printf( "cur: %s\n", cur.ascii() );
		prevItem = tmpItem;
		tmpItem = (PackageListItem*)tmpItem->firstChild();
		while( tmpItem ){
//printf( "tmpItem: %s\n", tmpItem->text(0).ascii() );
			if( !tmpItem->text(0).compare( cur ) ){
				prevItem = tmpItem;
				break;
			}
			tmpItem = (PackageListItem*)tmpItem->nextSibling();
		}
		if( !tmpItem )
			prevItem = tmpItem = new PackageListItem( prevItem, cur );
	}while( pos > 0 );
	return prevItem;
}

void PackageSelect::initializeSelection()
{
	for( int i = 0; i < install->ps.numPackages; i++ ){
		if( install->ps.packages[i]->item )
			install->ps.packages[i]->item->setPackageSelected(
				install->ps.packages[i]->selected, false );
	}
	int i = 0;
	root->initSize( &i, &i );
}




















