/***************************************************************************
                               fdisk.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "config.h"
#include "fdisk.h"
#include "liconloader.h"
#include <qregexp.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qvalidator.h>
#include <qobjectlist.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qdir.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <linux/hdreg.h>	/* HDIO_GETGEO */
#include <linux/unistd.h>	/* _syscall */
#include <fcntl.h>		/* O_RDWR */
#include <sys/ioctl.h>
#include <stdlib.h> /* system() */
#include <sys/swap.h>
#include <sys/mount.h>
#include <stream.h>
#include <mountpoint.h>
#include <stdio.h>
#include <mntent.h>

static struct SysTypes {
    unsigned char type;
    const char *name;
} sys_types[] = {
    {0, QT_TR_NOOP("Not used")},
    {1, "DOS 12-bit FAT"},
    {2, "XENIX /"},
    {3, "XENIX /usr"},
    {4, "DOS 16-bit FAT <32M"},
    {5, "Extended" }, // was: "DOS Extended"
    {6, "DOS / Windows" },  // was "DOS 16-bit FAT >=32M"
    {7, "HPFS / NTFS"},
    {8, "AIX boot or SplitDrive"},
    {9, "AIX data or Coherent"},
    {0x0a, "OS/2 Boot Manager"},
    {0x0b, "Win95 FAT32"},
    {0x0c, "Win95 FAT32 (LBA)"},
    {0x0e, "Win95 FAT16 (LBA)"},
    {0x0f, "Win95 Extended (LBA)"},
    {0x10, "OPUS"},
    {0x11, "Hidden DOS FAT12"},
    {0x12, "Compaq diagnostics"},
    {0x14, "Hidden DOS FAT16"},
    {0x16, "Hidden DOS FAT16 (big)"},
    {0x17, "Hidden HPFS/NTFS"},
    {0x18, "AST Windows swapfile"},
    {0x24, "NEC DOS"},
    {0x3c, "PartitionMagic recovery"},
    {0x40, "Venix 80286"},
    {0x41, "Linux/MINIX (sharing disk with DRDOS)"},
    {0x42, "SFS or Linux swap (sharing disk with DRDOS)"},
    {0x43, "Linux native (sharing disk with DRDOS)"},
    {0x50, "DM (disk manager)"},
    {0x51, "DM6 Aux1 (or Novell)"},
    {0x52, "CP/M or Microport SysV/AT"},
    {0x53, "DM6 Aux3"},
    {0x54, "DM6"},
    {0x55, "EZ-Drive (disk manager)"},
    {0x56, "Golden Bow (disk manager)"},
    {0x5c, "Priam Edisk (disk manager)"},
    {0x61, "SpeedStor"},
    {0x63, "GNU HURD or Mach or Sys V/386 (such as ISC UNIX)"},
    {0x64, "Novell Netware 286"},
    {0x65, "Novell Netware 386"},
    {0x70, "DiskSecure Multi-Boot"},
    {0x75, "PC/IX"},
    {0x77, "QNX4.x"},
    {0x78, "QNX4.x 2nd part"},
    {0x79, "QNX4.x 3rd part"},
    {0x80, "MINIX until 1.4a"},
    {0x81, "MINIX / old Linux"},
    {0x82, "Swap"}, // was Linux swap
    {0x83, "Linux"}, // was Linux native
    {0x84, "OS/2 hidden C: drive"},
    {0x85, "Linux extended"},
    {0x86, "NTFS volume set"},
    {0x87, "NTFS volume set"},
    {0x93, "Amoeba"},
    {0x94, "Amoeba BBT"},		/* (bad block table) */
    {0xa0, "IBM Thinkpad hibernation"}, /* according to dan@fch.wimsey.bc.ca */
    {0xa5, "BSD/386"},			/* 386BSD */
    {0xa6, "OpenBSD"},
    {0xa7, "NeXTSTEP 486"},
    {0xa9, "NetBSD"},
    {0xb7, "BSDI fs"},
    {0xb8, "BSDI swap"},
    {0xc1, "DRDOS/sec (FAT-12)"},
    {0xc4, "DRDOS/sec (FAT-16, < 32M)"},
    {0xc6, "DRDOS/sec (FAT-16, >= 32M)"},
    {0xc7, "Syrinx"},
    {0xdb, "CP/M or Concurrent CP/M or Concurrent DOS or CTOS"},
    {0xe1, "DOS access or SpeedStor 12-bit FAT extended partition"},
    {0xe3, "DOS R/O or SpeedStor"},
    {0xe4, "SpeedStor 16-bit FAT extended partition < 1024 cyl."},
    {0xf1, "SpeedStor"},
    {0xf2, "DOS 3.3+ secondary"},
    {0xf4, "SpeedStor large partition"},
    {0xfe, "SpeedStor >1024 cyl. or LANstep"},
    {0xff, "Xenix Bad Block Table"}
};

bool Fdisk::startup = true;

#define SIZE(a)	(int(sizeof(a)/sizeof(a[0])))
QString Fdisk::sysname(unsigned char type) {
    struct SysTypes *s;

    for (s = sys_types; s - sys_types < SIZE(sys_types); s++)
      if (s->type == type)
				return s->name;
    return "Unknown";
}

int Fdisk::systype( const QString& name) {
    struct SysTypes *s;

    for (s = sys_types; s - sys_types < SIZE(sys_types); s++)
      if ( name == s->name )
				return s->type;
    return 0;
}


bool Fdisk::isExtendedType( unsigned char t ) {
    return t == 0x05 || // DOS Extended
	   t == 0x0f || // Win95 Extended (LBA)
	   t == 0x85 || // Linux extended
	   t == 0xe1 || // DOS/SpeedStor 12-bit FAT extended partition
	   t == 0xe4;   // SpeedStor 16-bit FAT extended partition < 1024 cyl
}

bool Fdisk::isSwapType( unsigned char type) {
    return type == 0x82;
}


bool Fdisk::isVFatType( unsigned char type){
    return type == 0x0b || type == 0x0c;
}

QDict<QString> * Fdisk::newPartitions = 0;

bool Fdisk::isNew( const QString& partition )
{
    if (!newPartitions)
	return QString::null;
    QString* s = (*newPartitions)[partition];
    if (s)
	return *s;
    return QString::null;
}

void Fdisk::markAsNew( const QString& partition )
{
    if ( isNew(partition) )
    	return; // nothing to do

    if (!newPartitions) {
    	newPartitions = new QDict<QString>;
    	newPartitions->setAutoDelete( TRUE );
    }
    newPartitions->insert( partition, new QString( partition) );
}

QDict<void> * Fdisk::formatPartitions = 0;

bool Fdisk::willBeFormated( const QString& partition )
{
    if( !formatPartitions )
			return FALSE;
    void* b = (*formatPartitions)[partition];
    return b != 0;
}

void Fdisk::setFormatting( const QString& partition, bool value )
{
    if ( willBeFormated(partition) == value)
			return; // nothing to do

    if (!formatPartitions) {
			formatPartitions = new QDict<void>;
    }
    if ( value )
			formatPartitions->insert( partition, (void*)1 );
    else
			formatPartitions->remove( partition );
}

/* trys to get the disk geometry of the given device <disk>. Returns TRUE on success. */
bool Fdisk::getDiskGeometry( const QString& disk, int& heads, int& sectors, int& cylinders)
{
    heads = 0;
    sectors = 0;
    cylinders = 0;

    // get harddisk geometry
    int fd;
    struct hd_geometry g;
    if( (fd = ::open( disk.ascii(), O_RDONLY)) != -1 ) {
    	if( !ioctl(fd, HDIO_GETGEO, &g) ) {
    	    heads = g.heads;
    	    sectors = g.sectors;
    	    cylinders = g.cylinders;
    	}
    	::close(fd);
    	return TRUE;
    }
    return FALSE;
}



/*
  returns the partition table of the device <disk> in sfdisk's dump format.
  In fact, this function uses sfdisk for this operation.
 */
QString Fdisk::getPartitionTable( const QString& disk)
{
    QString result;

    int fd[2];
    ::pipe(fd);
    int pid = ::fork();
    if (pid == 0) {
			::close( fd[0] );
	
		 	if (fd[1] != STDOUT_FILENO) {
 	  	  dup2( fd[1], STDOUT_FILENO);
 	    	::close( fd[1] );
		 	}
	
			// get a partition table dump from sfdisk
			execl("./extrabin/sfdisk", "./extrabin/sfdisk", "--quiet", "-uS", "-d", disk.ascii(), 0);
			exit(1);
    }
//printf( "getPartitionTable pid: %d\n", pid );
    ::close( fd[1] );

    QFile f;
    if ( f.open(IO_ReadOnly, fd[0]) ) {    // file opened successfully
		 	QTextStream t( &f );	    // use a text stream
			result = t.read();
		 	::close( fd[0] );
    }

    ::waitpid(pid, NULL, 0);

    return result;
}

/* writes the partition table of the device <disk>. The tables has to
  be in sfdisk's dump format.  In fact, this function uses sfdisk for
  this operation. Returns TRUE on success. */
bool Fdisk::writePartitionTable( const QString& disk, const QString& table)
{
    QString result;
    int fd[2];
    ::pipe(fd);
    int pid = ::fork();
    if (pid == 0) {
			::close( fd[1] );
	
		 	if (fd[0] != STDIN_FILENO) {
	 	    dup2( fd[0], STDIN_FILENO);
 		    ::close( fd[0] );
		 	}
	
			// write a partition table using sfdisk
			int res = execl( "./extrabin/sfdisk", "./extrabin/sfdisk", "-uS",
	      "-f", "-L",
#ifdef GBLI_DEVEL
        "-n", // don't write to disk
#endif
        disk.ascii(), 0 );
			exit( res );
		}
//printf( "writePartitionTable pid: %d\n", pid );
    ::close( fd[0] );

    QFile f;
    if ( f.open( IO_WriteOnly, fd[1] ) ) {    // file opened successfully
		 	QTextStream t( &f );	    // use a text stream
			t << table;
		 	::close( fd[1] );
    }

		int status = 0;
    ::waitpid( pid, &status, 0 );
//printf( "writePartitionTable status: %d\n", status );
		if( WEXITSTATUS(status) != 0 ){
			// do something, like showing error message on upper level?
			// bad news - it seems to be returning 0 all the time ?!?
		}

    return TRUE;
}


/*
  Return the list of all IDE and SCSI disk drives that are available
  on the system right now.
 */
QStringList Fdisk::availableDisks()
{
    QStringList disks;

    QString disk;
    int fd;

    // IDE drives
    QString base = "/dev/hd";
    char c = 'a';
    do {
    	disk = base + c;
    	if ( (fd = ::open( disk.ascii(), O_RDWR)) != -1) {
    	    ::close( fd );
    	    disks.append( disk );
    	}
    	c++;
    } while (c <= 'h' );
	
    // SCSI  drives
    base = "/dev/sd";
    c = 'a';

#ifdef DO_NOT_LOOK_AT_FIRST_SCSI_DISK
    c = 'b';
#endif

    do {
    	disk = base + c;
    	if ( (fd = ::open( disk.ascii(), O_RDONLY)) != -1) {
    	    ::close( fd );
    	    disks.append( disk );
    	}
    	c++;
    } while (c <= 'p' );

    // Mylex DAC960
    QString num;   
    QDir rd_dir("/dev/rd");
    if (rd_dir.count() > 0) {
      for (int controller=0; controller<8; controller++)
      	for (int logical=0; logical<32; logical++) {
      	  disk = QString("/dev/rd/c%1d%2").arg(controller).arg(logical);
      	  if ( (fd = ::open( disk.ascii(), O_RDONLY)) != -1) {
      	    ::close( fd );
      	    disks.append( disk );
      	  }
      	}
    }

    // Compaq Smart Array
    QDir ida_dir("/dev/ida");
    if (ida_dir.count() > 0) {
      for (int controller=0; controller<8; controller++)
      	for (int logical=0; logical<32; logical++) {
      	  disk = QString("/dev/ida/c%1d%2").arg(controller).arg(logical);
      	  if ( (fd = ::open( disk.ascii(), O_RDONLY)) != -1) {
      	    ::close( fd );
      	    disks.append( disk );
      	  }
      	}
    }
    
    return disks;
}

// checks if this partition is the install source
bool Fdisk::isInstallSource(QString name)
{
  FILE *mtab = fopen("/etc/mtab", "r");

  if( mtab ){
    struct mntent *mnt = getmntent( mtab );
    while( mnt ){
  	  if( !strcmp( mnt->mnt_fsname, name.ascii() ) )
  	    return TRUE;
  	  mnt = getmntent(mtab);
  	}
    fclose( mtab );
  }
  return FALSE;
}



