/***************************************************************************
				                         grub.cpp
                             -------------------
    begin                : Tue Oct 29 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Anisimov
    author email         : aa@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "grub.h"
#include "linlinehelp.h"
#include "partitionview.h"
#include "mountpoint.h"
#include "gblidebug.h"
#include "smp.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h> 

#include <alloca.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <sys/wait.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <stream.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qtextstream.h>
#include <qpushbutton.h>
#include <sys/utsname.h>
#include <stdio.h>
#include <qdir.h>
#include <qregexp.h>
#include <qlistview.h>

//---------------------------Initrd STUFF Here-------------------------
int GRUB::mkinitrd( const char *a_szPrefix, const char * a_szKernelVersion, const char * a_szInitrdImage ) {
int iRez = 0;
char szInitrdName[ 60 ];
char szCmd[ 512 ];

    if ( can_do_smp ) sprintf( szInitrdName, "%ssmp", a_szKernelVersion );
    else	sprintf( szInitrdName, "%s", a_szKernelVersion );

//    winStatus(32, 3, _( "Creating initial ramdisk..." ) );

   sync();
   system( "/bin/umount /proc" );
   char cmd[128];
   sprintf( cmd, "/bin/mount -t proc proc %s/proc", a_szPrefix );
   system( cmd );

   sprintf( szCmd, "chroot %s /bin/sh -c \"/sbin/mkinitrd -f %s --ifneeded %s\"", a_szPrefix, a_szInitrdImage, szInitrdName );
   iRez = system( szCmd );

   sync();
   sprintf( cmd, "/bin/umount %s/proc", a_szPrefix );
   system( cmd );
   system( "/bin/mount -t proc proc /proc" );

    if ( iRez ) unlink("/mnt/boot/initrd");

    return iRez;
}

int GRUB::CreateInitrdImage( const char *a_szRootPath, const char *a_szKernelVersion ) {
char szAbsinitrdImage[ 60 ];
struct stat sb;
	if ( can_do_smp ) {
		sprintf( szIinitrdImage, "/boot/initrd-%ssmp.img", a_szKernelVersion );
		if ( mkinitrd( a_szRootPath, a_szKernelVersion, szIinitrdImage ) )
	    		can_do_smp = FALSE;
		else {
			sprintf( szAbsinitrdImage, "%s%s", a_szRootPath, szIinitrdImage );
	
			if ( !stat ( szAbsinitrdImage, &sb ) )
				useInitrd = TRUE;
		}
    	} else {
		sprintf( szIinitrdImage, "/boot/initrd-%s.img", a_szKernelVersion );
    		if ( mkinitrd( a_szRootPath, a_szKernelVersion, szIinitrdImage ) )
			return -1;
		sprintf( szAbsinitrdImage, "%s%s", a_szRootPath, szIinitrdImage );
		if ( !stat( szAbsinitrdImage, &sb ) )
			useInitrd = TRUE;
	}
	return 0;
}
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~`~~~~~~

GRUBInfoItem::GRUBInfoItem(QListView *parent, bool active, const QString &partition,
			     const QString &type, const QString &label)
  : QCheckListItem(parent, "", QCheckListItem::CheckBox)
{
  setText(1,partition);
  setText(2,type);
  setText(3,label);
  setOn(active);
}


GRUBInfoItem::~GRUBInfoItem()
{
}

int GRUB::link_file( int a_iSMP, const char *a_szKernelPrefix, const char *a_szKernelVersion, const char *a_szFileName, const char *a_szMapAs ) {
char szPathName[ 1024 ];
int iPathLen;
char szLinkPath[ 1024 ];
//char szLinkedAs[ 1024 ];

	if  ( chdir( a_szKernelPrefix ) ) return -1;

	iPathLen = readlink(  a_szMapAs, szPathName, 1024 );
	 //If iPathLen == -1 file not found or it`s not readable use default file name
	if ( iPathLen != -1 ) {
		//Check if link contain perfect SMP support & look`s like current version
		szPathName[ iPathLen ] = 0;
		//Check first version number
		if ( strstr( szPathName, a_szKernelVersion ) ) {
			if ( strstr( szPathName, "smp" ) ) {
				if ( a_iSMP ) return 0;
			}
		}
		if ( unlink( a_szMapAs ) ) return errno* -1;
	}
	sprintf( szLinkPath, "%s-%s", a_szFileName, a_szKernelVersion );
	if ( a_iSMP ) strcat( szLinkPath, "smp" );
	
	if ( symlink( szLinkPath, a_szMapAs ) ) return errno * -1;

	return 0;
}

int GRUB::link_boot_files( int a_iSMP, const char *a_szKernelPrefix, const char *a_szKernelVersion ) {
int iRetStat = 0;
	iRetStat = link_file( a_iSMP, a_szKernelPrefix, a_szKernelVersion,  "System.map", "System.map" );
	if ( iRetStat < 0 ) return iRetStat;
	iRetStat = link_file( a_iSMP, a_szKernelPrefix, a_szKernelVersion,  "module-info", "module-info" );
	if ( iRetStat < 0 ) return iRetStat;
	iRetStat = link_file( a_iSMP, a_szKernelPrefix, a_szKernelVersion,  "vmlinuz", "vmlinuz" );
	if ( iRetStat < 0 ) return iRetStat;

	return 0;
}

QString GRUB::lastVersionName( QString a_QFindPath, QString a_QVersionNumber ) {
QDir d( a_QFindPath );
QString MaxVer = "", MaxVerName = "";
d.setNameFilter( "vmlinuz-" + a_QVersionNumber + ".*" );
uint i;
const QFileInfoList *list = d.entryInfoList();
QFileInfoListIterator it( *list );      // create list iterator
QFileInfo *fi;                          // pointer for traversing
while ( (fi=it.current()) ) {           // for each file...
	QString QFinder = fi->fileName().mid( 8, fi->fileName().length() - 8 );
	int iSeekPos = QFinder.find( '-', 0 );
	if ( iSeekPos != -1 ) {
		i = QFinder.find( '-', 0 ) + 1;
		while ( QFinder[ i ] >= '0' && QFinder[ i ] <= '9' ) i++;
		QFinder = QFinder.mid( 0, i );
	} else {
		i = QFinder.length();
		if ( !i ) continue;	i--;
		while( !( QFinder[ i ] >= '0' && QFinder[ i ] <= '9' ) ) {
			i--; if ( !i ) break;			
		}		
		QFinder = QFinder.left( i + 1 );
	}

	if ( strcmp( QFinder, MaxVer ) > 0 ) {
		MaxVer = QFinder; MaxVerName = fi->fileName();
	}
++it;                               // goto next list element
}
	return MaxVerName;
}	
//Only version without filename
QString GRUB::lastVersion( QString a_QFindPath, QString a_QVersionNumber ) {
QDir d( a_QFindPath );
QString MaxVer = "", MaxVerName = "";
d.setNameFilter( "vmlinuz-" + a_QVersionNumber + ".*" );
uint i;
const QFileInfoList *list = d.entryInfoList();
QFileInfoListIterator it( *list );      // create list iterator
QFileInfo *fi;                          // pointer for traversing
while ( (fi=it.current()) ) {           // for each file...
	QString QFinder = fi->fileName().mid( 8, fi->fileName().length() - 8 );
	int iSeekPos = QFinder.find( '-', 0 );
	if ( iSeekPos != -1 ) {
		i = QFinder.find( '-', 0 ) + 1;
		while ( QFinder[ i ] >= '0' && QFinder[ i ] <= '9' ) i++;
		QFinder = QFinder.mid( 0, i );
	} else {
		i = QFinder.length();
		if ( !i ) continue;	i--;
		while( !( QFinder[ i ] >= '0' && QFinder[ i ] <= '9' ) ) {
			i--; if ( !i ) break;			
		}		
		QFinder = QFinder.left( i + 1 );
	}

	if ( strcmp( QFinder, MaxVer ) > 0 ) {
		MaxVer = QFinder; MaxVerName = fi->fileName();
	}
++it;                               // goto next list element
}
	return MaxVer;
}	

char *GRUB::ConvLinuxDev2GRUB( char *a_szConvDevName, const char *a_szDeviceName ) {
char *cpPointer, *cpDeviceNamePtr;
char szTmpDriveNum[ 2 ];
char szTmpPartitionNum[ 255 ];
int iMaxDevNum;
	iMaxDevNum = 0;
	szTmpDriveNum[ 0 ] = 0; szTmpDriveNum[ 1 ] = 0;
  /*!!!!!!!!!!!!!!!*/
 	QStringList ds( Fdisk::availableDisks() );
 	QStringList::Iterator a;
	/*!!!!!!!!!!!!!*/
	cpPointer = strstr( a_szDeviceName, "/dev/" );
	if ( cpPointer ) cpDeviceNamePtr = cpPointer + 5;
	else cpDeviceNamePtr = (char *)a_szDeviceName;
	cpPointer = strstr( cpDeviceNamePtr, "fd" );
	if ( cpPointer ) {
		if ( cpPointer - cpDeviceNamePtr ) return NULL; //device which contain string fd somewhere in the text
		strncpy( a_szConvDevName, cpPointer, 3 ); //Copy also one digit number like fd0
		a_szConvDevName[ 3 ] = 0;		
	}
	cpPointer = strstr( cpDeviceNamePtr, "sd" );
	if ( cpPointer ) {
		if ( cpPointer - cpDeviceNamePtr ) return NULL; //device witch contain string fd somethere in the text
		strcpy( a_szConvDevName, "hd" );

		//FIXME: SCSI disks in BIOSes is last disks
		char cDriveStart = 0;
		for (a=ds.begin(); a != ds.end(); ++a) {
			if ( (*a).mid( 5, 1 ).ascii()[0] == 'h' ) {
				if( (*a).mid( 7, 1 ).ascii()[0] > cDriveStart ) {
					cDriveStart = (*a).mid( 7, 1 ).ascii()[0];
					iMaxDevNum ++;
				}
			}
		}
		//****************************************

		/*
		if ( *( cpPointer + 2 ) >= 'a' && *( cpPointer + 2 ) <= 'z' )
			szTmpDriveNum[ 0 ] = ( *( cpPointer + 2 ) - 'a' ) + 0x30;
		*/
		for (a=ds.begin(); a != ds.end(); ++a) {
	   		 if (*( cpPointer + 2 ) > (*a).mid( 7, 1 ).ascii()[0]  )  iMaxDevNum ++;
		}
		szTmpDriveNum[ 0 ] = iMaxDevNum + 0x30;

		strcat( a_szConvDevName, szTmpDriveNum ); //Copy drive number
		if ( *( cpPointer + 3 ) ) {
			sprintf( szTmpPartitionNum, ",%d", atoi( cpPointer + 3 ) - 1 );
			strcat( a_szConvDevName, szTmpPartitionNum );//copy partition number					
		}
	}
	cpPointer = strstr( cpDeviceNamePtr, "hd" );
	if ( cpPointer ) {
		if ( cpPointer - cpDeviceNamePtr ) return NULL; //device witch contain string fd somethere in the text
		strcpy( a_szConvDevName, "hd" );
		/*
		if ( *( cpPointer + 2 ) >= 'a' && *( cpPointer + 2 ) <= 'z' )
			szTmpDriveNum[ 0 ] = ( *( cpPointer + 2 ) - 'a' ) + 0x30;
		*/
		for (a=ds.begin(); a != ds.end(); ++a) {
	   		 if (*( cpPointer + 2 ) > (*a).mid( 7, 1 ).ascii()[0]  ) iMaxDevNum ++;
		}
		szTmpDriveNum[ 0 ] = iMaxDevNum + 0x30;

		strcat( a_szConvDevName, szTmpDriveNum ); //Copy drive number
		
		if ( *( cpPointer + 3 ) ) {
			sprintf( szTmpPartitionNum, ",%d", atoi( cpPointer + 3 ) - 1 );
			strcat( a_szConvDevName, szTmpPartitionNum );//copy partition number					
		}
	}
	return a_szConvDevName;
}

GRUB::GRUB( PartitionView *pv, QWidget *parent, const char *name )
  : QWidget(parent, name), pview(pv), firstTimeMBRWarning( true )
{
	// Now construct the page
  QHBoxLayout *hbox = new QHBoxLayout(this, 0, 6);

  QVBoxLayout *vbox = new QVBoxLayout( 6 );
	hbox->addStretch( 1 );
	hbox->addLayout( vbox, 8 );
	hbox->addStretch( 1 );

	vbox->addSpacing( 10 );

  QLabel *label = new QLabel(tr("Operating systems to load:"), this);
  vbox->addWidget(label);
  vbox->addSpacing(6);

  list = new QListView(this);
  list->setSorting( -1 );
  list->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
  list->setAllColumnsShowFocus(TRUE);
  vbox->addWidget(list);

  list->addColumn(""/*tr("bootable")*/);
  list->addColumn(tr("Partition"));
  list->addColumn(tr("Type"));
  list->addColumn(tr("Label"));

  vbox->addSpacing(18);

  label = new QLabel(tr("Where to install GRUB:"),this);
  vbox->addWidget(label);
  vbox->addSpacing(6);

  QButtonGroup *bg = new QButtonGroup(0);
  connect(bg, SIGNAL(clicked(int)), this, SLOT(targetChanged(int)));

  QHBoxLayout *hb = new QHBoxLayout(vbox);

  mbr = new QRadioButton(tr("&Master boot record (MBR)"), this);
  mbr->setFocusPolicy(QWidget::StrongFocus);
  hb->addWidget(mbr);
  bg->insert(mbr,0);

  label = new QLabel(tr("of"), this);
  hb->addWidget(label);
  disks = new QComboBox(this);
  disks->insertStringList( Fdisk::availableDisks() );
  connect(disks, SIGNAL(activated(const QString &)), this, SLOT(diskChanged(const QString &)));
  hb->addWidget(disks);
  diskinfo = new QLabel(this);
  diskChanged(disks->text(0));
  hb->addWidget(diskinfo);
  hb->addStretch();

  // do we have only ide or only scsi disks?
  int ide=0;
  int scsi=0;
  int special=0;
  QStringList ds( Fdisk::availableDisks() );
  QStringList::Iterator a;
  for (a=ds.begin(); a != ds.end(); ++a) {
    if ((*a).left(8) == "/dev/hd")
      ide++;
    else if ((*a).left(8) == "/dev/sd")
      scsi++;
    else
      special++;
  }

  // if yes, MBR will be on the first disk
  if (special==0 && (ide == 0 || scsi == 0)) {
    label->hide();
    disks->hide();
  }

  target = new QRadioButton(tr("&Target partition"), this);
  target->setFocusPolicy(QWidget::StrongFocus);
  vbox->addWidget(target);
  bg->insert(target,1);

  QHBoxLayout *hh2 = new QHBoxLayout( vbox );

  other = new QRadioButton(tr("&Other:"), this);
  other->setFocusPolicy(QWidget::StrongFocus);
  hh2->addWidget(other);
  bg->insert(other,2);

  otherDest = new QLineEdit(this);
  hh2->addWidget(otherDest);
	hh2->addStretch( 2 );

	vbox->addSpacing( 5 );
  no_grub = new QRadioButton(tr("Skip"), this);
  no_grub->setFocusPolicy(QWidget::StrongFocus);
  vbox->addWidget(no_grub);
  bg->insert(no_grub,3);

	vbox->addSpacing( 10 );

  WriteConfigFileFlag = 0;
/*  LInlineHelp *inlineHelp = new LInlineHelp(this);
  inlineHelp->setDocument( "lilo_overview.qml" );
  hbox->addWidget( inlineHelp );*/

  targetChanged( 0/*1:Maddy*/ );

  useInitrd = FALSE;
}


GRUB::~GRUB()
{
}

int GRUB::GoodOldLILOConfWrite( ) {

	/* Small backup why not? :)*/
	rename("/mnt/etc/lilo.conf", "/mnt/etc/lilo.conf.rpmsave");
	rename("/mnt/etc/silo.conf", "/mnt/etc/silo.conf.rpmsave");


	QDir dir;
	dir.mkdir("/mnt/root/etc");
 	QFile f("/mnt/root/etc/lilo.conf");
    if ( !f.open( IO_WriteOnly ) ) return -1;

	QTextStream conf(&f);
	
	conf << "#" << endl;
    conf << "# /etc/lilo.conf - generated by GBLI" << endl;
    conf << "#" << endl << endl;

	conf << "boot=";
  	if (mbr->isChecked())
		conf << disks->currentText() << endl;
	if (target->isChecked())
    	conf << MountPoints::mountPartition("/") << endl;
    if (other->isChecked())
    	conf << otherDest->text() << endl;

    conf << "map=/boot/map" << endl;
	conf << "install=/boot/boot.b" << endl;
   	conf << "prompt" << endl;
    conf << "timeout=50" << endl;

    conf << "image=/boot/vmlinuz" << endl;
	conf << "\tlabel=" << "Best Linux" << endl;
	conf << "\troot=" << MountPoints::mountPartition("/") << endl;
	if ( useInitrd )
	    conf << "\tinitrd=" << szIinitrdImage << endl;
	//	    if (append) fprintf(f, "\tappend=\"%s\"\n", append);
	conf << "\tread-only" << endl;


    QListViewItem *item = list->firstChild();
    item = item->nextSibling();

	while (item) {

      QCheckListItem *ci = static_cast<QCheckListItem*>(item);
	
      if (ci && ci->isOn()) {
		if ( ( item->text(3) ).contains( "Linux" ) ) {
			    conf << "image=/boot/vmlinuz" << endl;
				conf << "\tlabel=" << item->text(3) << endl;
				conf << "\troot=" << item->text(1) << endl;
		} else {
			conf << "other=" << item->text(1) << endl;
			conf << "\tlabel=" << item->text(3) << endl;
			conf << "\ttable=" << item->text(1) << endl;

       		if ( strcmp( item->text(1), MountPoints::mountPartition("/") ) ) {
			    /* boot off the second drive, so reverse the BIOS maps */
			    conf << "\tmap-drive=0x80" << endl;
			    conf << "\t   to = 0x81" << endl;
		    	conf << "\tmap-drive=0x81" << endl;
			    conf << "\t   to = 0x80" << endl;
			}

		}
      }
      item = item->nextSibling();
    }
	conf << flush;
	f.close();
	return 0;
}

void GRUB::init()
{
  // clear list
  list->clear();

  // add other partitions
  QString system, label;
  int nr=0, linuxnr=0;

  QStringList all = pview->allPartitions();
  QStringList::Iterator a;
  for (a = all.begin(); a != all.end(); ++a) {

    int id = pview->idOfPartition(*a);

    if (id == 0x01 ||	id == 0x04 ||
			id == 0x11 ||	id == 0x14 ||
			id == 0x16 ||	id == 0x24 ||
			id == 0xc1 ||	id == 0xc4 ||
			id == 0xc6 ||	id == 0xc4 ||
			id == 0x06 ||	id == 0x0b ||
			id == 0x0c || 	id == 0x0e
		)
      {
    	if (nr == 0)
	  label = "Windows/DOS";
	else
	  label = QString("Windows/DOS %0").arg(nr);
	nr++;
	
	new GRUBInfoItem(list, FALSE, *a, Fdisk::sysname(id), label);
      }
    else if ( id == 0x83 && (MountPoints::mountPartition("/") != *a) ) {

	if ( linuxnr == 0 )
	  label = "Linux";
	else
	  label = QString("Linux %0").arg( linuxnr );

        new GRUBInfoItem(list, FALSE, *a, Fdisk::sysname(id), label);	
    }/* else if (id != 0x82 && (MountPoints::mountPartition("/") != *a))
      {
	// check for an installed lilo

	int dev = open(*a, O_RDONLY);
	if (dev >= 0)
	  {
	    char buffer[12];
	    if (read(dev, buffer, 12) == 12)
	      {
		if ((strncmp(&buffer[2],"LILO",4) == 0)
		    || (strncmp(&buffer[6],"LILO",4) == 0))
		  {
		    QString label = *a;
		    int pos = label.findRev('/');
		    if (pos>0)
		      label.remove(0,pos+1);
		    label = "lilo-"+label;

		    new GRUBInfoItem(list, FALSE, *a, Fdisk::sysname(id), label);
		  }
	      }
	  }
      }
*/
  }

  // add ourselves
  QString root = MountPoints::mountPartition("/");
  QListViewItem *l = new QListViewItem(list);
  l->setText(1,root);
  l->setText(2,"Linux");
  l->setText(3,"Best Linux");

  // if we do not install onto the first disk, propose to write the MBR
// by Maddy
/*  if (root.left(8) == "/dev/hda" || root.left(8) == "/dev/sda")
    target->setChecked(TRUE);
  else*/
    mbr->setChecked(TRUE);
		targetChanged( 0 );// quick hack

  target->setText(tr("&Target partition (")+root+")");

  // lay out the width of the columns
  list->setColumnWidth(0,20);
  list->setColumnWidth(1,32+list->columnWidth(1));
  list->setColumnWidth(2,32+list->columnWidth(2));
  list->setColumnWidth(3,32+list->columnWidth(3));
}

void GRUB::targetChanged(int index)
{
	if( firstTimeMBRWarning && index ){
		firstTimeMBRWarning = false;
		index = askToSwitchToMBR( index );
		if( index ){
	  	disks->setEnabled( index == 0 );
		  otherDest->setEnabled( index == 2 );
		}
	}else{	
		if( index == 3 )
			index = askToSwitchToMBR( index );
  	disks->setEnabled( index == 0 );
	  otherDest->setEnabled( index == 2 );
	}
}

int GRUB::askToSwitchToMBR( int new_index )
{
		int res = QMessageBox::warning( this, tr("Master Boot Record"),
					tr("NB! If you choose not to install GRUB bootloader to\n"
          "the MBR (Master Boot Record) then it is likely that\n"
          "your computer will not boot Linux after restarting\n"
          "unless you manually configure some other boot loader\n"
          "to support Best Linux.\n"
          "So, it is assumed that you really know what you are doing.\n"
          "Install GRUB bootloader to MBR?"),
					QMessageBox::Yes, QMessageBox::No );
		if( res == QMessageBox::Yes ){
	    mbr->setChecked( true );
			return 0;
		}else{
			return new_index;
		}
}

void GRUB::diskChanged(const QString &disk)
{
  QString device(disk);
  QString desc;

  if (device.left(8) == "/dev/rd/")
    desc = tr("Mylex disk");

  if (device.left(9) == "/dev/ida/")
    desc = tr("Compaq Smart Array");

  if (device.left(7) == "/dev/hd") {
    device.remove(0,5);
    QFile file("/proc/ide/"+device+"/model");
    if (file.open(IO_ReadOnly)) {
      QTextStream ts(&file);
      ts >> desc;
      file.close();
    }
  }

  // TODO: Identify the SCSI disks

  if (!desc.isEmpty())
    diskinfo->setText("("+desc+")");
  else
    diskinfo->setText("");
}

int GRUB::writeConfFile( QString a_QSConfFilePath, bool a_addRunlevel5 ) { //Must be as "/mnt/root/etc/grub.conf"
char szConvDevName[ 255 ];
QFile f( a_QSConfFilePath );
  if (f.open(IO_WriteOnly)) {

//Set write Config Write flag here
  WriteConfigFileFlag = 1;
//~~~~~~~~~~~~~~~~~~~~~~~~
    QTextStream conf(&f);

    conf << tr("# This file has been generated by Graphical Best Linux Installer\n");
    conf << tr("# Options\n");
    conf << "color= light-gray/blue black/light-gray\n";

    //Here get current language and setup specifical language file
   char *lang = getenv("LANG");
    if ( lang && strcmp( lang, "en" ) ){
	if( QFile( QString("/mnt/root/boot/grub/")+ lang +".grub" ).exists() )
	conf << "messages=/boot/grub/" << lang << ".grub\n";
    }
    conf << "default=0\n";
    conf << "timeout=30\n";

    QString QSKernelName;
    QString QSBLPartition = ConvLinuxDev2GRUB( szConvDevName, ( MountPoints::mountPartition("/") ).ascii() );

    if( a_addRunlevel5 ) {
	conf << "\n\ntitle=" << tr("Best Linux graphical mode") << endl;
  	conf << "root=(" <<  QSBLPartition << ")" << endl;
	conf << "kernel=/boot/vmlinuz 5 root=" << MountPoints::mountPartition("/") << endl;
	if ( useInitrd ) conf << "initrd=" << szIinitrdImage << endl;
   }

   conf << "\ntitle=" << tr("Best Linux text mode") << endl;
   conf << "root=(" << QSBLPartition << ")" << endl;
   conf << "kernel=/boot/vmlinuz 3 root=" << MountPoints::mountPartition("/") << endl;
   if ( useInitrd ) conf << "initrd=" << szIinitrdImage << endl;

   conf << "\ntitle=" << tr("Best Linux failsafe mode") << endl;
   conf << "root=(" << QSBLPartition << ")" << endl;
   conf << "kernel=/boot/vmlinuz 1 root=" << MountPoints::mountPartition("/") << endl;
   if ( useInitrd ) conf << "initrd=" << szIinitrdImage << endl;

    QListViewItem *item = list->firstChild();
    item = item->nextSibling();

    while (item) {

      QCheckListItem *ci = static_cast<QCheckListItem*>(item);

      if (ci && ci->isOn()) {
	if ( ( item->text(3) ).contains( "Windows/DOS" ) ) {
			
		QString QSInstPartition = ConvLinuxDev2GRUB( szConvDevName, ( item->text(1) ).ascii() );

   		conf << "\n\ntitle=" << item->text(3) << endl;
		conf << "root=(" << QSInstPartition << ")" << endl;
		conf << "makeactive" << endl;
    	conf << "chainloader +1" << endl;
		if ( strncmp( QSInstPartition, QSBLPartition, 3 ) ) {
            /* boot off the second drive, so reverse the BIOS maps */
	    	conf << "map (hd0) (" << QSBLPartition.left( 3 ) <<  ")" << endl;
			conf << "map (" << QSBLPartition.left( 3 ) << ") (hd0)" <<  endl;
		}
	} else
		if ( ( item->text(3) ).contains( "Linux" ) ){
	     		conf << "\n\ntitle=" << item->text(3) << endl;
			conf << "root=(" << ConvLinuxDev2GRUB( szConvDevName, ( item->text(1) ).ascii() ) << ")" << endl;
			conf << "kernel=/boot/" << "vmlinuz" << " root=" << item->text(1) << endl;

        	} else {
	     	conf << "\n\ntitle=" << item->text(3) << endl;
			conf << "root=(" << ConvLinuxDev2GRUB( szConvDevName, ( item->text(1) ).ascii() ) << ")" << endl;
	}
      }

      item = item->nextSibling();
    }
    conf << flush;
    f.close();
  }
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
return 0;
}
void GRUB::finish( bool a_addRunlevel5 )
{
//GbliDebug( "GRUB::finish" );
//  QFile f("/mnt/root/boot/grub/lilo.conf");//???
  QString GRUBInstallPath;
//  QFile f("/mnt/root/etc/grub.conf");
  QFile cpuinfo("/proc/cpuinfo");
  char szConvDevName[ 255 ], szRootDeviceName[ 255 ];
  char **argv;
  int iTmp;


   argv = (char **)alloca(sizeof(*argv) * 4);
   for ( iTmp = 0; iTmp < 4; iTmp++ ) argv[ iTmp ] = (char *)alloca(1000);

  if (mbr->isChecked())
   GRUBInstallPath = disks->currentText();
  if (target->isChecked())
   GRUBInstallPath = MountPoints::mountPartition("/");
  if (other->isChecked())
   GRUBInstallPath = otherDest->text();

  can_do_smp = FALSE;
  if (cpuinfo.open(IO_ReadOnly)) {
     QTextStream cinfo(&cpuinfo);
     QString line;
     QRegExp r = QRegExp("flags.* apic ");

     line = cinfo.readLine();
     while ( !line.isNull() ) {
      	if (line.contains(r)) {
      	    can_do_smp = TRUE;
      	    break;
      	}
      	line = cinfo.readLine();
     }
     cpuinfo.close();
  }
//	can_do_smp = detectSMP();

//Now save old LILO settings to config file
  GoodOldLILOConfWrite( );
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

 QString  QSKVer = lastVersion( "/mnt/root/boot", "2.2" );
//I don`t now is it better place for that but why not?
    if ( !QSKVer.isEmpty() ) link_boot_files( can_do_smp ?1:0, "/mnt/root/boot", QSKVer.ascii() );
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

 CreateInitrdImage( "/mnt/root/",  QSKVer.ascii() );

 if ( !WriteConfigFileFlag ) writeConfFile( "/mnt/root/etc/grub.conf", a_addRunlevel5 );

  ConvLinuxDev2GRUB( szConvDevName, ( ( MountPoints::mountPartition("/") ).ascii() ) );
  strcpy( szRootDeviceName, szConvDevName );

  if ( !strstr( szConvDevName, "," ) )
  	strcat( szRootDeviceName, ",0" );

  if( !no_grub->isChecked() ){
  	system( "/bin/umount /proc" );
  	system( "/bin/mount -t proc proc /mnt/root/proc" );

  	char cmd[512];
  	sprintf( cmd, "chroot /mnt/root /bin/sh -c \"/sbin/grub --no-curses --batch  <<EOT\nroot=(%s)\ninstall=/boot/grub/stage1 d (%s) /boot/grub/stage2 0x8000 p /etc/grub.conf\nquit\nEOT\n\" > /dev/null",
  			 szRootDeviceName, ConvLinuxDev2GRUB( szConvDevName, GRUBInstallPath ) );
  	system( cmd );

  	system( "/bin/umount /mnt/root/proc" );
  	system( "/bin/mount -t proc proc /proc" );
	}
}

//#ifdef DEVEL_createBootDisk
//It`s no GRUB page work, it must be moved on different class name, but now why not? :)
int GRUB::makeBootdisk(char * prefix, char * kernelVersion) {

//    if (loadModule("loop", DRIVER_OTHER, DRIVER_MINOR_NONE, NULL)) return INST_ERROR;
	
//    winStatus(32, 3, _("Bootdisk"), _("Creating bootdisk..."));
   char cmd[512];

   system( "/bin/umount /proc" );
   system( "/bin/mount -t proc proc /mnt/root/proc" );

   sprintf( cmd, "chroot %s /bin/sh -c \" /sbin/mkbootdisk --noprompt -g --device /dev/fd0 %s\"",  prefix, kernelVersion );
   system( cmd );

   system( "/bin/umount /mnt/root/proc" );
   system( "/bin/mount -t proc proc /proc" );

//    removeModule("loop");

    return 0;
}
#define NOSTUFFHERE -1
#define NOBOOTDISKS 1
#define LINUXNOTFOUND -2
int GRUB::createBootdisk( bool a_addRunlevel5 ) {
int stage;
stage = 1;
//Check if start stuff in right place closed now
if (access("/mnt/root/sbin/mkbootdisk", X_OK) || access("/mnt/root/sbin/mkinitrd", X_OK) ) return NOSTUFFHERE;
if ( !WriteConfigFileFlag ) writeConfFile( "/mnt/root/etc/grub.conf", a_addRunlevel5 );
/**///printf( "Checked mkbootdisk & mkinitrd\n" );
 while (stage < 4) {
	switch (stage) {
		case 1:
			switch( QMessageBox::information( /*( QWidget * )  */this, tr("Bootdisk"),
					tr("A custom bootdisk provides a way of booting into your "
		  			"Linux system without relying on the normal bootloader. "
		  			"This is useful if you don't want to install lilo on your "
		  			"system, another operating system removes lilo, or lilo "
		 			  "doesn't work with your hardware configuration. A custom "
		  			"bootdisk can also be used with the Best Linux rescue image, "
		  			"making it much easier to recover from severe system "
		  			"failures.\n\n"
		  			"Would you like to create a bootdisk for your system?"),
					QMessageBox::Yes, QMessageBox::No, 0 ) ) {
				case QMessageBox::Yes: break;
				case QMessageBox::No: return NOBOOTDISKS; break;
				default: return NOBOOTDISKS; break;
			}
			stage = 2;
			break;
		case 2:
			switch( QMessageBox::warning(  /*( QWidget * )*/this, tr("Bootdisk"),
					tr("Insert a blank floppy in the first drive /dev/fd0."),
					 QMessageBox::Ok, QMessageBox::Cancel, 0 ) ) {
		  		case QMessageBox::Cancel: stage = 1; break;
				case QMessageBox::Ok: stage = 3; break;
				default: stage = 1;
			}
	    	break;
		case 3:
			qApp->setOverrideCursor( Qt::waitCursor );
			QString QSKernelName;
			QSKernelName = lastVersion( "/mnt/root/boot", "2.2" );
			if ( QSKernelName.isEmpty() ) {
				QSKernelName = lastVersion( "/mnt/root/boot", "2.0" );
				if ( QSKernelName.isEmpty() ) return LINUXNOTFOUND;
			}

	   		makeBootdisk( "/mnt/root", (char*)QSKernelName.ascii() );
    		stage++;
			qApp->restoreOverrideCursor();
	    		break;			
	}
 }

return 0;
}
//#endif






































