/***************************************************************************
                                ini_file.cpp
                             -------------------                                         
    begin                : Tue Oct 19 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Anisimov
    author email         : aa@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/
#include "ini_file.h"
/**/ #include <qmessagebox.h>
IniFile::IniFile(){
 iEnumCounter = 0;
}
/********************************************************************
* Function:
*    GetWordN
*
* Arguments:
*		char *a_szSrc -
*		char *a_szDst -
*		char *a_szDet -
*		unsigned short wNum -
*
* Returns:
*		int
*
* Copyright 
*
********************************************************************/

int IniFile::GetWordN(char *a_szSrc, char *a_szDst, char *a_szDet, unsigned short wNum)
{
  unsigned short wI, wCur = 0;
  char cFound;

  for ( ; *a_szSrc; a_szSrc++ )
  {
	cFound = 1;
	for ( wI = 0; a_szDet[wI]; wI++ )
	{
	  if ( *(a_szSrc + wI) != a_szDet[wI] )
	  {
		cFound = 0;
		break;
	  }
	}

	if ( cFound )
	{
	  a_szSrc += wI;
	  wCur++;
	}
	if ( wCur == wNum ) *a_szDst++ = *a_szSrc;
	if ( wCur > wNum ) break;
  }

  a_szDst = 0;
  return wCur >= wNum;
}


/********************************************************************
* Function:
*    Left - extract first left characters from string
*
* Arguments:
*		char *a_szDest - return substring
*		char *a_szSource - source string
*		unsigned long a_dwCount - count character
*
* Returns:
*    int = 0 ( always )
*
* Copyright 
*
********************************************************************/

int IniFile::Left(char *a_szDest, char *a_szSource, unsigned long a_dwCount)
{
  memcpy(a_szDest, a_szSource, a_dwCount);
  a_szDest[a_dwCount] = 0;
  return 0;
}

/********************************************************************
* Function:
*    Right - extract first right characters from string
*
* Arguments:
*		char *a_szDest - return substring
*		char *a_szSource - source string
*		unsigned long a_dwCount - count character
*
* Returns:
*    int = 0 ( always )
*
* Copyright 
*
********************************************************************/

int IniFile::Right(char *a_szDest, char *a_szSource, unsigned long a_dwCount)
{
  memcpy(a_szDest, a_szSource + ( strlen(a_szSource) - a_dwCount ), a_dwCount);
  a_szDest[a_dwCount] = 0;
  return 0;
}

/********************************************************************
* Function:
*     Mid - extract substring from string
*
* Arguments:
*		char *a_szDest - return string
*		char *a_szSource - source string
*		unsigned long a_dwPosition - number of first character
*       unsigned long a_dwCount - count of characters
*
* Returns:
*     int = 0 ( always )
*
* Copyright 
*
********************************************************************/

int IniFile::Mid(char *a_szDest, char *a_szSource, unsigned long a_dwPosition,
		unsigned long a_dwCount)
{
  memcpy(a_szDest, a_szSource + a_dwPosition - 1, a_dwCount);
  a_szDest[a_dwCount] = 0;
  return 0;
}


/********************************************************************
* Function:
*		ProfileStr - Read value of key from section in ini file.
*
* Arguments:
*		char *a_szFileName	- profile name
*		char *a_szSection 	- section name
*		char *a_szKey 		- key name
*		char *a_szDefault 	- default value of key
*		char *a_szRetValue 	- return key value
*
* Returns:
*     int - if error occured 	- 0
*           other 				- length of key value
*
* Comments:
*     Length of line in profile must be < 256 bytes
*     Lower case and upper case chars is differential
*
* Copyright  Alexey Ptashkin
*
********************************************************************/

int IniFile::ProfileStr( const char *a_szFileName, const char *a_szSection,
			   const char *a_szKey, char *a_szDefault, char *a_szRetValue)
{
  char 	*szSection, *szKey;
  FILE	*iFd;
  char sBuffer[ 256 ];
  /*char sBuffer2[ 256 ];*/
  int iFlag = 0;

  iFd = fopen(a_szFileName, "r");

  if ( !iFd )
  {
	strcpy( a_szRetValue, a_szDefault );
	return 0;
  }

  szSection = (char *) malloc( strlen( a_szSection ) + 4 );

  if ( !szSection )
  {
	fclose( iFd );
	strcpy( a_szRetValue, a_szDefault );
	return 0;
  }

  szKey = (char *) malloc( strlen( a_szKey ) + 2 );

  if ( szKey == NULL )
  {
	fclose( iFd );
	free( szSection );
	strcpy( a_szRetValue, a_szDefault );
	return 0;
  }

  szSection[ 0 ] = '[';
  szSection[ 1 ] = 0;
/*  strcat( szSection, strupper( a_szSection ) );*/
  strcat( szSection, a_szSection );
  strcat( szSection, "]" );
  szKey[ 0 ] = 0;
/*  strcat( szKey, strupper( a_szKey ) );*/
  strcat( szKey, a_szKey );
  strcat( szKey, "=" );

  while ( fgets( sBuffer, 256, iFd ) )
  {

  /*	strcpy( sBuffer2, sBuffer );
	strupper( sBuffer2 );*/
	if( iFlag == 0 && !strncmp( sBuffer, szSection, strlen( szSection ) ) )
	  iFlag = 1;
	else if( iFlag == 1 )
	{
	  //printf( "Read from file %s\n", sBuffer );
	  if( sBuffer[ 0 ] == '[' )
	  {
		fclose( iFd );
		free( szSection );
		free( szKey );
		strcpy( a_szRetValue, a_szDefault );
		return 0;
	  }
	  if( !strncmp( sBuffer, szKey, strlen( szKey ) ) )
	  {
		iFlag = 2;
		break;
	  }
	}
  }

  fclose( iFd );
  free( szSection );

  if( iFlag == 0 )
  {
	 strcpy( a_szRetValue, a_szDefault );
	 return 0;
  }
  else
  {	
	 strcpy( a_szRetValue, sBuffer + strlen( szKey ) );
	 if( *( a_szRetValue + strlen( a_szRetValue ) - 1 ) == '\n' )
	   *( a_szRetValue + strlen( a_szRetValue ) - 1 ) = 0;
	 if( *( a_szRetValue + strlen( a_szRetValue ) - 1 ) == '\r' )
	   *( a_szRetValue + strlen( a_szRetValue ) - 1 ) = 0;

  }
  free( szKey );
  return strlen( a_szRetValue );
}

 /********************************************************************
* Function:
*		ProfileEnum - Read key and value of from section in ini file.
*
* Arguments:
*		char *a_szFileName	- profile name
*		char *a_szSection 	- section name
*		char *a_szKey 		- return key name
*		char *a_szValue 	- return key value
*
* Returns:
*     int - if error occured file or memory allocation 	- 0
	    no more records found - -1
*           other 				- length of key value
*
* Comments:
*     Length of line in profile must be < 256 bytes
*     Lower case and upper case chars is differential
*
* Copyright  Anton Anisimov
*
********************************************************************/
int IniFile::ProfileEnum( const char *a_szFileName, const char *a_szSection, char *a_szKey, char *a_szValue )
{
  char 	*szSection;
  FILE	*iFd;
  char sBuffer[ 256 ];
  unsigned char bEqualPos = 0;
  int iFlag = 0;
  int iCurCounter = 0;

  if ( strcmp( szSectionName, a_szSection ) ) iEnumCounter = 0;

  iFd = fopen(a_szFileName, "r");

  if ( !iFd )
  {
	return -1;
  }

  szSection = (char *) malloc( strlen( a_szSection ) + 4 );

  if ( !szSection )
  {
	fclose( iFd );
	return -1;
  }

  szSection[ 0 ] = '[';
  szSection[ 1 ] = 0;
  strcat( szSection, a_szSection );
  strcat( szSection, "]" );
  strcpy( szSectionName, a_szSection );

  while ( fgets( sBuffer, 256, iFd ) )
  {
	if( iFlag == 0 && !strncmp( sBuffer, szSection, strlen( szSection ) ) )
	  iFlag = 1;
	else if( iFlag == 1 )
	{
	  if( sBuffer[ 0 ] == ';' ) {
		//iEnumCounter++;
		iCurCounter++;
		szOldEnumKeyStr[ 0 ] = 0;
		continue;
	  }
	  if( sBuffer[ 0 ] == '[' )
	  {
		fclose( iFd );
		free( szSection );
		iEnumCounter = 0;
		return -1;
	  } else {
		if ( strchr( sBuffer, '=' ) == NULL ) {
			iCurCounter++;
			szOldEnumKeyStr[ 0 ] = 0;
			continue;
		}
		bEqualPos = ( strchr( sBuffer, '=' ) - sBuffer );
		if ( strncmp( szOldEnumKeyStr, sBuffer, bEqualPos ) && iCurCounter >= iEnumCounter ) {
			strncpy( szOldEnumKeyStr, sBuffer, bEqualPos );
			szOldEnumKeyStr[ bEqualPos ] = 0;
			iEnumCounter++;
			iFlag = 2;
			break;
		} else iCurCounter++;
	  }
	
	}
  }

  fclose( iFd );
  free( szSection );

  if( iFlag == 0 )
  {
	 return -1;
  }
  if ( iFlag == 1 ) {
	iEnumCounter = 0;
	return -1;
  }
  if ( iFlag == 2 )
  {
	strcpy( a_szKey, szOldEnumKeyStr );
	strcpy( a_szValue, sBuffer + bEqualPos + 1 );

	 if( *( a_szValue + strlen( a_szValue ) - 1 ) == '\n' )
	   *( a_szValue + strlen( a_szValue ) - 1 ) = 0;
	 if( *( a_szValue + strlen( a_szValue ) - 1 ) == '\r' )
	   *( a_szValue + strlen( a_szValue ) - 1 ) = 0;

  }

  return strlen( a_szKey );
}
/********************************************************************
* Function:
*		ProfileStrEnum - Read data from section in ini file.
*
* Arguments:
*		char *a_szFileName	- profile name
*		char *a_szSection 	- section name
*		char *a_szData 		- return data record in section
*
* Returns:
*     int - if error occured file or memory allocation 	- 0
	    no more records found - -1
*           other 				- length of key value
*
* Comments:
*     Length of line in profile must be < 256 bytes
*     Lower case and upper case chars is differential
*
* Copyright  Anton Anisimov
*
********************************************************************/
int IniFile::ProfileStrEnum( const char *a_szFileName, const char *a_szSection, char *a_szData )
{
  char 	*szSection;
  FILE	*iFd;
  char sBuffer[ 256 ];
//  unsigned char bEqualPos;
  int iFlag = 0;
  int iCurCounter = 0;

  if ( strcmp( szSectionName, a_szSection ) ) iEnumCounter = 0;

  iFd = fopen(a_szFileName, "r");

  if ( !iFd )
  {
	return 0;
  }

  szSection = (char *) malloc( strlen( a_szSection ) + 4 );

  if ( !szSection )
  {
	fclose( iFd );
	return 0;
  }

  szSection[ 0 ] = '[';
  szSection[ 1 ] = 0;
  strcat( szSection, a_szSection );
  strcat( szSection, "]" );
  strcpy( szSectionName, a_szSection );

  while ( fgets( sBuffer, 256, iFd ) )
  {
	if( iFlag == 0 && !strncmp( sBuffer, szSection, strlen( szSection ) ) )
	  iFlag = 1;
	else if( iFlag == 1 )
	{
	  if( sBuffer[ 0 ] == '[' )
	  {
		fclose( iFd );
		free( szSection );
		iEnumCounter = 0;
		return -1;
	  } else {
//		bEqualPos = ( strchr( sBuffer, '=' ) - sBuffer );
		if ( strcmp( szOldEnumKeyStr, sBuffer ) && iCurCounter >= iEnumCounter ) {
			strcpy( szOldEnumKeyStr, sBuffer );
			iEnumCounter++;
			iFlag = 2;
			break;
		} else iCurCounter++;
	  }
	
	}
  }

  fclose( iFd );
  free( szSection );

  if( iFlag == 0 )
  {
	 return 0;
  }
  if ( iFlag == 1 ) {
	iEnumCounter = 0;
	return -1;
  }
  if ( iFlag == 2 )
  {
	strcpy( a_szData, sBuffer );

	 if( *( a_szData + strlen( a_szData ) - 1 ) == '\n' )
	   *( a_szData + strlen( a_szData ) - 1 ) = 0;
	 if( *( a_szData + strlen( a_szData ) - 1 ) == '\r' )
	   *( a_szData + strlen( a_szData ) - 1 ) = 0;

  }

  return strlen( a_szData );
}
/********************************************************************
* Function:
*    SetProfileStr - set value of key a_szKey from section a_szSection
*
* Arguments:
*		char *a_szFileName - profile name
*		char *a_szSection - section name
*		char *a_szKey - key name
*       char *asValue - new key value
*
* Returns:
*    int = -1 if can't open file a_szFileNmae
*          -2 if can't open tmp file
*          -3 if not enough memory
*           0 if was add section a_szSection
*           1 if was add key a_szKey
*           2 if was change value of key a_szKey
*
* Copyright 
*
********************************************************************/

int IniFile::SetProfileString( const char *a_szFileName, char *a_szSection,
					  char *a_szKey, char * asValue )
{
   char	/**sRetValue,*/ *sSection, *sKey;
   char	*sOneString;
   FILE	*iFd, *iFdNew;
   int	i = 0;

   iFd = fopen( a_szFileName, "r+" );
   if( !iFd ) return -1;

   iFdNew = fopen( "tmp$$$", "w" );
   if( !iFdNew ) return -2;

   sOneString = ( char * ) malloc( 256 ) ;
   sSection = ( char * ) malloc(  strlen( a_szSection ) + 4 ) ;

   if ( !sOneString || !sSection )
   {
	 if( sOneString ) free( sOneString );
	 if( sSection ) free( sSection );
	 fclose( iFd );
	 fclose( iFdNew );
	 remove( "tmp$$$" );
	 return -3;
   }

   sSection[ 0 ] = '[';
   sSection[ 1 ] = 	0 ;

   strcat( sSection,  a_szSection );
   strcat( sSection,  "]" );

   sKey = ( char * ) malloc( strlen( a_szKey ) + 3 + strlen( asValue ) );

   if ( !sKey )
   {
	 free( sSection );
	 free( sOneString );
	 fclose( iFd );
	 fclose( iFdNew );
	 remove( "tmp$$$" );
	 return -3;
   }

   strcpy( sKey, a_szKey );
   strcat( sKey,  "=" );

   while( fgets( sOneString, 256,  iFd ) )
   {
	  if( !i )
	  {
		if( !strncmp( sOneString, sSection, strlen( sSection ) ) ) i = 1;
	  }
	  else if( i == 1 )
	  {
		if( !strncmp( sOneString, "[", 1 )  )
		{
			strcat( sKey, asValue );
			fputs( sKey, iFdNew	);
			fputs( "\n", iFdNew	);
			i = 2;
		}
		if( !strncmp( sOneString, sKey, strlen( sKey ) ) )
		{
			strcat( sKey, asValue );
			fputs( sKey, iFdNew	);
			fputs( "\n", iFdNew	);
			i = 3;
			continue;
		}
	  }
	  fputs( sOneString, iFdNew );
   }

   if(i == 1 )
   {
	  strcat( sKey, asValue );
	  fputs( sKey, iFdNew );
	  fputs( "\n", iFdNew );
   }
   fclose( iFdNew );

   if( !i )
   {
	 fseek( iFd, 0L, SEEK_END );
	 fputs( "\n", iFd );
	 fputs( sSection, iFd );
	 strcat( sKey, asValue );

	 fputs( "\n", iFd );
	 fputs( sKey, iFd );
	 fclose( iFd );
	 remove( "tmp$$$" );
   }
   else
   {
	 remove( a_szFileName );
	 rename( "tmp$$$", a_szFileName );
   }
   free( sSection );
   free( sOneString );
   free( sKey );

   return i;
}



















