/***************************************************************************
                               letcfiles.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "letcfiles.h"

#include "lmouse.h"
#include "lvideomodeselector.h"
#include "lvideocard.h"
#include "lmonitor.h"
#include "lkeyboard.h"
#include "ltemplate.h"

#include <math.h>
#include <time.h>

#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstrlist.h>

#define __USE_XOPEN
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <stdlib.h>
#include <stdio.h>
#include "config.h"
#include "lnetwork.h"
#include "llanguage.h"
#include "sysconfigfile.h"

LEtcFiles::LEtcFiles( const char * directory )
{
    dir = directory;
		gettimeofday( &time1, NULL );
}


QString today( int offset = 0 )
{
  QString r;
  QTextStream result(r, IO_WriteOnly);
  result << (int)( time(0L)/86400 + offset );
  return r;
}


static char mksalt( int seed )
{
    int num = seed % 64;

    if (num < 26)
			return 'a' + num;
    else if (num < 52)
			return 'A' + (num - 26);
    else if (num < 62)
			return '0' + (num - 52);
    else if (num == 63)
			return '.';
    else
			return '/';
}

void LEtcFiles::writePasswdStuff(
					const QString & userName,
					const QString & userL,
          const QString & userP,
				  const QString & rootPassword,
				  const QString & homeParent,
					const QString & lang )
{
  struct timeval time2;
  char salt[3];
  char cmd[200];
	int pid;

	gettimeofday( &time2, NULL );

	salt[0] = mksalt( time1.tv_usec );
	salt[1] = mksalt( time2.tv_usec );
	salt[2] = '\0';

	enableShadow();

  sprintf( cmd, "/usr/sbin/groupadd -g 500 %s", userL.ascii() );
  if( !(pid = fork()) ) {
		chroot( NEWROOT );
		chdir("/");

		exit( system(cmd) );
  }
printf( "writePasswdStuff1 pid: %d\n", pid );
  waitpid(pid, NULL, 0);

  sprintf( cmd, "/usr/sbin/useradd -c \'%s\' -m -n -s /bin/bash -g 500 %s", userName.ascii(), userL.ascii() );
  if( !(pid = fork()) ) {
		chroot( NEWROOT );
		chdir("/");

		exit( system(cmd) );
  }
printf( "writePasswdStuff2 pid: %d\n", pid );
  waitpid(pid, NULL, 0);

  QFile shadowFile( dir + "/shadow" );
  QFile newShadowFile( dir + "/shadow.new" );
	if( shadowFile.open( IO_ReadOnly ) && newShadowFile.open( IO_WriteOnly ) ){
  	::fchmod( shadowFile.handle(), 0600 );
  	QTextStream shadow( &shadowFile );
  	QTextStream newShadow( &newShadowFile );
		QString temp;
		while( (temp = shadow.readLine()).length() ){
			if( temp.left( 5 ) == "root:" ){
		  	newShadow << "root:" << ::crypt( rootPassword, salt ) << ":" << today() << ":0:99999:7:-1:-1:\n";
			}else if( temp.left( userL.length()+1 ) == (userL + ":") ){
				gettimeofday( &time2, NULL );
				salt[1] = mksalt( time2.tv_usec + 375 );
		    newShadow << userL << ":" << ::crypt( userP, salt )
	  		 << ":" << today() << ":0:99999:7:-1:" << today( 45 ) << ":\n";
			}else
				newShadow << temp << endl;
		}
		shadowFile.close();
		newShadowFile.close();
		::sync();

    ::unlink( (dir + "/shadow").ascii() );
		::rename( (dir + "/shadow.new").ascii(), (dir + "/shadow").ascii() );
		::sync();
	}
	
  int numLanguages = sizeof(languages) / sizeof(struct langInfo);
	char* short_lang = "C";
	char* charset = "iso-8859-1";
  for( int i = 0; i < numLanguages; i++ )
    if( !strcasecmp( languages[i].lang, lang ) ){
			short_lang = languages[i].key;
			charset = languages[i].charset;
			break;
		}

	if( strcmp( short_lang, "C" ) && strcmp( short_lang, "en" ) ){
  	char buf[128];
  	sprintf( buf, "/%s/.kderc", userL.ascii() );
  	ConfigFile config = ConfigFile( homeParent.ascii(), buf );
   	FILE *fp = config.open( "a" );
    if( fp ){
  		fprintf( fp, "[Locale]\nLanguage=%s:C:C\n", short_lang );
  		fclose( fp );
  	}

  	char databuf[2048];
  	QFile file( homeParent + QString( buf ) );
  	memset( databuf, 0, sizeof( databuf ) );
  	if( file.open( IO_ReadOnly ) ){
  		file.readBlock( databuf, sizeof( databuf ) );
  		file.close();
  	}

  	QString configData;
  	configData = QString::fromLatin1( databuf );
  	substituteCharset( configData, QString::fromLatin1( languages[0].charset ), QString::fromLatin1( charset ) );
  	if( file.open( IO_WriteOnly ) ){
  		file.writeBlock( configData.ascii(), configData.length() );
  		file.close();
  	}

  	// .kderc for root seems to be quite clean
    SysConfigFile sysconfig( NEWROOT, "/root/.kderc" );
    sysconfig.load();
  	sprintf( buf, "%s:C:C", short_lang );
    sysconfig.set( "Language", buf, false );
    sysconfig.save();

  	// It's for login screen
  	fp = fopen( (QString( NEWROOT ) + "/usr/share/config/kdmrc").ascii(), "a" );
  	if( fp ){
  		fprintf( fp, "[Locale]\nLanguage=%s\n", short_lang );
  		fclose( fp );
  	}
  	file.setName( (QString( NEWROOT ) + "/usr/share/config/kdmrc") );
  	memset( databuf, 0, sizeof( databuf ) );
  	if( file.open( IO_ReadOnly ) ){
  		file.readBlock( databuf, sizeof( databuf ) );
  		file.close();
  	}
  	configData = QString::fromLatin1( databuf );
  	substituteCharset( configData, QString::fromLatin1( languages[0].charset ), QString::fromLatin1( charset ) );
  	if( file.open( IO_WriteOnly ) ){
  		file.writeBlock( configData.ascii(), configData.length() );
  		file.close();
  	}

/*		// Corrections for kvt (terminal)
  	if( !strcmp( short_lang, "ru" ) || !strcmp( short_lang, "uk_UA" ) ){
  		system( ("cp /usr/share/config/kvtrc "+ QString( NEWROOT ) +"/root/.kde/share/config/").ascii() );
      SysConfigFile sysconfig1( NEWROOT, "/root/.kde/share/config/kvtrc" );
      sysconfig1.load();
      sysconfig1.set( "defaultFont",	"courier,12,5,koi8-r,50,0", false );
      sysconfig1.save();
  		system( ("cp /usr/share/config/kvtrc "+ QString( NEWROOT ) +"/"+ userL +"/.kde/share/config/").ascii() );
      SysConfigFile sysconfig2( NEWROOT, "/"+ userL +"/.kde/share/config/kvtrc" );
      sysconfig2.load();
      sysconfig2.set( "defaultFont",	"courier,12,5,koi8-r,50,0", false );
      sysconfig2.save();
  	}

  	// Let's do some little nice hacks
  	if( !strcmp( short_lang, "ru" ) || !strcmp( short_lang, "uk_UA" ) ||
  			!strcmp( short_lang, "fi" ) || !strcmp( short_lang, "se" ) ){
  		if( QFileInfo( "/usr/share/config/kikbdrc" ).exists() ){
  			system( ("cp /usr/share/config/kikbdrc "+ QString( NEWROOT ) +"/"+ userL +"/.kde/share/config/").ascii() );
  	    SysConfigFile sysconfig3( NEWROOT, "/"+ userL +"/.kde/share/config/kikbdrc" );
    	  sysconfig3.load();
  			if( !strcmp( short_lang, "ru" ) )
  				sysconfig3.set( "Map1", "rum", false );
  			else if( !strcmp( short_lang, "uk_UA" ) )
  				sysconfig3.set( "Map1", "uam", false );	
  			else
  				sysconfig3.set( "Map1", short_lang, false );
  			sysconfig3.set( "AutoStart", "true", false );
  	    sysconfig3.save();
      	file.setName( QString( NEWROOT ) +"/"+ userL +"/.kde/share/config/kikbdrc" );
      	memset( databuf, 0, sizeof( databuf ) );
      	if( file.open( IO_ReadOnly ) ){
      		file.readBlock( databuf, sizeof( databuf ) );
      		file.close();
      	}
      	configData = QString::fromLatin1( databuf );
      	substituteCharset( configData, QString::fromLatin1( languages[0].charset ), QString::fromLatin1( charset ) );
      	if( file.open( IO_WriteOnly ) ){
      		file.writeBlock( configData.ascii(), configData.length() );
      		file.close();
      	}
  		}
  	}*/
	}
	::sync();
}

void LEtcFiles::substituteCharset( QString &a_str, const QString &oldOne, const QString &newOne )
{
	int pos = 0;
	int len = oldOne.length();
	int count = 20000;
	while( (pos = a_str.find( oldOne, pos )) >= 0 && count-- > 0 ){
		a_str.remove( pos, len );
		a_str.insert( pos, newOne );
		pos++;
	}
}

void LEtcFiles::enableShadow()
{
	int pid = fork();
	if( !pid ){
		ConfigFile config( dir, "/sysconfig/network" );
  	FILE *fp = config.open( "a" );
    if( fp )
			fclose( fp );

		chroot( NEWROOT );
		exit( execl( "/usr/sbin/authconfig", "/usr/sbin/authconfig",
			"--kickstart", "--nostart", "--enablemd5", "--useshadow", 0 ) );
	}
	::waitpid( pid, NULL, 0 );
	pid = 0;
}

void LEtcFiles::writeXConfig( LVideoCard * card,
			      LMonitor * monitor,
			      LVideoModeSelector * mode,
			      LMouse * mouse,
			      LKeyboard * keyboard,
			      const QString & mouseLinkName )
{
  if ( !card || !monitor || !mode || !mouse || !keyboard )
    return;

  QString tpl     ;
  if (card->defaultVGA16())
    tpl = "./templates/XF86Config.VGA16";
  else
    tpl = "./templates/XF86Config";

  LTemplate xconfig(tpl);
  QString value;

  xconfig.addToken("KEYBOARD_MODEL", keyboard->model());
  xconfig.addToken("KEYBOARD_LAYOUT", keyboard->layout());
  xconfig.addToken("MOUSE_NAME", mouse->getMouseName());
  xconfig.addToken("MOUSE_LINK", mouseLinkName);
  xconfig.addToken("MONITOR_VENDOR", monitor->vendorName());
  xconfig.addToken("MONITOR_MODEL", monitor->modelName());
  xconfig.addToken("MONITOR_HSYNC", monitor->horizSync());
  xconfig.addToken("MONITOR_REFRESH", monitor->vertRefresh());
  xconfig.addToken("MODE_NAME", mode->name());
  xconfig.addToken("MODE_LINE", mode->line());
  xconfig.addToken("CARD_CLOCKLINE", card->clockLine());
  value.setNum( (int)(card->memory()/1024) );
  xconfig.addToken("CARD_MEMORY", value);
  value.setNum(mode->defaultColorDepth());  
  xconfig.addToken("COLOR_DEPTH", value);

  if (mode->size() != mode->virtualResolution()){
      QString virt = "        Virtual %0 %1";
      xconfig.addToken("VIRTUAL_SIZE", virt.arg((mode->virtualResolution()).width()).arg((mode->virtualResolution()).height()));
  }else
    xconfig.addToken("VIRTUAL_SIZE", "");

  if (mouse->emulate3buttons())
    xconfig.addToken("EMULATE3RD", "Emulate3Buttons");
  else
    xconfig.addToken("EMULATE3RD", "");    

  if (mouse->hasWheel())
    xconfig.addToken("WHEEL", "ZAxisMapping 4 5");
  else
    xconfig.addToken("WHEEL", "");

  xconfig.write( dir +"/X11/XF86Config" );

/*  // set the mouse symlink
  ::unlink(directory()+"/dev/mouse");
  ::symlink(mouse->portName(), directory()+"/dev/mouse");

  // link the X-Server
  ::unlink(directory()+"/"+XSERVER_PATH+"/X");
  ::symlink(card->server(), directory()+"/"+XSERVER_PATH+"/X");*/
}

void
LEtcFiles::writeGpmConfig(LMouse *mouse)
{
  if (!mouse)
    return;

  QFile	gpmFile( dir + "/sysconfig/mouse" );
  if( !gpmFile.open(IO_WriteOnly|IO_Append) )
    return;

  QTextStream	gpm(&gpmFile);
  gpm << "MOUSETYPE=\"" << mouse->getGPMName() << "\"\n";
  gpm << "XMOUSETYPE=\"" << mouse->getMouseName() << "\"\n";
  gpm << "FULLNAME=\"" << mouse->getMouseName() << "\"\n";
  gpm << "XEMU3=\""     << (mouse->emulate3buttons() ? "yes" : "no") << "\"\n";
	gpmFile.close();
}

void LEtcFiles::writeProfile( const char *a_lang )
{
	ConfigFile config( dir, "/profile" );
  FILE *fp = config.open( "a" );
  if( fp ){


		fclose( fp );
	}else
		printf( (const char*)"Couldn't open %s/profile for writing!\n", (const char*)dir.ascii() );
}

#include "llanguage.h"

void LEtcFiles::writeConfig( const char * a_lang )
{
	unsigned int i18nSize = QFileInfo( QString( dir ) + "/sysconfig/i18n" ).size();

	ConfigFile config( dir, "/sysconfig/i18n" );
  FILE *fp = config.open( "a" );

	if( fp ){
    int numLanguages = sizeof(languages) / sizeof(struct langInfo);
  	int i = 0;
    for( ; i < numLanguages; i++ )
  		if( !strcasecmp( a_lang, languages[i].lang ) )
  			break;

		if( i18nSize )
			fprintf( fp, "\n" );
  	fprintf( fp, "LANG=%s\n", languages[ i ].key );
  	fprintf( fp, "LINGUAS=%s\n", languages[ i ].key );
  	fprintf( fp, "LC_ALL=%s\n", languages[ i ].lc_all );
  	fprintf( fp, "LANGUAGE=%s\n", languages[ i ].lc_all );
  	fprintf( fp, "LOCALE=%s\n", languages[ i ].lc_all );
		if( strlen( languages[ i ].font ) )
	  	fprintf( fp, "SYSFONT=%s\n", languages[ i ].font );
		if( strlen( languages[ i ].map ) )
	  	fprintf( fp, "SYSFONTACM=%s\n", languages[ i ].map );

  	fclose( fp );
	}else
		printf( (const char*)"Couldn't open %s/sysconfi/i18n for writing!\n", (const char*)dir.ascii() );

	config = ConfigFile( dir, "/sysconfig/desktop" );
  fp = config.open( "a" );
	if( fp ){
		fprintf( fp, "KDE" );
		fclose( fp );
	}

/*	config = ConfigFile( dir, "/bashrc" );
  fp = config.open( "a" );
	if( fp ){
    fprintf( fp, "alias l=\"ls -lAp --color=auto\"\n" );
    fprintf( fp, "alias ls=\"ls -p --color=auto\"\n" );
		fprintf( fp, "alias less=\"less -r\"\n" );
		fprintf( fp, "alias c=\"cd ..\"\n" );
		fprintf( fp, "alias h=\"history 25|more\"\n" );
		fprintf( fp, "alias k=\"kill\"\n" );
		fprintf( fp, "alias m=\"more\"\n" );
		fprintf( fp, "alias z=\"zmore\"\n" );
		fprintf( fp, "alias grep=\"grep -n\"\n" );
		fprintf( fp, "alias pico=\"pico -w -b -z\"\n" );
		fclose( fp );
	}*/

	config = ConfigFile( dir, "/conf.modules" );
  fp = config.open( "a" );
	if( fp ){
		fprintf( fp, "alias parport_lowlevel parport_pc\n" );
    fprintf( fp, "pre-install pcmcia_core /etc/rc.d/init.d/pcmcia start\n" );
		fclose( fp );
	}

	// may be run kudzu here?

}









































