/***************************************************************************
																	lmouse.cpp
                             -------------------
    begin                : Fri Oct 29 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "lmouse.h"

#include <qlayout.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qapplication.h>
#include <qsocketnotifier.h>
#include <qcombobox.h>
#include <stdio.h>
#include <qfileinfo.h>

#include "linlinehelp.h"

// xf86misc.h needs this
#include <X11/Xlib.h>
// for the mouse shit
#include <X11/extensions/xf86misc.h>

// ioctl
#include <sys/ioctl.h>

// fcntl
#include <fcntl.h>
#include <unistd.h>

// kill
#include <signal.h>
// elmer 1106
#include <iostream.h>

/* XPM */
static const char *mouse_x[] = {
/* width height ncolors chars_per_pixel */
"16 16 3 1",
/* colors */
". c None",
"  c #000000",
"M c #FFFFFF",
/* pixels */
"MM..............",
"M M.............",
"M  M............",
"M   M...........",
"M    M..........",
"M     M.........",
"M      M........",
"M       M.......",
"M        M......",
"M     MMMMM...M.",
"M  M  M..M M.M M",
"M M.M  M..M M M.",
"MM..M  M...M M..",
".....M  M.M M M.",
".....M  MM M.M M",
"......MM..M...M."
};

LMouseView::LMouseView( QWidget *parent, const char *name)
    : QWidget( parent, name )
{
    state = NoButton;
    wheelOffset = 0;
    hasWheel = FALSE;
    setMaximumWidth( 200 );
}

void LMouseView::paintState( bool onlyWheel)
{
    int bw = width()/3;
    int bh = height()*2/5;

    QRect left( 0, 0, bw, bh );
    QRect mid( bw, 0, bw, bh );
    QRect right( 2*bw, 0, bw, bh );
    QRect wheel( width()/2 - bw/4, bh/4, bw/2, bh/2 );
    QRect body( 0, bh, 3*bw, height()-bh-1 );

    QPainter p(this);

    if (!onlyWheel) {
			p.setBrush( state & LeftButton ? black : white );
			p.drawRect( left );

			p.setBrush( state & MidButton ? black : white );
			p.drawRect( mid );

			p.setBrush( state & RightButton ? black : white );
			p.drawRect( right);

			p.setBrush( white );
			p.drawRect( body );

			p.drawText( body, AlignCenter | WordBreak, tr("Test mouse here!") );
    }

    if (hasWheel) {
			p.drawRect( wheel.x()-1, wheel.y()-1, wheel.width()+2, wheel.height()+2);
			p.fillRect( wheel.x(), wheel.y(), wheel.width(), wheel.height(), white );
			p.setBrush( HorPattern );
			p.setBrushOrigin( 0, wheelOffset );
			p.drawRect( wheel );
    }
}

void LMouseView::paintEvent( QPaintEvent * )
{
    paintState();
}

bool LMouseView::eventFilter( QObject *, QEvent * e)
{

    if (e->type() == QEvent::MouseButtonPress || e->type() == QEvent::MouseButtonRelease) {
			state = ( (QMouseEvent*) e)->stateAfter();
			repaint (FALSE );
    }
    else if (e->type() == QEvent::Wheel ) {
			if ( ((QWheelEvent*) e)->delta() < 0 )
	  	  wheelOffset++;
			else
	  	  wheelOffset--;
			paintState ( TRUE );
    }
    return FALSE;
}

void LMouseView::setWheel( bool b )
{
    hasWheel = b;
    resetState();
}


void LMouseView::resetState()
{
    state = (ButtonState) 0 ;
    repaint( TRUE );
}

void LMouseView::enterEvent( QEvent * )
{
    // do nothing on enter. Remember that we get an enter event when a
    // combobox (or any other popup) close. You mouseMoveEvent instead
    // (see below).
}

void LMouseView::mouseMoveEvent( QMouseEvent * )
{
    setFocus();
}

/* ################################################################ */

LMouseObserver::LMouseObserver( QObject * parent, const char * name )
    : QObject( parent, name)
{
		rn = NULL;
    hasDetected = false;
		bool ps2 = true;
		QFileInfo mf( "/dev/mouse" );
		if( mf.exists() ){
			char mlink[128];
			memset( mlink, 0, sizeof( mlink ) );
			readlink( "/dev/mouse", mlink, 128 );
printf( "Mouse device: %s\n", mlink );
			if( strcmp( mlink, "psaux" ) )
				ps2 = false;

		}

    mouse = PS2;
    port = PSAux;
    protocol = MTYPE_PS_2;
		if( !ps2 ){
			mouse = MS;
			port = S0;
			protocol = MTYPE_MICROSOFT;
		}
    emulate3rd = false;
		allowCheck = false;
		blProbe = false;
		blProbing = false;
}

void LMouseObserver::runProbing()
{
	if( blProbing ) return;
	blProbing = true;
	// elmer 1106
	cout << "running mouseprobe" << endl;
#ifndef GBLI_DEVEL
//#ifdef abcdef
	    int fd[2];
	    ::pipe( fd );
  	  QApplication::flushX();
  	  pid = ::fork();
    	if (pid == 0) {
				::close( fd[0] );
	
			 	if (fd[1] != STDOUT_FILENO) {
 	  		  dup2( fd[1], STDOUT_FILENO);
 	    		::close( fd[1] );
			 	}
				execl( "./mouse/mdetect", "./mouse/mdetect", 0 );
				exit(1);
	    }
printf( "LMouseObserver::runProbing pid: %d\n", pid );
  	  ::close( fd[1] );

    	int s;
	    if ( (s = fcntl( fd[0], F_GETFL, 0 ) ) >= 0 ) {
				s |= FNDELAY;
				fcntl( fd[0], F_SETFL, s );
	    }

  	  rn = new QSocketNotifier( fd[0], QSocketNotifier::Read, this);
			blProbe = true;
    	connect( rn, SIGNAL(activated(int)),
	    	this, SLOT(readMouseInfo(int)) );

	    qApp->setOverrideCursor( QCursor( QPixmap( mouse_x ) ) );
#endif
}

LMouseObserver::~LMouseObserver()
{
    if ( rn ) {
			qApp->restoreOverrideCursor();
			delete rn;
			rn = 0;
			if ( pid )
	  	  ::kill( pid, 15 );
			pid = 0;
    }
}

void LMouseObserver::readMouseInfo( int fd )
{
printf("readMouseInfo\n");

    int available = 0;

 if ( ::ioctl( fd, FIONREAD, (char*)&available ) >= 0 ) {
	if ( available ) {
	    // text came in
	    QByteArray rba( available );
	    int nread;
	    nread = ::read( fd, rba.data(), available );
	    for( int i=0; i < nread; i++ )
		mdetect += rba[i];
	} else {
printf("gotall\n");
    // we got it all
    if( rn ){
			qApp->restoreOverrideCursor();
      delete rn;
   	  rn = 0;
     	::close( fd );
		}

		// oooh.  slow motion.
		if ( mdetect.find( "Detected psaux mouse" ) >= 0 ){
  	  mouse = PS2;
	    protocol = MTYPE_PS_2;
		}else if ( mdetect.find( "Detected logitech mouse" ) >= 0 ){
	    mouse = Logitech;
			protocol = MTYPE_LOGITECH;
		}else if ( mdetect.find( "Detected microsoft mouse" ) >= 0 ){
	    mouse = MS;
	    protocol = MTYPE_MICROSOFT;
		}else if ( mdetect.find( "Detected USB mouse") >= 0 ){
  	  mouse = USB;
	    protocol = MTYPE_PS_2;
		}

		if ( mdetect.find( "/dev/psaux" ) >= 0 )
	    port = PSAux;
		else if ( mdetect.find( "/dev/ttyS0" ) >= 0 )
	    port = S0;
		else if ( mdetect.find( "/dev/ttyS1" ) >= 0 )
  	  port = S1;
		else if ( mdetect.find( "/dev/ttyS2" ) >= 0 )
	    port = S2;
		else if ( mdetect.find( "/dev/ttyS3" ) >= 0 )
	    port = S3;
		else if ( mdetect.find( "/dev/hidbp-mse-0" ) >= 0)
	    port = HIDBP;

		blProbing = false;
    hasDetected = TRUE;
    emit gotMouse();
	}
 }
}

void LMouseObserver::setType( Type p )
{
printf( "Type: %d\n", (int)p );
    mouse = (Type)p;
		switch( mouse ) {
			case PS2: 	port = PSAux;		break;
			case USB:  	port = HIDBP;   break;
			default:    port = port;    break;
		}
}

void LMouseObserver::tellXServer()
{
	if( !blProbe && !allowCheck ) return;

//Not a good idea to run such things on a live system though
printf("tellXServer - type: %d, protocol: %d, port: %d\n", mouse, protocol, port );
#ifdef GBLI_DEVEL

#else
		XF86MiscMouseSettings m;
    ::XF86MiscGetMouseSettings( qt_xdisplay(), &m );

    QString newname = getPortName();
printf( "Port name: %s, baudrate: %d\n", newname.ascii(), m.baudrate );
		if( QFileInfo( "/tmp/mouselnk" ).readLink() != newname ){
	    ::unlink( "/tmp/mouselnk" );
  	  ::symlink( newname, "/tmp/mouselnk" );
		}
		m.type = protocol;
    m.emulate3buttons = emulate3rd;
    m.flags |= MF_REOPEN;

    // set the cursor position, NO MATTER when the fork()ing mouse
    // movements might arrive.
    qApp->setOverrideCursor( Qt::waitCursor );
    QApplication::flushX();
    int res = (int)::XF86MiscSetMouseSettings( qt_xdisplay(), &m );
    qApp->restoreOverrideCursor();
printf( "Set result: %d\n", res );
#endif
    QApplication::syncX();
    QApplication::flushX();
		blProbe = false;
}

const char * LMouseObserver::getPortName()
{
  switch( port ) {
    case PSAux:
			return "/dev/psaux";
    case S0:
			return "/dev/ttyS0";
    case S1:
			return "/dev/ttyS1";
    case S2:
			return "/dev/ttyS2";
    case S3:
			return "/dev/ttyS3";
    case HIDBP:
      return "/dev/hidbp-mse-0";
  }
  return "/dev/psaux";
}


/* ################################################################## */

LMouse::LMouse( QWidget * parent, const char * name )
    : QWidget( parent, name )
{
//printf( "come to mouse!" );
//	m_pbttnProbe = new QPushButton( "  "+ tr( "Probe" ) +"  ", this );

//  m_pchkActivate = new QCheckBox( tr("&Apply settings instantly"), this );

	m_plblType = new QLabel( tr("Type:"), this );
	m_plboxType = new QListBox( this );
	m_plboxType->insertItem( tr("PS/2 mouse"), LMouseObserver::PS2 );
	m_plboxType->insertItem( tr("Serial mouse"), LMouseObserver::Logitech );
	m_plboxType->insertItem( tr("Microsoft compatible (serial)"), LMouseObserver::MS );
	m_plboxType->insertItem( tr("USB mouse"), LMouseObserver::USB );
	m_plboxType->setMinimumHeight( m_plboxType->sizeHint().height()+ 15 );
	m_plboxType->setMinimumWidth( m_plboxType->sizeHint().width()+ 15 );
	int iMaxWidth = m_plboxType->sizeHint().width()+ 40;
	m_plboxType->setMaximumWidth( iMaxWidth );

	m_plblProtocol = new QLabel( tr("Protocol:"), this );
  m_plboxProtocol = new QListBox( this );
	m_plboxProtocol->setMinimumHeight( 60 );
	m_plboxProtocol->setMaximumWidth( iMaxWidth );

	m_plblPort = new QLabel( tr("Port:"), this );
	m_plboxPort = new QListBox( this );
	m_plboxPort->insertItem( "/dev/ttyS0 (COM1)" );
	m_plboxPort->insertItem( "/dev/ttyS1 (COM2)" );
	m_plboxPort->insertItem( "/dev/ttyS2 (COM3)" );
	m_plboxPort->insertItem( "/dev/ttyS3 (COM4)" );
	m_plboxPort->setMinimumHeight( m_plboxPort->sizeHint().height()+ 7 );
	m_plboxPort->setMaximumWidth( iMaxWidth );

  m_pchkEmulate = new QCheckBox( tr("&Emulate 3rd button"), this );

  m_pchkActivate = new QPushButton( tr("&Apply settings instantly"), this );

  m_testMouse = new LMouseView( this );
  m_testMouse->installEventFilter( m_testMouse );
	m_testMouse->setFixedSize( 100, 140 );

  observer = new LMouseObserver( this, "Observer" );
	observer->setProtocol( MTYPE_MICROSOFT );
  gotMouse();

	/** Initialize layouts */
	QVBoxLayout *loFirst = new QVBoxLayout( 7 );
//  loFirst->addWidget( m_pbttnProbe );
	loFirst->addWidget( m_plblType );
	loFirst->addWidget( m_plboxType );
	loFirst->addSpacing( 10 );
	loFirst->addWidget( m_plblProtocol );
	loFirst->addWidget( m_plboxProtocol );
	loFirst->addSpacing( 10 );
	loFirst->addWidget( m_plblPort );
	loFirst->addWidget( m_plboxPort );

	QHBoxLayout *loHTest = new QHBoxLayout();
	loHTest->addStretch( 1 );
	loHTest->addWidget( m_testMouse );
	loHTest->addStretch( 1 );

	QVBoxLayout *loVTest = new QVBoxLayout();
	loVTest->addStretch( 1 );
	loVTest->addLayout( loHTest, 1 );
	loVTest->addStretch( 1 );

	QVBoxLayout *loSecond = new QVBoxLayout( 7 );
	loSecond->addStretch( 1 );
	loSecond->addSpacing( 10 );
	loSecond->addWidget( m_pchkEmulate );
	loSecond->addSpacing( 10 );
  loSecond->addWidget( m_pchkActivate );
	loSecond->addSpacing( 10 );
	loSecond->addStretch( 1 );
	loSecond->addLayout( loVTest );
	loSecond->addSpacing( 20 );

	QHBoxLayout *loHorz = new QHBoxLayout( this, 7 );

	loHorz->addStretch( 1 );
	loHorz->addLayout( loFirst, 4 );
	loHorz->addSpacing( 5 );
	loHorz->addLayout( loSecond, 4 );
	loHorz->addStretch( 1 );

//	connect( m_pbttnProbe, SIGNAL( clicked() ), observer, SLOT( runProbing () ) );
  connect( m_plboxType, SIGNAL(highlighted(int)), this, SLOT(setType(int)) );
  connect( m_plboxProtocol, SIGNAL(highlighted(int)), this, SLOT(setProtocol(int)) );
  connect( m_plboxPort, SIGNAL(highlighted(int)), this, SLOT(setPort(int)) );
  connect( m_pchkEmulate, SIGNAL(toggled(bool)), this, SLOT(emulateClicked(bool)) );
  connect( observer, SIGNAL( gotMouse() ), this, SLOT( gotMouse() ) );
	connect( m_pchkActivate, SIGNAL(clicked()/*toggled(bool)*/), observer, SLOT(checkValue()/*setAllowCheck(bool)*/) );

//	m_pbttnProbe->setFocus();
	m_plboxType->setFocus();
}

void LMouse::showEvent( QShowEvent * )
{
	m_plboxType->setFocus();
}

void LMouse::hideEvent( QHideEvent * )
{
  observer->tellXServer();
}

LMouse::~LMouse()
{
	  delete m_plboxType;
	  delete m_plboxProtocol;
	  delete m_plblPort;
	  delete m_plboxPort;
	  delete m_pchkEmulate;
	  delete m_testMouse;
}

void LMouse::gotMouse()
{
	switch( observer->getPort() ){
		case LMouseObserver::S0: m_plboxPort->setCurrentItem( 0 ); break;
		case LMouseObserver::S1: m_plboxPort->setCurrentItem( 1 ); break;
		case LMouseObserver::S2: m_plboxPort->setCurrentItem( 2 ); break;
		case LMouseObserver::S3: m_plboxPort->setCurrentItem( 3 ); break;
		default:                 m_plboxPort->setCurrentItem( 0 ); break;
	}

  setType( observer->getType() );
}

int LMouse::getBackProtocol( int i )
{
	switch( i ){
		case MTYPE_PS_2:
		case MTYPE_MICROSOFT: return 0; // zero element in list
		case MTYPE_LOGITECH: return 0;
		default: return 0;
	}
}

void LMouse::setPort( int a_i )
{
	m_plboxPort->setCurrentItem( a_i );
	observer->setPort( getPort() );
	observer->tellXServer();
}

LMouseObserver::Port LMouse::getPort()
{
	switch( m_plboxPort->currentItem() ){
		case 0: return LMouseObserver::S0;
		case 1: return LMouseObserver::S1;
		case 2: return LMouseObserver::S2;
		case 3: return LMouseObserver::S3;
		default: return LMouseObserver::PSAux;
	}
}

void LMouse::setUpProtocolList()
{
	m_plboxProtocol->clear();
	m_plboxProtocol->setEnabled( true );
	switch( m_plboxType->currentItem() ){
		case LMouseObserver::PS2:
		case LMouseObserver::USB:
		  m_plboxProtocol->insertItem( tr("Generic PS/2 mouse") );
		  m_plboxProtocol->insertItem( tr("Auto") );
		  m_plboxProtocol->insertItem( tr("Microsoft IntelliMouse") );
	  	m_plboxProtocol->insertItem( tr("ThinkingMouse") );
		  m_plboxProtocol->insertItem( tr("Logitech MouseManPlus") );
  		m_plboxProtocol->insertItem( tr("GlidePoint") );
		  m_plboxProtocol->insertItem( tr("NetMouse") );
  		m_plboxProtocol->insertItem( tr("NetScroll") );
			break;
		case LMouseObserver::Logitech:
		  m_plboxProtocol->insertItem( tr("Logitech protocol (old)") );
		  m_plboxProtocol->insertItem( tr("Logitech MM Series") );
		  m_plboxProtocol->insertItem( tr("Logitech MouseMan") );
		  m_plboxProtocol->insertItem( tr("Auto") );
		  m_plboxProtocol->insertItem( tr("Mouse Systems") );
		  m_plboxProtocol->insertItem( tr("Bus mouse") );
		  m_plboxProtocol->insertItem( tr("MMHittab") );
		  m_plboxProtocol->insertItem( tr("GlidePoint serial") );
		  m_plboxProtocol->insertItem( tr("Microsoft IntelliMouse") );
		  m_plboxProtocol->insertItem( tr("Logitech MouseManPlus") );
		  m_plboxProtocol->insertItem( tr("ThinkingMouse") );
		  m_plboxProtocol->insertItem( tr("System Mouse") );
		  m_plboxProtocol->insertItem( tr("Xqueue") );
		  m_plboxProtocol->insertItem( tr("OSMouse") );
			break;
		case LMouseObserver::MS:
			m_plboxProtocol->insertItem( tr("Microsoft") );
			m_plboxProtocol->setEnabled( false );
			break;	
	}
  setProtocol( getBackProtocol( observer->getProtocol() ) );
}

int LMouse::getProtocol()
{
	switch( m_plboxType->currentItem() ) {
		case LMouseObserver::PS2:
		case LMouseObserver::USB:
			switch( m_plboxProtocol->currentItem() ) {
		    case 0: return MTYPE_PS_2;
				case 1: return MTYPE_AUTOMOUSE;
	  	  case 2: return MTYPE_IMPS2;
				case 3: return MTYPE_THINKINGPS2;
				case 4: return MTYPE_MMANPLUSPS2;
				case 5: return MTYPE_GLIDEPOINTPS2;
				case 6: return MTYPE_NETPS2;
				case 7: return MTYPE_NETSCROLLPS2;
			}
		case LMouseObserver::Logitech:
			switch( m_plboxProtocol->currentItem() ) {
				case 0: return MTYPE_LOGITECH;
				case 1: return MTYPE_MMSERIES;
				case 2: return MTYPE_LOGIMAN;
				case 3: return MTYPE_AUTOMOUSE;
				case 4: return MTYPE_MOUSESYS;
				case 5: return MTYPE_BUSMOUSE;
				case 6: return MTYPE_MMHIT;
				case 7: return MTYPE_GLIDEPOINT;
				case 8: return MTYPE_IMSERIAL;
				case 9: return MTYPE_IMSERIAL;
				case 10:return MTYPE_THINKING;
				case 11:return MTYPE_SYSMOUSE;
				case 12:return MTYPE_XQUEUE;
				case 13:return MTYPE_OSMOUSE;
			}
		case LMouseObserver::MS:	
			switch( m_plboxProtocol->currentItem() ) {
		    default: return MTYPE_MICROSOFT;
			}
		default:
			return MTYPE_MICROSOFT;
	}
}

void LMouse::setType( int p )
{
  observer->setType( (LMouseObserver::Type)p );
	m_plboxType->setCurrentItem( p );
  bool serial = (p == LMouseObserver::Logitech) || (p == LMouseObserver::MS);

	m_plboxPort->setEnabled( serial );
	observer->setPort( getPort() );

	setUpProtocolList();
}

void LMouse::setProtocol( int i )
{
		m_plboxProtocol->setCurrentItem( i );
		observer->setProtocol( getProtocol() );
    m_testMouse->setWheel( hasWheel() );
		observer->setType( (LMouseObserver::Type)m_plboxType->currentItem() );
		observer->tellXServer();
}

bool LMouse::hasWheel()
{
// BestLinux TODO: I think it's not very correct for our list of mouses
  return
		(observer->getType() == LMouseObserver::PS2) &&
 	  (observer->getProtocol() != MTYPE_PS_2  &&
			observer->getProtocol() != MTYPE_GLIDEPOINTPS2);
}

void LMouse::emulateClicked( bool a_bl )
{
  observer->setEmulation( a_bl );
	observer->tellXServer();
}

const char * LMouse::getGPMName()
{
	switch( observer->getProtocol() ) {
    case MTYPE_MICROSOFT     : return "Microsoft";
		case MTYPE_MOUSESYS      : return "MouseSystems";
    case MTYPE_MMSERIES      : return "MMSeries";
    case MTYPE_LOGITECH      : return "Logitech";
    case MTYPE_BUSMOUSE      : return "Busmouse";
    case MTYPE_LOGIMAN       : return "MouseMan";
    case MTYPE_PS_2          : return "ps/2";
    case MTYPE_MMHIT         : return "MMHitTab";
    case MTYPE_GLIDEPOINT    : return "GlidePoint";
    case MTYPE_IMSERIAL			 : return "ms3";
    case MTYPE_THINKING      : return "ms3";
    case MTYPE_IMPS2         : return "imps2";
    case MTYPE_THINKINGPS2   : return "ps/2";
    case MTYPE_MMANPLUSPS2   : return "ps/2";
    case MTYPE_GLIDEPOINTPS2 : return "ps/2";
    case MTYPE_NETPS2        : return "netmouse";
    case MTYPE_NETSCROLLPS2  : return "netmouse";
    case MTYPE_SYSMOUSE      : return "SysMouse";
    case MTYPE_AUTOMOUSE     : return "pnp";
		case MTYPE_XQUEUE        : return "Xqueue";
		case MTYPE_OSMOUSE       : return "OSMouse";
		default                  : return "Microsoft";
	}
}

const char * LMouse::getMouseName()
{
	switch( observer->getProtocol() ) {
    case MTYPE_MICROSOFT     : return "Microsoft";
		case MTYPE_MOUSESYS      : return "MouseSystems";
    case MTYPE_MMSERIES      : return "MMSeries";
    case MTYPE_LOGITECH      : return "Logitech";
    case MTYPE_BUSMOUSE      : return "BusMouse";
    case MTYPE_LOGIMAN       : return "MouseMan";
    case MTYPE_PS_2          : return "PS/2";
    case MTYPE_MMHIT         : return "MMHitTab";
    case MTYPE_GLIDEPOINT    : return "GlidePoint";
    case MTYPE_IMSERIAL			 : return "IntelliMouse";
    case MTYPE_THINKING      : return "ThinkingMouse";
    case MTYPE_IMPS2         : return "IMPS/2";
    case MTYPE_THINKINGPS2   : return "ThinkingMousePS/2";
    case MTYPE_MMANPLUSPS2   : return "MouseManPlusPS/2";
    case MTYPE_GLIDEPOINTPS2 : return "GlidePointPS/2";
    case MTYPE_NETPS2        : return "NetMousePS/2";
    case MTYPE_NETSCROLLPS2  : return "NetScrollPS/2";
    case MTYPE_SYSMOUSE      : return "SysMouse";
    case MTYPE_AUTOMOUSE     : return "Auto";
		case MTYPE_XQUEUE        : return "Xqueue";
		case MTYPE_OSMOUSE       : return "OSMouse";
		default                  : return "Microsoft";
	}
}

