/***************************************************************************
  																lmouse.h
                             -------------------
    begin                : Fri Oct 29 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LMOUSE_H
#define LMOUSE_H

#include <qwidget.h>
#include <qlabel.h>
#include <qstring.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include "config.h"

class QLabel;
class QSocketNotifier;
class QListBox;

class LMouseView : public QWidget{
    Q_OBJECT
public:
    LMouseView( QWidget *parent=0, const char *name=0);
    ~LMouseView() {};
    void setWheel( bool );
    void resetState();

protected:
    void paintEvent( QPaintEvent * );
    bool eventFilter( QObject *, QEvent * );

    void enterEvent( QEvent * );
    void mouseMoveEvent( QMouseEvent * );
    void focusInEvent( QFocusEvent * ){};
    void focusOutEvent( QFocusEvent * ){};

private:
    void paintState( bool = FALSE);
    ButtonState state;
    int wheelOffset;
    bool hasWheel;
};

class LMouseObserver : public QObject{
    Q_OBJECT
public:
    LMouseObserver( QObject * parent, const char * name = 0 );
    ~LMouseObserver();

    enum Type { PS2, Logitech, MS, USB };
    enum Port { PSAux, S0, S1, S2, S3, HIDBP };

    void setType( Type m );
		Type getType() { return mouse; };
    void setProtocol( int p ) { protocol = p; };
		int  getProtocol() { return protocol; };
    void setPort( Port p ) { port = (Port)p; };
		Port getPort() { return port; };
    void setEmulation( bool emulate ) {	emulate3rd = emulate; };
		const char * getPortName();

public slots:
    void tellXServer();
		void runProbing();

signals:
    void gotMouse();

private slots:
		void setAllowCheck( bool state ) {
			allowCheck = state;
			tellXServer(); };
		void checkValue(){
			allowCheck = true;
			tellXServer();
			allowCheck = false; };
    void readMouseInfo(int);

private:
    Type mouse;
    Port port;
    int protocol;

    bool hasDetected;
    QString mdetect;
    bool emulate3rd;
    QSocketNotifier * rn;
    int pid;
		bool blProbe;
		bool allowCheck;
		bool blProbing;
};


class LMouse: public QWidget
{
    Q_OBJECT
public:
    LMouse( QWidget * parent, const char * name = 0 );
    ~LMouse();

    const char * getMouseName();
    const char * getPortName() { return observer->getPortName(); };
		const char * getGPMName();
    LMouseObserver::Type getMouseType() { return observer->getType(); }
    bool emulate3buttons() { return m_pchkEmulate->isChecked(); };
    bool hasWheel();
		void showEvent( QShowEvent *);
		void hideEvent( QHideEvent *);

private slots:
		void setPort( int a_i );
    void gotMouse();
    void setType( int );
    void setProtocol( int i );
    void emulateClicked( bool );

protected:
	int getProtocol();
	int getBackProtocol( int i );
	LMouseObserver::Port getPort();
	void setUpProtocolList();
  LMouseObserver* observer;

	QPushButton *m_pbttnProbe;
	QLabel     *m_plblType;
	QListBox   *m_plboxType;
	QLabel     *m_plblProtocol;
	QListBox   *m_plboxProtocol;
	QLabel     *m_plblPort;
	QListBox   *m_plboxPort;
	QCheckBox  *m_pchkEmulate;
//	QCheckBox  *m_pchkActivate;
	QPushButton  *m_pchkActivate;
	LMouseView *m_testMouse;

};


#endif

















