/***************************************************************************
                          lnetwork.h -  description
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef LNETWORK_H
#define LNETWORK_H

#include <qwidget.h>
#include <qframe.h>
#include <qlineedit.h>
#include <netinet/in.h>
#include "configfile.h"
#include "sysconfigfile.h"

class QButtonGroup;
class QCheckBox;
class QRadioButton;
class LLabel;
class LNetwork;
class QSocket;


class LAddressEdit: public QFrame
{
    Q_OBJECT
public:
    enum Mode { Address, Netmask, Router, Other };
    LAddressEdit( Mode, QWidget * parent, const char * name = 0 );
    ~LAddressEdit();

    QString text();
    struct in_addr address();

    bool eventFilter( QObject *, QEvent * );

    QSize sizeHint() const;
    QSizePolicy sizePolicy() const;

    bool edited() const;

    void setEnabled( bool );

public slots:
    void setAddress( const QString & );
    void setAddress( struct in_addr );

signals:
    void returnPressed();

protected:
    void drawContents( QPainter * );

private:
    QLineEdit * b1, * b2, * b3, * b4;
    friend class LNetwork;
    Mode m;
    bool e;
};


class LNetinfo : public QObject {
    Q_OBJECT;
public:
    enum Source {
	Default = 0,
	Wildguess = 1,
	Probed = 2,
	User = 3
    };
    enum Setup {
	NoEthernet,
	UseDHCP,
	UseStaticConfig
    };

protected:
    class Item {
    public:
	Item(const char *, const char *);
	~Item();

	bool			set(const char *, Source);

	Source			source;
	QString			name;
	QString			value;
    };
public:
    LNetinfo();
    ~LNetinfo();

    enum Probe {
	Idle = 0,
	Netmask,
	GBLI,

	MaxProbe
    };

    // Set various items
    // (either from previous bootp/dhcp probe, or by user input).
    bool			setMode(Setup);
    bool			setAddress(const char *, Source);
    bool			setNetmask(const char *, Source);
    bool			setGateway(const char *, Source);
    bool			setNameserver(const char *, Source);
    bool			setNameserver2(const char *, Source);
    bool			setHostname(const char *, Source);
    bool			setProxyName(const char *, Source);
    bool			setProxyPort(const char *, Source);
    bool			setNoProxyFor(const char *, Source);
    bool			setNISDomain(const char *, Source);

    // Get these items
    inline const char *		getAddress()
				{ return address.value; }
    inline const char *		getNetmask()
				{ return netmask.value; }
    inline const char *		getGateway()
				{ return gateway.value; }
    inline const char *		getNameserver()
				{ return nameserver.value; }
    inline const char *		getNameserver2()
				{ return nameserver2.value; }
    inline const char *		getHostname()
				{ return hostname.value; }
    inline const char *		getProxyName()
				{ return proxy.value; }
    inline const char *		getProxyPort()
				{ return proxyport.value; }
    inline const char *		getNoProxy()
				{ return noproxyfor.value; }

    inline const char *		getNISDomain()
				{ return nisdomain.value; }

    // Start the network probe
    void			probe(bool);

    // Save configuration
    void			save(const char *root);

    bool hadDHCP() { return dhcp; };

signals:
    void			newConfig();

private slots:
    void			inputNetprobe();

protected:
    void			inputBOOTP();
    void			startProbe(Probe, bool);
    void			stopProbe();
    bool			saveEther(const char *);
    bool			saveResolv(const char *);
#ifdef NOT_USED
    bool			saveNIS(const char *);
#endif
    bool			saveSysNetwork(const char *);
    bool			saveProxy(const char *);
		bool      saveHosts(const char *root);
#ifdef NO_USED
    bool			saveTmpNetwork();
#endif

protected:
    const char *netprobePath;

    Probe			currentProbe;
    QSocket *			probeSock;
    int				probePid;

    Setup			mode;
    Item			address;
    Item			netmask;
    Item			gateway;
    Item			nameserver;
    Item			nameserver2;
    Item			hostname;
    Item			proxy;
    Item			proxyport;
    Item			noproxyfor;
    Item			nisdomain;
 
    bool      dhcp;
    
};

class LNetwork: public QWidget
{
    Q_OBJECT
public:
    LNetwork( LNetinfo *, QWidget * parent, const char * name = 0 );
    ~LNetwork();

    QString ip() { return ipEdit->text(); }
    QString netmask() { return netmaskEdit->text(); }
    QString gateway() { return routerEdit->text(); }
    QString hostname() { return nameEdit->text(); }

    QString nameserver1() { return ns1Edit->text(); }
    QString nameserver2() { return ns2Edit->text(); }
    QString nameserver3() { return ns3Edit->text(); }
    QString nisDomain() { return nisEdit->text(); }

		QString proxy() { return proxyEdit->text(); }
		QString proxyport() { return proxyPortEdit->text(); }
		QString noproxyfor() { return noproxyforEdit->text(); }

//    LNetinfo::Setup mode() const;

public slots:
    void setMode( int );

private slots:
    void		addInputWidget(QWidget *);
    bool		eventFilter(QObject *, QEvent *);
    void		updateWidgets();
    void		updateInfo();

private:
    LAddressEdit * ipEdit;
    LAddressEdit * netmaskEdit;

    LAddressEdit * routerEdit;

    LAddressEdit * ns1Edit;
    LAddressEdit * ns2Edit;
    LAddressEdit * ns3Edit;

    QLineEdit * proxyEdit;
    QLineEdit * proxyPortEdit;
    QLineEdit * noproxyforEdit;

    QLineEdit * nameEdit;

    QLineEdit * nisEdit;

private:
    QButtonGroup * bg;
    LNetinfo * info;
};


#endif



