/***************************************************************************
                               lsound.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/



#include <stdlib.h> 
#include <sys/types.h>
#include <errno.h>
#include <sys/errno.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h> 
#include <linux/soundcard.h>
#include <unistd.h>

#include <qlayout.h>
#include <qstring.h>
#include <qstrlist.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qaccel.h>

#include "linlinehelp.h"
#include "lsound.h"
#include "liconloader.h"
#include "config.h"


LSound::LSound(QWidget *parent, const char *name)
  : QWidget(parent, name)
{
  // If all else fails, use standard soundblaster
  // FIXME: this is a large hack...
  if (!LSound::hasSound()) {
  	if (!system("modprobe sb io=0x220 irq=0x5 dma=0x1 dma16=0x5 mpu_io=0x330 >>/tmp/insmod 2>&1")) {
  		QFile moddef("/etc/modules/default");
  		moddef.open(IO_WriteOnly | IO_Append);
  		QTextStream md(&moddef);
  		md << "sb" << "\n";
  		moddef.close();
  		QFile modconf("/etc/conf.modules");
  		modconf.open(IO_WriteOnly | IO_Append);
  		QTextStream mc(&modconf);
  		mc << "options sb io=0x220 irq=0x5 dma=0x1 dma16=0x5 mpu_io=0x330" << "\n";
  		modconf.close();
  	}
  }

  // ----------------------------------------------------------------

  QHBoxLayout *hbox = new QHBoxLayout(this,6);

  QVBoxLayout *vbox = new QVBoxLayout( 6 );
	hbox->addStretch( 1 );
	hbox->addLayout( vbox, 8 );
	hbox->addStretch( 1 );

  QLabel *label = new QLabel(tr("A sound card was detected:"), this);
  vbox->addWidget(label);

  vbox->addSpacing(12);
  
  QFile sound("/proc/sound");
  if (sound.open(IO_ReadOnly))
    {
      QGridLayout *grid = new QGridLayout(vbox,1,1);

      QTextStream ts(&sound);
      QString label_txt;
      QStrList contents;
      int cnt=0;
      QString line = "";
      while (!line.isNull())
	{
	  line = ts.readLine();
	  if (line.isNull())
	    line = ts.readLine();

	  if (line == "Audio devices:")
	    {
	      label_txt = tr("Audio devices:");
	      contents.clear();
	    }
	  else if (line == "Synth devices:")
	    {
	      if (contents.count() > 0)
		{
		  grid->addWidget(new QLabel(label_txt,this), cnt, 0);
		  for (const char *it = contents.first(); it != 0; it = contents.next())
		      grid->addWidget(new QLabel(it, this), cnt++, 1);
		}	      
	      label_txt = tr("Synth devices:");
	      contents.clear();
	    }
	  else if (line == "Midi devices:")
	    {
	      if (contents.count() > 0)
		{
		  grid->addWidget(new QLabel(label_txt,this), cnt, 0);
		  for (const char *it = contents.first(); it != 0; it = contents.next())
		      grid->addWidget(new QLabel(it, this), cnt++, 1);
		}	      
	      label_txt = tr("Midi devices:");
	      contents.clear();
	    }
	  else if (line == "Timers:")
	    {
	      if (contents.count() > 0)
		{
		  grid->addWidget(new QLabel(label_txt,this), cnt, 0);
		  for (const char *it = contents.first(); it != 0; it = contents.next())
		      grid->addWidget(new QLabel(it, this), cnt++, 1);
		}	      
	      label_txt = tr("Timers:");
	      contents.clear();
	    }
	  else if (line == "Mixers:")
	    {
	      if (contents.count() > 0)
		{
		  grid->addWidget(new QLabel(label_txt,this), cnt, 0);
		  for (const char *it = contents.first(); it != 0; it = contents.next())
		      grid->addWidget(new QLabel(it, this), cnt++, 1);
		}	      
	      label_txt = tr("Mixers:");
	      contents.clear();
	    }
	  else if (!line.isEmpty())
	    {
	      if (int pos = line.find(": ") > 0)
		line.remove(0,pos+2);
	      contents.append(line);
	    }
	}
      
      if (contents.count() > 0)
	{
	  grid->addWidget(new QLabel(label_txt,this), cnt, 0);
	  for (const char *it = contents.first(); it != 0; it = contents.next())
	    grid->addWidget(new QLabel(it, this), cnt++, 1);
	}	      
      grid->setColStretch(0,1);
      grid->setColStretch(1,2);

      sound.close();
    }
  
  vbox->addSpacing(12);

  QFrame *line = new QFrame(this);
  line->setFrameStyle(QFrame::HLine | QFrame::Sunken);
  vbox->addWidget(line);

  QGridLayout *grid = new QGridLayout(vbox,1,1,6);

  if (LSound::hasSound())
    {
      QPushButton *wav = new QPushButton(this);
      wav->setPixmap(LIconLoader::loadIcon("play.xpm"));
      wav->setFixedSize(32,32);
      grid->addWidget(wav,0,0);
      label = new QLabel(tr("Test digital sound"), this);
      grid->addWidget(label,0,1);
      connect(wav, SIGNAL(clicked()), this, SLOT(testWav()));
    }

  if (LSound::hasMidi())
    {
      QPushButton *midi = new QPushButton(this);
      midi->setPixmap(LIconLoader::loadIcon("play.xpm"));
      midi->setFixedSize(32,32);
      grid->addWidget(midi,1,0);
      label = new QLabel(tr("Test MIDI playback"), this);
      grid->addWidget(label,1,1);
      connect(midi, SIGNAL(clicked()), this, SLOT(testMidi()));
    }

  QAccel *a = new QAccel(this);
  a->connectItem(a->insertItem(Qt::CTRL+Qt::ALT+Qt::Key_S), this, SLOT(testMpg()));  
  
  grid->setColStretch(1,1);

  vbox->addSpacing(12);

  mixer = open("/dev/mixer", O_RDWR);
  if (mixer >= 0)
    {
      QGridLayout *mgrid = new QGridLayout(vbox,1,1,6);

      label = new QLabel(tr("Volume:"),this);
      mgrid->addWidget(label,0,0);
      
      QSlider *slider = new QSlider(0,100,25,75,QSlider::Horizontal,this);
      slider->setTickmarks(QSlider::Below);
      slider->setTickInterval(25);
      mgrid->addWidget(slider,0,1);      
      mgrid->setColStretch(1,1);
      
      connect(slider, SIGNAL(valueChanged(int)), this, SLOT(volumeChanged(int)));

      label = new QLabel(tr("Balance:"),this);
      mgrid->addWidget(label,1,0);
      
      slider = new QSlider(0,100,25,50,QSlider::Horizontal,this);
      slider->setTickmarks(QSlider::Below);
      slider->setTickInterval(10);
      mgrid->addWidget(slider,1,1);      
      
      connect(slider, SIGNAL(valueChanged(int)), this, SLOT(balanceChanged(int)));

      volume = 75;
      balance = 50;
      setMixer(volume, balance);
    }

  vbox->addStretch();

/*  LInlineHelp *inlineHelp = new LInlineHelp(this);
  inlineHelp->setDocument("sound_overview.qml");
  hbox->addWidget(inlineHelp);*/
}


LSound::~LSound()
{
  if (mixer >= 0)
    ::close(mixer);
}


void LSound::testWav()
{
  QFile data("/mnt/root/tmp/test.wav");
  if (!data.exists()) {
    ::mkdir("/mnt/root/tmp/",01777);
    system("cp ./sounds/test.wav /mnt/root/tmp/test.wav");
  }
  system("sox /mnt/root/tmp/test.wav -t .ossdsp /dev/dsp &");
}


void LSound::testMidi()
{
  QFile data("/mnt/root/tmp/test.mid");
  if (!data.exists()) {
    ::mkdir("/mnt/root/tmp/",01777);
    system("cp ./sounds/test.mid /mnt/root/tmp/test.mid");
  }
  system("playmidi /mnt/root/tmp/test.mid &");
}


void LSound::testMpg()
{
  QFile data("/tmp/mnt/root/test.mp3");
  if (!data.exists()) {
    ::mkdir("/mnt/root/tmp/",01777);
    system("cp ./sounds/test.mp3 /mnt/root/tmp/test.mp3");
  }
  system("mpg123 /mnt/root/tmp/test.mp3 &");
}


void LSound::balanceChanged(int value)
{
  balance = value;
  setMixer(volume, balance);
}


void LSound::volumeChanged(int value)
{
  volume = value;
  setMixer(volume, balance);
}


void LSound::setMixer(int vol, int bal)
{
  int volume, vr, vl;

  if (mixer < 0)
    return;

  // calculate the volumes
  vr = vl = 100;
  if (bal<50)
    vr = 2*bal;
  else
    vl = 200 - 2*bal;

  vl = vl*vol/100;
  vr = vr*vol/100;
  volume = vl + (vr << 8);

  int mask;
  ioctl(mixer, SOUND_MIXER_READ_DEVMASK, &mask);
  for (int dev=0; dev<SOUND_MIXER_NRDEVICES; dev++)
    if ((mask & (1 << dev)) != 0)
      if (dev != SOUND_MIXER_MIC)
	ioctl(mixer, MIXER_WRITE(dev), &volume);
      else
	{
	  volume = 0;
	  ioctl(mixer, MIXER_WRITE(dev), &volume);
	}
}


void LSound::finish()
{
  system("rm /mnt/root/tmp/test.mid");
  system("rm /mnt/root/tmp/test.wav");
  system("rm /mnt/root/tmp/test.mp3");
}


bool LSound::hasSound()
{
  int fd;

  fd = ::open( "/dev/dsp", O_WRONLY );
  if( fd != -1 ) {
    ::close( fd );
    return TRUE;
  }
  if( errno == EBUSY ) // we are playing?
    return TRUE;
  return FALSE;
}


bool LSound::hasMidi()
{
  int fd;

  fd = ::open("/dev/sequencer",O_WRONLY);
  if (fd!=-1) {
    ::close(fd);
    return TRUE;
  }
  if (errno==EBUSY) // we are playing?
    return TRUE;
  return FALSE;
}


bool LSound::hasMixer()
{
  int fd;

  fd = ::open("/dev/mixer",O_WRONLY);
  if (fd!=-1) {
    ::close(fd);
    return TRUE;
  }
  if (errno==EBUSY) // we are mixing?
    return TRUE;
  return FALSE;
}



