/***************************************************************************
                               lvideocard.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "lvideocard.h"
#include "linlinehelp.h"
#include "lxserver.h"
#include "lcarddb.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qregexp.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtextstream.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qwizard.h>
#include <qvalidator.h>

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>


static QString resolutionLabel;


LVideoCard::LVideoCard( QWidget * parent, const char * name )
    : QWidget( parent, name ), db("./VideoCards")
{
		setFontPropagation( QWidget::AllChildren );
		setFont( QFont::defaultFont() );
    maxClock = 0;
    clocks.resize( 1 );
    clocks[0] = 25.175;
    ram = 256 * 1024;
    maxRes = QSize( 0, 0 );
    hasProbed = FALSE;
		
    // try to detect the graphics card
    LCardDBEntry *entry = db.probe();    

		QHBoxLayout *loHorz = new QHBoxLayout( this );

		QVBoxLayout *loVert = new QVBoxLayout();
    QGridLayout * g = new QGridLayout( 1, 1, 6 );
		loVert->addSpacing( 10 );
		loVert->addLayout( g );
		loVert->addSpacing( 10 );

		loHorz->addStretch( 1 );
		loHorz->addLayout( loVert, 8 );
		loHorz->addStretch( 1 );	

/*    LInlineHelp* inlineHelp = new LInlineHelp( this );
    inlineHelp->setDocument( "videocard_overview.qml" );
    g->addMultiCellWidget( inlineHelp, 0, 8, 2, 2 );*/

    videoCard = new QComboBox(TRUE,this);
    QStrList list;
    db.getCardList(list);
    list.sort();
    videoCard->insertStrList(list);
    g->addWidget( videoCard, 1, 1 );
    g->addWidget( new QLabel( tr( "Card Type" ), this ), 1, 0 );
    connect(videoCard, SIGNAL(activated(const QString &)), this, SLOT(cardSelected(const QString &)));

    QString text;
    if (entry)
      text = tr( "Your graphics card has been recognized. Probing for "
		 "hardware details should be safe.");
    else
      text = tr(
        "Your graphics card has not been recognized. Try to select your card from the "
        "list. <p> If your card is not in the list, chances are that probing will "
        "find the necessary information, but it might also hang your system. In that "
        "case, choose VGA compatible from the list." );

//		g->addWidget( new QLabel(text,this),3,1);
    card_msg = new QLabel( text, this );
		card_msg->setFont( QFont::defaultFont() );
    g->addWidget( card_msg, 3, 1 );
    card_msg->setAlignment( AlignTop + AlignLeft + WordBreak );
//		g->setRowStretch( 3, 3 );
    card_msg->setMinimumHeight( card_msg->heightForWidth( 400 )+ 30 );
    card_msg->setFixedWidth( 400 );

    QFrame *hline = new QFrame(this);
    hline->setFrameStyle(QFrame::HLine | QFrame::Sunken);
    g->addMultiCellWidget(hline, 4,4, 0,1);

    g->addMultiCellWidget(new QLabel(tr("Hardware details:"), this), 5,5,0,1);
    g->addRowSpacing(5,18);
    g->addRowSpacing(1,18);

    g->addWidget( new QLabel( tr( "Video RAM" ), this ), 6, 0, AlignTop );

    QHBoxLayout *rbox = new QHBoxLayout();
    g->addLayout(rbox, 6,1);
    
    videoRam = new QComboBox(TRUE,this);
    videoRam->setValidator( new QIntValidator( 256, 256*1024, videoRam ) );
    videoRam->insertItem("256");
    videoRam->insertItem("1024");
    videoRam->insertItem("2048");
    videoRam->insertItem("4096");
    videoRam->insertItem("8192");
    videoRam->insertItem("16384");
    videoRam->insertItem("32768");
    videoRam->insertItem("65536");
    videoRam->insertItem("131072");
    rbox->addWidget(videoRam);
    
    QLabel *l = new QLabel("KB",this);
    rbox->addWidget(l);
    rbox->addStretch();
    
    g->addWidget( new QLabel( tr( "Mode clock(s)" ), this ), 7, 0, AlignTop );
    QVBoxLayout * v = new QVBoxLayout( 7 );
    g->addLayout( v, 7, 1 );
    maxClockButton
			= new QRadioButton( tr( "Programmable up to (MHz):" ), this );
    clocksButton
			= new QRadioButton( tr( "Fixed clocks (MHz):" ), this );
    v->addWidget( maxClockButton );
    maxClockBox = new QLineEdit( this );
    QHBoxLayout * h = new QHBoxLayout( 6 );
    v->addLayout( h );
    h->addSpacing( 20 );
    h->addWidget( maxClockBox );
    h->addSpacing( 20 );
    v->addWidget( clocksButton );
    clocksBox = new QLineEdit( this );
    h = new QHBoxLayout( 6 );
    v->addLayout( h );
    h->addSpacing( 20 );
    h->addWidget( clocksBox );
    h->addSpacing( 20 );

    connect( clocksBox, SIGNAL(textChanged(const QString &)),
	     this, SLOT(controlNextButton()) );
    connect( maxClockBox, SIGNAL(textChanged(const QString &)),
	     this, SLOT(controlNextButton()) );

    QButtonGroup * bg = new QButtonGroup( this );
    bg->hide();
    bg->insert( clocksButton );
    bg->insert( maxClockButton );
    connect( bg, SIGNAL(clicked(int)),
	     this, SLOT(updateClocks()) );

    probeButton = new QPushButton( tr( "Probe" ), this );
    g->addWidget( probeButton, 8, 0, AlignTop );
    connect( probeButton, SIGNAL(clicked()),
	     this, SLOT(probeVideoCard()) );

    id = new QLabel(tr( "You can enter the details of your card here, or you can "
              "probe for the values. Note that probing may hang your system."),this);
    g->addWidget( id, 8, 1, AlignTop );
    id->setAlignment( AlignTop + AlignLeft + WordBreak );
    id->setMinimumHeight(id->heightForWidth( 400 ) );
    id->setFixedWidth( 400 );
    
    g->setRowStretch( 8, 42 );
    //    g->setColStretch( 1, 1 );

    QFile f( "/tmp/xserver-output" );
    if ( f.open( IO_ReadOnly ) ) { // file opened successfully
		 	QTextStream t( &f );	    // use a text stream
			QString result = t.read();
			parseXF86Output( result );
    }

    if (entry){
			cardName = entry->Name;
			serverChanged(entry->Server);
    }
    updateCardInfo();

		setFont( QFont::defaultFont() );

    probeButton->setFocus();
}


LVideoCard::~LVideoCard()
{
}


bool LVideoCard::supportsModeClock( float f )
{
    // if programmable, allow just 0.1% deviation
    if ( maxClock > 1.0 )
			return f <= maxClock * 1.001;
    // otherwise, allow up to 0.4%
    int i = clocks.size();
    while( --i >= 0 )
			if ( f >= clocks[i] * 0.996 && f <= clocks[i] * 1.004 )
	  	  return TRUE;
    return FALSE;
}


void LVideoCard::probeVideoCard()
{
    if ( QMessageBox::information( this, tr( "Probe Graphics Card" ),
				      tr( "While probing for the correct video card, amount of memory and video\n"
					  "mode clocks, the screen will briefly turn black as if switching video \n"
					  "mode, then switch back to this mode.\n\n"
					  "This process should take about five seconds.\n" ),
				   tr( "Probe" ), tr( "Abort" ), QString::null,
				   0, 1 ) == 1 )
			return;

   	if( QFileInfo( "/tmp/xlink" ).exists() ){
   		if( !QFileInfo( "/tmp/xlink" ).readLink().length() ||
   			!QFile( QFileInfo( "/tmp/xlink" ).readLink() ).exists() ){
  				::unlink( "/tmp/xlink" );
					if( LXServer::runSVGA )
	  				::symlink( "/usr/X11R6/bin/XF86_SVGA", "/tmp/xlink" );
					else
	  				::symlink( "/usr/X11R6/bin/XF86_VGA16", "/tmp/xlink" );
   		}
   	}else{
			if( LXServer::runSVGA )
				::symlink( "/usr/X11R6/bin/XF86_SVGA", "/tmp/xlink" );
			else
				::symlink( "/usr/X11R6/bin/XF86_VGA16", "/tmp/xlink" );
  	}

    serverError = FALSE;

    QApplication::flushX();
    QString result;
    int fd[2];
    ::pipe(fd);
    int pid = ::fork();
    if ( pid == 0 ) {
    	::close( fd[0] );
    	
    	dup2( fd[1], 2 );
    	int i = getdtablesize();
    	while( --i >= 0 )
   	    if ( i != 2 )
	    		::close( i );
    	::open( "/dev/null", O_RDWR );
    	::dup( 0 );
    	
    	execl( "/tmp/xlink", "xlink", "-bestRefresh",
    	        "-xf86config", "/tmp/XF86Config_own", "-probeonly", ":2", "vt09", 0 );
    	exit(1);
    }
printf( "probeVideoCard pid: %d\n", pid );
    ::close( fd[1] );

    QFile f;
    if ( f.open(IO_ReadOnly, fd[0]) ) {    // file opened successfully
     	QTextStream t( &f );	    // use a text stream
    	result = t.read();

    	// dump output for debugging
    	QFile f2("/tmp/xserver.out");
    	if (f2.open(IO_WriteOnly)) {
    	  QTextStream ts2(&f2);
    	  ts2 << result.ascii();
    	  f2.close();
    	}

     	::close( fd[0] );
    	hasProbed = TRUE;
    	parseXF86Output( result );
    	if (!serverError){
    	    updateCardInfo();
    	    updateClocks();
   	  }
    }

    ::waitpid(pid, NULL, 0);

    videoRam->setFocus();

    if (!serverError)
      QMessageBox::information( this, tr( "Probe Graphics Card" ),
				tr( "Probing is complete and apparently successful." ),
				"OK" );
    else
      QMessageBox::information( this, tr( "Probe Graphics Card" ),
				tr( "Probing failed. Your card is not supported by this server." ),
				"OK" );
}


void LVideoCard::parseXF86Output( const QString & xf86 )
{
    int pos;
    int len;

    QRegExp error("server error");
    pos = error.match(xf86, 0, &len);
    if (pos >= 0)
      serverError = TRUE;

    QRegExp none("None of the configured devices were detected");
    pos = none.match(xf86, 0, &len);
    if (pos >= 0)
      serverError = TRUE;
    
    QRegExp vramExp("(--).*video *ram[ ]*[:=]?",FALSE);
    pos = vramExp.match( xf86, 0, &len );
    if ( pos >= 0 ) {
      if ( sscanf( xf86.mid( pos+len, 20 ), "%d", &ram ) == 1 )
	ram = ram * 1024;
    }
    else
      {
	QRegExp memExp("(--).*mem:[ ]*",FALSE);
	pos = memExp.match( xf86, 0, &len );
	if ( pos >= 0 ) {
	  if ( sscanf( xf86.mid( pos+len, 20 ), "%d", &ram ) == 1 )
	    ram = ram * 1024;
	}
      }                                                                                                          

    QRegExp clockExp( "(--) [^ ]*: Maximum allowed dot-clock: " );
    pos = clockExp.match( xf86, 0, &len );
    if ( pos >= 0 ) {
	// found programmable clock
	QString tmp = xf86.mid( pos+len, 20 );
	(void)sscanf( tmp, "%f", &maxClock );
    } else {
	QRegExp clocksExp( "(--) [^ ]*: clocks: " );
	pos = clocksExp.match( xf86, 0, &len );
	if ( pos >= 0 ) {
	    // found other clock
	    QString tmp = xf86.mid( pos+len, 100 ).simplifyWhiteSpace();
	    int i = 0;
	    clocks.resize( 0 );
	    float clock;
	    while( sscanf( tmp, "%g", &clock ) == 1 ) {
		i++;
		clocks.resize( i );
		clocks[i-1] = clock;
		pos = tmp.find( ' ' );
		if ( pos > 0 )
		    tmp = tmp.mid( pos+1, tmp.length() );
	    }
	}
    }
	
    QRegExp c1Exp( "(--) [^ ]*: PCI: " );
    QRegExp c2Exp( "(--) [^ ]*: chipset: ", FALSE );
    pos = c1Exp.match( xf86, 0, &len );
    if ( pos < 0 )
	pos = c2Exp.match( xf86, 0, &len );
    if ( pos >= 0 ) {
	// found chipset, either via PCI or other bus
	QString tmp = xf86.mid( pos+len, 100 ).simplifyWhiteSpace();
	pos = tmp.find( ", " );
	if ( pos > 3 ) // not RIGHT at the start, ok?
	    tmp.truncate( pos );
	cardName = tmp;
    }

    QRegExp dsExp( "(--) [^ ]*: [^ ]*: Display Size: " );
    pos = dsExp.match( xf86, 0, &len );
    if ( pos >= 0 ) {
	// found max size of LCD display
	QString tmp = xf86.mid( pos+len, 100 );
	int x, y;
	if ( sscanf( tmp, "x=%d; y=%d", &x, &y ) == 2 )
	    maxRes = QSize( x, y );
    }
}


void LVideoCard::updateCardInfo()
{
    videoCard->setEditText( cardName );

    QString v; 
    v.setNum(ram/1024);
    videoRam->setEditText(v);

    if ( maxClock > 1.0 ) {
	QString tmp;
	tmp.sprintf( "%g", maxClock );
	maxClockBox->setText( tmp );
	maxClockBox->setEdited( FALSE );
	clocksBox->setEnabled( FALSE );
	maxClockBox->setEnabled( TRUE );
	if ( clocksButton->isChecked() )
	    clocksButton->setChecked( FALSE );
	if ( !maxClockButton->isChecked() )
	    maxClockButton->setChecked( TRUE );
    } else {
	if ( maxClockButton->isChecked() )
	    maxClockButton->setChecked( FALSE );
	if ( !clocksButton->isChecked() )
	    clocksButton->setChecked( TRUE );
	QString tmp;
	int i=0;
	while( i < (int)clocks.size() ) {
	    if ( tmp.length() )
		tmp += ", ";
	    QString tmp2;
	    tmp2.sprintf( "%g", clocks[i] );
	    tmp += tmp2;
	    i++;
	}
	clocksBox->setText( tmp );
	clocksBox->setEnabled( TRUE );
	maxClockBox->setEnabled( FALSE );
	clocksBox->setEdited( FALSE );
    }
}


bool LVideoCard::memoryProbed()
{
    return memory() == ram; // hm...
}


int LVideoCard::memory()
{
  bool ok;
  uint  value;

  value = videoRam->currentText().toUInt(&ok);

  if (ok)
    return value*1024;
  else
    return ram;
}


QString LVideoCard::clockLine()
{
    if ( ( maxClockBox->isEnabled() && !maxClockBox->edited() ) ||
	 ( clocksBox->isEnabled() && !clocksBox->edited() ) )
			return QString( "    # Autoprobe clocks\n" );

    if ( maxClock > 1.0 ) {
			QString tmp;
			// MM990714: This keyword DOES NOT work!
			tmp.sprintf( "    # MaxClock %g\n", maxClock );
			return tmp;
    }

    if ( !clocks.size() )
			return QString( "    # No usable clocks entered\n" );

    QString tmp( "    Clocks " );
    for( uint i=0; i<clocks.size(); i++ ) {
	QString num;
	num.setNum( clocks[i] );
	tmp = tmp + " " + num;
    }
    return tmp + "\n";
}



void LVideoCard::updateClocks()
{
    if ( maxClockButton->isChecked() ) {
	maxClockBox->setEnabled( TRUE );
	if ( clocksBox->hasFocus() )
	    maxClockBox->setFocus();
	clocksBox->setEnabled( FALSE );
	maxClockBox->setEdited( FALSE );
    } else {
	clocksBox->setEnabled( TRUE );
	if ( maxClockBox->hasFocus() )
	    clocksBox->setFocus();
	maxClockBox->setEnabled( FALSE );
	clocksBox->setEdited( FALSE );
    }
}


void LVideoCard::controlNextButton()
{
    bool ok = TRUE;
    if ( maxClockBox->isEnabled() ) {
	double d = maxClockBox->text().toDouble( &ok );
	ok = ok && d > 1.0 && d < 500.0; // ### 500.0
    } else if ( clocksBox->isEnabled() ) {
	QString tmp( clocksBox->text().simplifyWhiteSpace() );
	ok = tmp.length() > 0;
	while ( ok && tmp.length() ) {
	    int i = tmp.find( "," );
	    QString next;
	    if ( i > 0 ) {
		next = tmp.mid( i+1, 32767 );
		tmp = tmp.left( i );
	    }
	    double d = tmp.toDouble( &ok );
	    if ( ok && ( d < 1.0 || d > 500.0 ) )
		ok = FALSE;
	    tmp = next;
	}
    }
    emit nextAvailable( this, ok );
}


void LVideoCard::hide()
{
    QWidget::hide();
    // seminasty hack: always run this baby.  it works because the
    // stuff is never used BEFORE this screen, therefore computing the
    // variables on Back does not hurt.
    if ( maxClockBox->isEnabled() ) {
	bool ok;
	double d = maxClockBox->text().toDouble( &ok );
	if ( ok && d > 1.0 && d < 500.0 )
	    maxClock = d;
	clocks.resize( 0 );
    } else if ( clocksBox->isEnabled() ) {
	QString tmp( clocksBox->text().simplifyWhiteSpace() );
	int i = 0;
	maxClock = 0;
	clocks.resize( 0 );
	float clock;
	while( tmp.length() && sscanf( tmp, "%g", &clock ) == 1 ) {
	    i++;
	    clocks.resize( i );
	    clocks[i-1] = clock;
	    uint pos = 0;
	    while( pos < tmp.length() && tmp[pos] != ',' && tmp[pos] != ' ' )
		pos++;
	    tmp = tmp.mid( pos+1, tmp.length() ).simplifyWhiteSpace();
	}
    }
}

#include "installselect.h"

void LVideoCard::serverChanged(const QString &name)
{
  x_server = name;

  QString server = LXServer::path() + name;

 	if( QFileInfo( "/tmp/xlink" ).exists() ){
 		if( !QFileInfo( "/tmp/xlink" ).readLink().length() ||
 			!QFile( QFileInfo( "/tmp/xlink" ).readLink() ).exists() ){
				if( LXServer::runSVGA )
					server = LXServer::path() + "XF86_SVGA";
				else
					server = LXServer::path() + "XF86_VGA16";
 		}
 	}else	if( !QFileInfo( server ).exists() ){
		if( LXServer::runSVGA )
			server = LXServer::path() + "XF86_SVGA";
		else
			server = LXServer::path() + "XF86_VGA16";
	}

  ::unlink( "/tmp/xlink" );
  ::symlink( server.ascii(), "/tmp/xlink" );

  // ugly hack to find out the RPM name
  QString rpm = x_server;
  rpm.replace(QRegExp("XF86"), "XFree86");
  rpm.replace(QRegExp("8514"), "IBM8514");
  rpm.replace(QRegExp("_"), "-");

  // Add the X-Server rpm to the files to install
/*  char ** item;
  for( item = skipList; *item; item++ ) {
		if( !strcmp( *item, rpm ) )//change skip list
			*item = NULL;
  }*/

  // disable the memory and clock info if using standard VGA16
  videoRam->setEnabled(!defaultVGA16());
  clocksBox->setEnabled(!defaultVGA16());
  maxClockBox->setEnabled(!defaultVGA16());
  maxClockButton->setEnabled(!defaultVGA16());
  clocksButton->setEnabled(!defaultVGA16());
  probeButton->setEnabled(!defaultVGA16());
}


void LVideoCard::cardSelected(const QString &name)
{
  LCardDBEntry *entry = db.lookup(name);

  if (entry)
    serverChanged(entry->Server);
  else
    serverChanged("XF86_SVGA");

  card_msg->setText(tr("Card was manually selected. Probing might hang your system."));
}


void LVideoCard::finish()
{
  // get the name of the server to use
  QString server = LXServer::path() + x_server;

  // create the X link
  QString source = server;
  QString target = "/mnt/root/" + LXServer::path() + "X";
  ::symlink( source.ascii(), target.ascii() );
	target = "/mnt/root/etc/X11/X";
  ::symlink( source.ascii(), target.ascii() );

}





































