/***************************************************************************
                             packagelistitem.cpp
                             -------------------                                         
    begin                : Sun Dec 26 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "packagelistitem.h"
#include <qstring.h>

PackageListItem::PackageListItem( QListView * parent, pkgSet *a_packages ):
	QListViewItem( parent, tr("All Packages") ), package( NULL ), packages( a_packages ), type( Root ), iTotalSize( 0 ), iSelectedSize( 0 ), selected( false )
{
	for( int i = 0; i < packages->numPackages; i++ )
		iTotalSize += packages->packages[i]->size;
}

PackageListItem::PackageListItem( PackageListItem * parent, QString a_label ):
	QListViewItem( parent, a_label ), package( NULL ), packages( NULL ), type( Group ), iTotalSize( 0 ), iSelectedSize( 0 ), selected( false )
{
}

PackageListItem::PackageListItem( PackageListItem * parent, QString a_label, packageInfo* a_package ):
	QListViewItem( parent, a_label ), package( a_package ), packages( NULL ), type( Package ), iTotalSize( a_package->size ), iSelectedSize( 0 ), selected( package->selected )
{
	package->item = this;
	if( !iTotalSize )
		iTotalSize = 1;
}

int PackageListItem::setPackageSelected()
{
	return setPackageSelected( !selected );
}

int PackageListItem::setPackageSelected( bool a_bl, bool a_blBacktrack )
{
	if( selected != a_bl ){
  	int oldSelectedSize = iSelectedSize;
  	if( type == Package ){ //Single Package
  		package->selected = a_bl;
  		int change = a_bl?iTotalSize:0 - oldSelectedSize;
//  printf( "sel: %d, change: %d\n", a_bl, change );
  		changeSize( change, a_blBacktrack );
  	}else{ // Group or Root
  		PackageListItem *tmpItem = (PackageListItem *)firstChild();
//  printf( "iSel: %d\n", iSelectedSize );
  		int newSize = 0;
  		while( tmpItem ){
  			newSize += tmpItem->setPackageSelected( a_bl, false );
				tmpItem = (PackageListItem *)tmpItem->nextSibling();
			}
//  printf( "iSel: %d, iNew: %d\n", iSelectedSize, newSize );
  		changeSize( newSize - oldSelectedSize, a_blBacktrack );
  	}
	}
	return iSelectedSize;
}

void PackageListItem::changeSize( const signed int a_size, bool a_blBacktrack )
{
//printf( "change size\n" );
	if( type != Root ){
  	iSelectedSize += a_size;
  	setText( 1, sizeToString( iSelectedSize ) );
   	selected = (iSelectedSize >= iTotalSize);
//printf( "sel: %d, total: %d\n", iSelectedSize, iTotalSize );
		if( a_blBacktrack/* && QListViewItem::parent()*/ )
			((PackageListItem*)QListViewItem::parent())->changeSize( a_size );
	}else{
		iSelectedSize = 0;
		bool select = true;
		for( int i = 0; i < packages->numPackages; i++ ){
			if( packages->packages[i]->selected ){
				iSelectedSize += packages->packages[i]->size;
			}else if( select && packages->packages[i]->inmenu ){
				select = false;
			}
		}
  	setText( 1, sizeToString( iSelectedSize ) );
		selected = select;
//printf( "set selected: %d\n", selected );
		emit totalChanged( iSelectedSize );
	}
}

void PackageListItem::initSize( int *a_iSelected, int *a_iTotal )
{
//printf( "init size\n" );
	PackageListItem *tmpItem = NULL;
	int a1, a2;
	switch( type ){
		case Package:
  		selected = package->selected;
  		iSelectedSize = selected?iTotalSize:0;
			break;
		case Group:
    	iSelectedSize = iTotalSize = 0;
    	tmpItem = (PackageListItem *)firstChild();
    	while( tmpItem ){
    		tmpItem->initSize( &a1, &a2 );
 	  		iSelectedSize += a1;
 	  		iTotalSize += a2;
    		tmpItem = (PackageListItem *)tmpItem->nextSibling();
    	}
 			selected = (iSelectedSize >= iTotalSize);
			break;
		case Root:
    	tmpItem = (PackageListItem *)firstChild();
    	while( tmpItem ){
    		tmpItem->initSize( &a1, &a2 );
    		tmpItem = (PackageListItem *)tmpItem->nextSibling();
    	}
 			changeSize( 0, false );// root counts itself by it's own method
   		emit totalChanged( iSelectedSize );
			break;
	}
  setText( 1, sizeToString( iSelectedSize ) );
  setText( 2, sizeToString( iTotalSize ) );

	*a_iSelected = iSelectedSize;
	*a_iTotal = iTotalSize;
}

QString PackageListItem::sizeToString( const int a_size )
{
	if( a_size <= 0 )	return QString( "" );
	int normsize = a_size;
	int level = 0;
  while( normsize > 1024 && level < 2 ){
		normsize >>= 10;
		level++;
	}
	switch( level ){
		case 1: return QString::number( normsize ) += "kB";
		case 2: return QString::number( normsize ) += "MB";
		case 3: return QString::number( normsize ) += "GB";
		case 4: return QString::number( normsize ) += "TB";// ;-)
		default:; return QString::number( normsize );
	}
}
