/***************************************************************************
                              partitioninfo.cpp
                             -------------------
    begin                : Thu Nov 11 1999
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd.
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.
                            http://bestlinux.net

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "config.h"
#include "partitioninfo.h"
#include "partitionview.h"
#include "mountpoint.h"
#include "fdisk.h"
#include "linlinehelp.h"
#include "liconloader.h"
#include "log.h" // try not to log on non-existent drive

#include <qapplication.h>
#include <qtextstream.h>
#include <qfile.h>
#include <qdir.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <linux/hdreg.h>	/* HDIO_GETGEO */
#include <linux/unistd.h>	/* _syscall */
#include <fcntl.h>		/* O_RDWR */
#include <sys/ioctl.h>
#include <stdlib.h> /* system() */
#include <qmessagebox.h>
#include <qregexp.h>
#include <sys/mount.h>
#include <sys/stat.h>

PartitionInfoItem::PartitionInfoItem( QListView* parent, const QString& descr, const QString& mp,
					const QString& dev, const QString& size,
					PartitionInfoItem::FormatState s)
    : QCheckListItem( parent, descr, QCheckListItem::CheckBox)
		
{
    setText( 1, mp);
    setText( 2, dev);
    setText( 3, size);
    device = dev;
    setFormatState( s );

    isoptional = isSwapfile() || mp == "swap"; // swap partitions or a swapfile are optional

    setOn( TRUE );

    setSelectable( FALSE );

    if ( !isOptional() )
			setPixmap(0, LIconLoader::loadIcon("hds_part.gif"));
}


void PartitionInfoItem::setFormatState(FormatState s)
{
    formatstate = s;
    switch (s) {
    case FormatYes:
			setText(4, PartitionInfo::tr("Yes"));
			break;
    case FormatNo:
			setText(4,  PartitionInfo::tr("No"));
			break;
    case FormatDone:
			setText(4,  PartitionInfo::tr("Done"));
			break;
    case FormatFormatting:
			setText(4, " "+ PartitionInfo::tr("Formatting") +" " );
			break;
    case FormatNothing:
    default:
			setText(4,  "" );
			break;
    }
}


void PartitionInfoItem::paintCell( QPainter * p,  const QColorGroup & cg,
				    int column, int width, int alignment )
{
    if ( formatState() == FormatFormatting ) {
    	QFont font = p->font();
    	QFont boldFont = font;
    	boldFont.setWeight( QFont::Bold );
    	p->setFont( boldFont );
    	QListViewItem::paintCell( p, cg, column, width, alignment );
    	p->setFont( font );
    }
    else {
    	if ( isOptional() )
    	    QCheckListItem::paintCell( p, cg, column, width, alignment );
    	else
    	    QListViewItem::paintCell( p, cg, column, width, alignment );
    }
}


PartitionInfo::PartitionInfo( PartitionView* pview_, QWidget *parent, const char *name )
    : QWidget( parent, name), done( false )
{
    pview = pview_;

    QHBoxLayout * lh = new QHBoxLayout( this, 0, 6);

    QVBoxLayout * l = new QVBoxLayout( 6 );
		lh->addStretch( 1 );
    lh->addLayout( l, 8 );
		lh->addStretch( 1 );

    l->addSpacing( 10 );

    view = new QListView( this );
		QFont font( QFont::defaultFont() );
		font.setFamily( "times" );
		font.setPointSize( 10 );
    view->setFont( font );
    view->setSorting( -1 );
    view->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
    l->addWidget( view, 10 );

    view->addColumn(tr("Purpose"));
    int mountPoint = view->addColumn(tr("Mount Point"));
    view->setColumnAlignment( mountPoint, AlignCenter );
    view->setColumnWidthMode( mountPoint, QListView::Maximum );

    view->addColumn(tr("Device"));
    int sizeCol = view->addColumn(" "+ tr("Size") +" ");
    view->setColumnAlignment( sizeCol, AlignRight );
    view->setColumnWidthMode( sizeCol, QListView::Maximum );

    int formatCol = view->addColumn(" "+ tr("Formatting") +" ");
    view->setColumnAlignment( formatCol, AlignCenter );
    view->setColumnWidthMode( formatCol, QListView::Maximum );

    formatButton = new QPushButton( tr("Format chosen partitions"), this );
    connect( formatButton, SIGNAL( clicked() ), this, SLOT( doFormatting() ) );

    l->addSpacing( 10 );
    l->addWidget( formatButton, 0, AlignCenter );
    l->addSpacing( 12 );

/*    LInlineHelp* inlineHelp = new LInlineHelp( this );
    inlineHelp->setDocument( "partitionsel_info_overview.qml" );
    lh->addWidget( inlineHelp );*/
}

PartitionInfo::~PartitionInfo()
{
}


void PartitionInfo::showEvent( QShowEvent * )
{
		formatButton->setEnabled( !done );

		if( !done )
	    init();
		else
			done = false;

    if ( formatButton->isEnabled() ){
#ifndef DO_NOT_REQUIRE_FORMATTING
      	emit nextEnabled( this, FALSE );
#endif
		}
}


void PartitionInfo::hideEvent( QHideEvent * )
{
//		done = false;
//    formatButton->setEnabled( TRUE ); //#####
}


void PartitionInfo::init()
{
    unMounting();

    bool valid = TRUE;
    view->clear();

    QString rootp = MountPoints::mountPartition( "/");
    QString userp = MountPoints::mountPartition( "/usr");
    QStringList swap = pview->allSwapPartitions();

    QStringList all = pview->allPartitions();
    QStringList::Iterator a;
    for (a = all.begin(); a != all.end(); a++) {
    	if (*a != rootp && *a != userp && !swap.contains(*a) ) {
    	    QString mp = MountPoints::mountPoint( *a );
    	    //	    if ( !mp.isNull() || Fdisk::willBeFormated(a) ) {
    	    if ( !mp.isNull() ) {
		    		new PartitionInfoItem( view, tr("Extra Partition"), mp, *a, pview->sizeOfPartitionAsString(*a),
    					Fdisk::willBeFormated(*a)?
    					PartitionInfoItem::FormatYes:PartitionInfoItem::FormatNo);
    	    }
    	}
    }

    if ( swap.isEmpty() ) {
	//### support for swap files disabled
// 	new PartitionInfoItem( view, tr("Virtual Memory"), "swap", tr("swapfile"), "96 MB",
// 				PartitionInfoItem::FormatNothing);
			printf( "Just check: swap IS EMPTY!?\n" );
			QMessageBox::information( this, tr("No Swap Partition"),
				tr("You must assign a swap partition for the install to proceed."),
				QMessageBox::Ok );
    } else {
      QStringList::Iterator s;
    	for (s = swap.begin(); s != swap.end(); ++s) {
					if( ((int)pview->sizeOfPartition(*s)) > 128 ){
						QString mess( tr("You have created a swap partition which is too large. The maximum size of a "
							"swap partition is %ld Megabytes.") );
						mess.replace( QRegExp( "%ld" ), "128" );
						char buf[128];
						sprintf( buf, " ( %s ). ", pview->sizeOfPartitionAsString(*s).ascii() );
						mess.replace( QRegExp( "\\. " ), buf );
						QMessageBox::information( this, tr("Swap Size Error"),
							mess,	QMessageBox::Ok );
					}
    	    new PartitionInfoItem( view , tr("Virtual Memory"), "swap", *s, pview->sizeOfPartitionAsString(*s),
    				    PartitionInfoItem::FormatNothing );
    	}
    }

    if ( !userp.isEmpty() ) {
			new PartitionInfoItem( view, tr("User Partition"), "/usr", userp, pview->sizeOfPartitionAsString(userp),
				PartitionInfoItem::FormatYes);
    }

    if ( rootp.isEmpty() ) {
			new PartitionInfoItem( view, tr("Root Partition"), "/", tr("None"), "0" );
			valid = FALSE;
    }else {
			new PartitionInfoItem( view, tr("Root Partition"), "/", rootp, pview->sizeOfPartitionAsString(rootp),
				PartitionInfoItem::FormatYes);
    }
}


/*!
  Formats all marked partitions with the appropriate file format (either ext2 or msdos)
 */
void PartitionInfo::doFormatting()
{
  // test, if the kernel will be installed at a position below the
  // 1024th cylinder. (Note that this can only happen in expert mode and perhaps 
  // should be handled right there.)

  QString boot = MountPoints::mountPartition("/boot");
  if (boot.isEmpty())
    boot = MountPoints::mountPartition("/");
 /*Rem by  aa
if (!pview->partitionBelow1024Cylinders(boot))
    QMessageBox::critical(this, tr("Warning"), tr("Root partition not completely within first 1024 cylinders.\nYour system may not boot."));
 */
    formatButton->setEnabled( FALSE );
    QApplication::setOverrideCursor( waitCursor );

    // first do the swap partitions (if any). We want to swap on them
    // before creating the file systems since that eats up so much
    // memory.

    PartitionInfoItem* i = (PartitionInfoItem*)view->firstChild();
    while (i) {
//printf( "SWAP: %s, isSwap %d, isOptional %d, isOn %d\n", i->device.ascii(), i->isSwapPartition(), i->isOptional(), i->isOn() );
    	if ( i->isSwapPartition() &&
    	     ( !i->isOptional() || i->isOn() ) ) {

					printf( "Formatting swap: %s!\n", i->device.ascii() );
					// HACK ?
					MountPoints::setMountPoint( i->device.ascii(), "swap" );

    	    pview->formatPartition( i->device );
    	}
    	i = (PartitionInfoItem*)i->nextSibling();
    }

    // now format all other partitions, while swap is (very likely)
    // available
	  int maxSlash = 1;
    i = (PartitionInfoItem*)view->firstChild();

    while (i) {
    	if ( i->formatState() == PartitionInfoItem::FormatYes )
    	    i->setFormatState( PartitionInfoItem::FormatFormatting );
    	view->viewport()->repaint();
    	QApplication::syncX();

    	if ( i->isPartition() )
    	    maxSlash = QMAX( maxSlash, MountPoints::mountPoint(i->device).contains('/') );
    	
    	if ( i->isPartition() &&
    	     i->formatState() == PartitionInfoItem::FormatFormatting
    	     && !i->isSwapPartition() )
    	    pview->formatPartition( i->device );
    	
    	if ( i->formatState() == PartitionInfoItem::FormatFormatting )
    	    i->setFormatState( PartitionInfoItem::FormatDone );
    	view->viewport()->repaint();
    	QApplication::syncX();
    	i = (PartitionInfoItem*)i->nextSibling();
    }

    QApplication::restoreOverrideCursor();
    emit nextEnabled(this, TRUE);
}


void PartitionInfo::unMounting()
{
	Log::closeMain();

  system("./extrabin/mount > /tmp/mount.log");
  QFile file("/tmp/mount.log");
  if (file.open(IO_ReadOnly)) {
    QStringList points;

    QRegExp regexp("/dev.* on /mnt/root");
    QTextStream inf(&file);
    while (!inf.eof()) {
      QString line = inf.readLine();
      if (regexp.match(line) >= 0) {
				line = line.mid(line.find(" on /mnt/root") + 4);
				line.truncate(line.find(" type"));
				QStringList::Iterator it = points.begin();
				while ( it != points.end() && (*it).length() > line.length() ) it++;
				points.insert(it, line);
      }
    }

    for (QStringList::ConstIterator it = points.begin(); it != points.end(); it++) {
      debug( "unmounting %s", (*it).ascii() );
      QString umount = "umount " + *it;
      if( system(umount.ascii()) ){
				perror( "unmounting" );
		    ::sync();
				sleep( 2 );
				// stop using the device, especially on debugging ;-)
				char buf[128];
				sprintf( buf, "fuser -k -HUP %s > /dev/null", (*it).ascii() );
				system( buf ); // sorry, guys
				if( system( umount.ascii() ) ) // once more
					printf( "unmounting still unsuccessful ;-(\n" );
			}
    }
  }
}

/* Mounts the partitions to the chosen mount points below /mnt/root/ */
void PartitionInfo::doMounting()
{
    if ( !isVisible() )
			init(); // doMounting can be used from outside

    PartitionInfoItem* i = (PartitionInfoItem*)view->firstChild();
    int maxSlash = 1;

    while (i) {
    	if ( i->isPartition() )
   	    maxSlash = QMAX( maxSlash, MountPoints::mountPoint(i->device).contains('/') );
    	i = (PartitionInfoItem*)i->nextSibling();
    }

    // now mount the new linux partitions for the installation

    // first the root partition
    QString rootp = MountPoints::mountPartition( "/");
    QString com;
    QTextOStream(&com) << "./extrabin/mount -text2 " << rootp << " /mnt/root";
    QApplication::flushX();
    system( com.ascii() );

    QDir dir;

    // then the others, sorted by the tree depth (i.e. number of slashes)
    for (int n = 1; n <= maxSlash; n++) {
    	i = (PartitionInfoItem*)view->firstChild();
    	while (i) {
   	    if ( i->isPartition() ) {
      		int id = pview->idOfPartition( i->device );
      		QString mp = MountPoints::mountPoint( i->device );
      		if ( i->device != rootp && mp.contains('/')==n ) {
printf( "mounting: %s\n", mp.ascii() );
      		  if( !mp.isEmpty() ){
      		    QString d( "/mnt/root" + mp );
							int pos = 10;
							while( pos < (signed)d.length() && (pos = d.find( "/", pos )) >= 0 ){
	      		    dir.mkdir(  d.left( pos ).ascii() );
                pos++;
							}
      		    dir.mkdir(  d.ascii() );
       		    com.truncate(0);
							if( id == 0x83 ){
        		    QTextOStream(&com) << "./extrabin/mount -text2 " << i->device << " " << d;
	       		    QApplication::flushX();
  	     		    system( com.ascii() );
							}else	if( id == 0x6 ){
        		    QTextOStream(&com) << "./extrabin/mount -tmsdos " << i->device << " " << d;
	       		    QApplication::flushX();
  	     		    system( com.ascii() );
							}
      		  }
      		}
   	    }
   	    i = (PartitionInfoItem*)i->nextSibling();
    	}
    }
    // ### ### does not check return value of mount ### ###
    writeFstab();

    publishMountPoints();
}


/*!
  inform the package installer about the mount points and available sizes
 */
void PartitionInfo::publishMountPoints()
{
    MountPoints::clearDirectories();
  
    PartitionInfoItem* i = (PartitionInfoItem*)view->firstChild();
    int maxSlash = 1;

    while (i) {
    	if ( i->isPartition() )
    	    maxSlash = QMAX( maxSlash, MountPoints::mountPoint(i->device).contains('/') );
    	i = (PartitionInfoItem*)i->nextSibling();
    }

    QString rootp = MountPoints::mountPartition( "/" );
    MountPoints::newDirectory( "/", pview->sizeOfPartition( rootp ) * 1024 );

    // then the others, sorted by the tree depth (i.e. number of slashes)
    for (int n = 1; n <= maxSlash; n++) {
    	i = (PartitionInfoItem*)view->firstChild();
    	while ( i ) {
   	    if ( i->isPartition() ) {
      		int id = pview->idOfPartition( i->device );
      		QString mp = MountPoints::mountPoint( i->device );
      		if ( i->device != rootp && mp.contains('/') == n ) {
     		    if( !mp.isEmpty() ){
							if( id == 0x83 || id == 0x6 )
	     		      MountPoints::newDirectory( mp, pview->sizeOfPartition( i->device ) * 1024 );
     		    }
  	   		}
	 	    }
   	    i = (PartitionInfoItem*)i->nextSibling();
    	}
    }
}

void PartitionInfo::writeFstab()
{
    QDir d;
    d.mkdir("/mnt/root/etc");
    QFile f( "/mnt/root/etc/fstab");
    if ( f.open(IO_WriteOnly) ) {
    	QTextStream t( &f );
    	PartitionInfoItem* i = (PartitionInfoItem*)view->firstChild();

    	QString line;
    	QString mp;
    	while( i ){
  	    if ( i->isPartition() && ( !i->isOptional() || i->isOn() ) ) {
	    		line = i->device;
  	  		mp = MountPoints::mountPoint( i->device );
    			if( mp.isEmpty() || mp == tr("Unused") ){//by Maddy
    		    i = (PartitionInfoItem*)i->nextSibling();
    		    continue;
	   		  }
  	  		line += "\t";
    			int id = pview->idOfPartition( i->device );
      		if ( id == 0x83 )
      		    line += mp + "\text2";
      		else if ( id == 0x6 )
      		    line += mp + "\tmsdos";
      		else if ( Fdisk::isVFatType( id ) )
      		    line += mp + "\tvfat";
      		else if (Fdisk::isSwapType( id ))
      		    line += "swap\tswap";
      		
      		line += "\tdefaults\t";
      		if( Fdisk::isSwapType( id ) || id == 0x6 ) {
      		    line += " 0 0 ";
      		} else {
     		    line += " 1 ";
     		    if ( mp == "/" )
	      			line += " 1 ";
     		    else
  	    			line += " 2 ";
      		}
      		
      		t << line << '\n';
        }
        i = (PartitionInfoItem*)i->nextSibling();
    	}

			QDir d( "/proc/ide/" );
			if( d.exists() && d.isReadable() ){
  			d.setFilter( QDir::Dirs );
  			d.setNameFilter( "hd*" );
  			const QFileInfoList *list = d.entryInfoList();
  			QFileInfoListIterator it( *list );      // create list iterator
  			QFileInfo *fi;
				int iCDNum = 0;
				char cddev[64];
  			while( (fi = it.current()) ){
//  printf( "Drive: %s\n", QString( fi->absFilePath() + "/media" ).ascii() );
  				QFile fileMedia( fi->absFilePath() + "/media" );
  				if( fileMedia.exists() && fileMedia.open( IO_ReadOnly ) ){
  					QString str;
  					if( fileMedia.readLine( str, 6 ) != -1 && str == "cdrom" ){
  						if( !iCDNum ){
								::mkdir( "/mnt/root/dev", 0755 );
								sprintf( cddev, "/dev/%s", fi->fileName().ascii() );
								::symlink( cddev, "/mnt/root/dev/cdrom" );
								::mkdir( "/mnt/root/mnt", 0755 );
								::mkdir( "/mnt/root/mnt/cdrom", 0755 );
  				    	t << "/dev/cdrom\t/mnt/cdrom\tiso9660\tnoauto,ro,users\t0 0 " << endl;
  						}else{
								sprintf( cddev, "/mnt/root/mnt/cdrom%d", iCDNum );
								::mkdir( cddev, 0755 );
  				    	t << "/dev/" << fi->fileName() << "\t/mnt/cdrom" << iCDNum << "\tiso9660\tnoauto,ro,users\t0 0 " << endl;
  						}
							iCDNum++;
						}
//  printf( "Type: %s\n", str.ascii() );
  					fileMedia.close();
  				}
  				++it;							
  			}
			}else{
				printf( "Couldn't read /proc/ide directory!\n" );
			}		

    	t << "/dev/fd0\t/mnt/floppy\tmsdos\tnoauto,users\t0 0 " << endl;
    	t << "proc\t/proc\tproc\tdefaults\t0 0 " << endl;
    	t << "devpts\t/dev/pts\tdevpts\tgid=5,mode=620\t0 0 " << endl;

    	f.close();

    }
}




































