/***************************************************************************
                              partitionitem.cpp
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "partitionitem.h"
#include "diskitem.h"
#include "diskorpartitionitem.h"
#include "mountpoint.h"
#include "fdisk.h"
#include "liconloader.h"

#include <sys/mount.h>

#include <qtextstream.h>

PartitionItem::PartitionItem( QListViewItem* parent, DiskItem* diskItem_, bool isNew_,
				int number_, bool bootable_, double start_, double size_, int id_)
    : DiskOrPartitionItem( parent )
{
    diskItem = diskItem_;
    willBeFormated = false;
    init( isNew_, number_, bootable_, start_, size_, id_);
    setOpen( TRUE );
}

void PartitionItem::init( bool isNew_, int number_, bool bootable_, double start_, double size_, int id_)
{
    number = number_;
    bootable = bootable_;
    start = start_;
    size = size_;
    id = id_;
    isNew = isNew_;
}

void PartitionItem::checkForFilesystem()
{
    if ( isNew ) {
			Fdisk::markAsNew( name() );
    } else if ( Fdisk::startup && id == 0x83 ) {
    	if ( ::mount( name(), "/mnt/tmp", "ext2",
    		      MS_MGC_VAL + MS_RDONLY, NULL ) ) {// trying to mount
    	    Fdisk::markAsNew( name() );
    	    isNew = TRUE;
    	} else {
    	    ::umount( "/mnt/tmp" );
    	}
    }
    bool mustBeFormated = ( Fdisk::isSwapType(id) |
			    MountPoints::mountPoint(name())== "/" |
			    MountPoints::mountPoint(name()) == "/usr" |
			    Fdisk::willBeFormated( name() ) );

    willBeFormated = mustBeFormated | isNew;
//printf( "checkForFileSystem %s : willBeFormated - %d\n", name().ascii(), willBeFormated );

    // do not format an extended partition
    if( Fdisk::isExtendedType(id) )
      willBeFormated = FALSE;

    // do not format your install source
    if( Fdisk::isInstallSource(name()) )
			willBeFormated = FALSE;

    Fdisk::setFormatting( name(), willBeFormated );

    updateView();
}

QString PartitionItem::name()
{
    QString result;
    if (disk().left(8) == "/dev/rd/" || disk().left(9) == "/dev/ida/")
      QTextOStream( &result ) << disk() << "p" << number;
    else
      QTextOStream( &result ) << disk() << number;
    return result;
}

int PartitionItem::cylinderOfSector( int s )
{
    return diskItem->cylinderOfSector( s );
}

double PartitionItem::megaByteOfSectors( int s )
{
    return diskItem->megaByteOfSectors( s );
}

QString PartitionItem::disk()
{
    return diskItem->disk;
}

int PartitionItem::heads()
{
    return diskItem->heads;
}

int PartitionItem::sectors()
{
    return diskItem->sectors;
}

int PartitionItem::cylinders()
{
    return diskItem->cylinders;
}


/* The item has changed. Inform the listview about the new values. */
void PartitionItem::updateView()
{
    setText(0, name());
    setPixmap(0, LIconLoader::loadIcon("hds_part.gif"));
    QString c;

    if (id != 0) {
	  	c.setNum( cylinderOfSector(start) );
			setText(1, c);
		 	c.setNum( cylinderOfSector(end()) );
			setText(2, c);
			c.setNum( sizeInMB() );
			setText(3, c + "MB");
    } else {
			// empty primary partition
			setText(1, "");
			setText(2, "");
			setText(3, "");
    }
    setText(4, QString(" ")+Fdisk::sysname( id ) );
    setText(5, bootable?"*": " ");
    if (MountPoints::mountPoint(name()) == QObject::tr("Unused"))
      setText(6, "");
    else
      setText(6, MountPoints::mountPoint( name() ) );

    if ( willBeFormated )
			setText(7, QObject::tr("Yes") );
    else
			setText(7, QObject::tr("No") );
}

/* Dump this partition in the sfdisk dump format */
QString PartitionItem::dump()
{   //  /dev/sda1 : start=       63, size=   96327, Id=83, bootable

    QString result;
    QTextOStream os( &result );
    os << name() << " : start= " << (unsigned long int)start << ", size= " << (unsigned long int)size << ", ";
      os.flags( QTextStream::hex );
    os << "Id =" << id;
    if (bootable)
			os << ", bootable";
    os << "\n";
//printf( "**dump: %s", result.ascii() );
    return result;
}

/* Is the partition an extended partition? */
bool PartitionItem::isExtended()
{
    return Fdisk::isExtendedType( id );
}

/* Is the partition a swap partition? */
bool PartitionItem::isSwap()
{
    return Fdisk::isSwapType( id );
}


