/***************************************************************************
                               partitionview.h
                             -------------------                                         
    begin                : Mon Dec 20 1999                                           
    copyright            : (C) 2000 SOT Finnish Software Engineering Ltd. 
    author		           : Anton Safonov (Maddy)
    author email         : as@bestlinux.net                                          
    url			             : http://www.bestlinux.net
    email		             : linux@bestlinux.net

	       	      Best Linux - Linux from Finland.                      
                            http://bestlinux.net                            

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef PARTITIONVIEW_H
#define PARTITIONVIEW_H

#include "config.h"

#include <qwidget.h>
#include <qlistview.h>
#include <qstring.h>

class ChangePartitionDialog;
class PartitionItem;
class DiskItem;
class DiskOrPartitionItem;

class PartitionView : public QListView{
    Q_OBJECT
public:
    PartitionView( QWidget *parent=0, const char *name=0 );
    ~PartitionView();

    // auxiliary function to use PartitionView as database for partitioning
    QStringList allPartitions();
    QStringList allSwapPartitions();
    PartitionItem* partition( const QString& name );
    double sizeOfPartition(const QString& name);
    QString sizeOfPartitionAsString(const QString& name);
    int idOfPartition(const QString& name);
    bool hasLinuxPartitions();
    DiskItem* disk( const QString& name );
    double capacityOfDisk(const QString& name);
    QString capacityOfDiskAsString(const QString& name);
    bool hasLinuxPartitions(const QString& name);
    bool hasDosWindowsPartitions(const QString& name);
    bool hasPartitions(const QString& name);

    bool partitionBelow1024Cylinders(const QString& name);

    void invalidateView();
    bool viewInvalidated() const { return view_invalidated; };
    void checkState();
    bool bootable( const QString& name);
    void setBootable( const QString& name, bool value);


 public slots:
    void updateView();

    void editCurrent();
    void addLogicalPartition();
    void removeOrClearCurrent();

    void writePartitionTable();
    void defaultPartitionTable();
    void rereadPartitionTable();
	
    void formatPartition( QString device);

signals:
    void currentAvailable( bool );
    void extendedAvailable( bool );
    void changed( bool );
    void notOriginal( bool );
    void diskAvailable( bool );

    // for the disk check progress bar
    void diskCheckMaximum(int);
    void diskCheckProgress(int);

private slots:
    void selectionChanged( QListViewItem * );

private:
    bool somethingWritten;

    DiskOrPartitionItem* current;

    PartitionItem* currentPartition();
    DiskItem* currentDisk();

    ChangePartitionDialog* partitionDialog;

    bool view_invalidated;

};

#endif

