/*
 * main.C. Part of the KDE project.
 *
 * Copyright (C) 1997 Matthias Ettrich
 *
 */

#include <qnamespace.h>
#include <qpushbt.h>
#include <qbitmap.h>
#include <qwindefs.h>
#include <qmsgbox.h>
#include <qdialog.h>
#include <qaccel.h>
#include <qstring.h>
#include <qfont.h>

#if defined(index)
#undef index
#endif

#include <qwidget.h>
#include <qmenubar.h>

#include "client.h"
#include "warning.h"
#include "logout.h"
#include "kwm.h"
#include "version.h"

#include "main.h"
#include "manager.h"

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <X11/X.h>
#include <X11/Xos.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/extensions/shape.h>
#include <X11/cursorfont.h>

#define INT8 _X11INT8
#define INT32 _X11INT32
#include <X11/Xproto.h>
#undef INT8
#undef INT32


int BORDER;

bool initting;
bool ignore_badwindow;

int handler(Display *d, XErrorEvent *e){
    char msg[80], req[80], number[80];
    bool ignore_badwindow = true; //maybe temporary

    if (initting &&
	(
	 e->request_code == X_ChangeWindowAttributes
	 || e->request_code == X_GrabKey
	 )
	&& (e->error_code == BadAccess)) {
      fprintf(stderr, ("lwm: it looks like there's already a window manager running.  lwm not started\n"));
      exit(1);
    }

    if (ignore_badwindow && (e->error_code == BadWindow || e->error_code == BadColor))
      return 0;

    XGetErrorText(d, e->error_code, msg, sizeof(msg));
    sprintf(number, "%d", e->request_code);
    XGetErrorDatabaseText(d, "XRequest", number, "<unknown>", req, sizeof(req));

    fprintf(stderr, "lwm: %s(0x%lx): %s\n", req, e->resourceid, msg);

    if (initting) {
        fprintf(stderr, ("lwm: failure during initialisation; aborting\n"));
        exit(1);
    }
    return 0;
}

static KWarning* kwarning = 0;
static Klogout* klogout = 0;
static QLabel* infoLabel = 0;
static myPushButton* infoIcon = 0;
static QFrame* infoFrame = 0;
static QFrame* infoFrameInner = 0;
static QFrame* infoFrameSeparator = 0;

static Client* top_client_before_button_press = 0;
extern bool do_not_draw;

Manager* manager;
MyApp* myapp = 0;

static int infoBoxVirtualDesktop = 0;
static Client* infoBoxClient = 0;
static QPixmap* pm_unknown = 0;
static void hideInfoBox(){
  infoFrame->hide();
  infoFrame->releaseMouse();
  do_not_draw = False;
}


static void createInfoBox(){
  if (!infoFrame){
    infoFrame = new QFrame(0, 0,
			   Qt::WStyle_Customize | Qt::WStyle_NoBorder | Qt::WStyle_Tool );
    infoFrame->setFrameStyle( QFrame::WinPanel | QFrame::Raised );
    infoFrameInner = new QFrame(infoFrame);
    infoFrameInner->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    infoFrameInner->show();
    infoLabel = new QLabel(infoFrame);
    infoLabel->show();
    infoFrameSeparator = new QLabel(infoFrame);
    infoFrameSeparator->setFrameStyle( QFrame::Panel | QFrame::Raised );
    infoIcon = new myPushButton(infoFrame);
  }
}
static QList<Window> *infoBoxWindowsWindow = 0;
static QList<QLabel> *infoBoxWindowsLabel = 0;
static void setInfoBoxText(QString text, Window w){
  if (!infoFrame)
    createInfoBox();
  if (!infoFrame->isVisible()){
    QFont fnt;
    fnt.setCharSet( QFont::KOI8R );// by Maddy
    fnt.setBold(true);
    fnt.setPointSize(14);
    infoLabel->setFont(fnt);
  }

  infoLabel->setText("");
  int d = 0;
  if (w != None){
    infoIcon->setPixmap(KWM::icon(w, 48, 48));
    if (infoIcon->pixmap()->isNull()){
      if (!pm_unknown)
	pm_unknown = loadIcon("unknown.xpm");
      infoIcon->setPixmap(*pm_unknown);
      if (infoIcon->pixmap()->isNull())
	w = None; // no icon :-(
    }
  }
  if (w != None){
    d = 54;
    infoLabel->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
  }
  else {
    infoLabel->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
  }
  int nw = 0;
  if (infoBoxWindowsLabel)
    nw = 10+infoBoxWindowsLabel->count() * 22;
  if (nw < infoLabel->fontMetrics().boundingRect(text).width())
    nw = infoLabel->fontMetrics().boundingRect(text).width();

  if (nw > QApplication::desktop()->width()*2/3-12-d){
    infoFrame->setGeometry(0,
			   QApplication::desktop()->height()/2-30,
			   QApplication::desktop()->width(),
			   w!=None?83:60);
    if (nw > QApplication::desktop()->width()-12-d)
      infoLabel->setAlignment(Qt::AlignVCenter);
  }
  else
    if (nw > QApplication::desktop()->width()/3-12-d){
      infoFrame->setGeometry(QApplication::desktop()->width()/6,
			     QApplication::desktop()->height()/2-30,
			     QApplication::desktop()->width()*2/3,
			     w!=None?83:60);
      if (infoLabel->fontMetrics().boundingRect(text).width() >
	  QApplication::desktop()->width()-12-d)
	infoLabel->setAlignment(Qt::AlignVCenter);
    }
    else
      infoFrame->setGeometry(QApplication::desktop()->width()/3,
			     QApplication::desktop()->height()/2-30,
			     QApplication::desktop()->width()/3,
			     w!=None?83:60);
  infoFrameInner->setGeometry(3, 3, infoFrame->width()-6, infoFrame->height()-6);

  if (w != None){
    infoIcon->setGeometry(6, 6, 48, 48);
    infoIcon->show();
    infoLabel->setGeometry(6+d, 6, infoFrame->width()-d-12,
			   infoFrame->height()-12-20);
    infoFrameSeparator->setGeometry(6, 55, infoFrame->width()-12, 2);
    infoFrameSeparator->show();
    if (infoBoxWindowsLabel){
      QLabel* l;
      for (l=infoBoxWindowsLabel->first();l;l=infoBoxWindowsLabel->next()){
	if (*(infoBoxWindowsWindow->at(infoBoxWindowsLabel->at())) == w){
	  l->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	  l->update();
	}
	else if (l->frameStyle() != QFrame::NoFrame){
	  l->setFrameStyle(QFrame::NoFrame);
	  l->update();
	}
      }
    }
  }
  else {
    infoIcon->hide();
    infoLabel->setGeometry(6, 6, infoFrame->width()-12,
			   infoFrame->height()-12);
    infoFrameSeparator->hide();
  }
  infoLabel->setText(text);

  if (!infoFrame->isVisible()){
    infoFrame->show();
    infoFrame->raise();
    infoFrame->grabMouse();
    do_not_draw = True;
  }
}

void setInfoBoxWindows(Client* c, bool traverse_all = false){
 static QPixmap* pm_menu = 0;
 if (!pm_menu)
   pm_menu = loadIcon("menu.xpm");


  if (!infoFrame)
    createInfoBox();
  if (!infoBoxWindowsWindow){
    infoBoxWindowsWindow = new QList<Window>;
    infoBoxWindowsWindow->setAutoDelete(true);
  }
  if (!infoBoxWindowsLabel)
    infoBoxWindowsLabel = new QList<QLabel>;
  QLabel* l;
  infoBoxWindowsWindow->clear();
  for (l=infoBoxWindowsLabel->first();l;l=infoBoxWindowsLabel->next())
    delete l;
  infoBoxWindowsLabel->clear();
  int x = 7;
  if (c) {
    Client* o = c;
    Window* w = 0;
    QPixmap pm;
    do {
      if (traverse_all ||
	  o->isOnDesktop(manager->currentDesktop())){
	l = new QLabel(infoFrame);
	l->setGeometry(x, 58, 20, 20);
	l->show();
	infoBoxWindowsLabel->append(l);
	w = new Window;
	*w = o->window;
	pm = KWM::miniIcon(*w, 16, 16);
	if (!pm.isNull())
	  l->setPixmap(pm);
	else
	  l->setPixmap(*pm_menu);
	infoBoxWindowsWindow->append(w);
	x += 22;
      }
      o = manager->previousClient(o);
    } while (o != c);
  }
}



kwmOptions options;

void debug_events(const char* s, long int l){
  printf("%s: %ld\n", s,l);
}
void debug_events(const char* s, void* v, long int l = 0){
  if (!l)
    printf("%s: %p\n", s,v);
  else
    printf("%s: %p / %ld\n", s,v,l);
}

QPushButton* ignore_release_on_this = 0;



void sighandler(int) {
    myapp->cleanup();
    QApplication::exit();
}

bool focus_grabbed(){
  return kwarning?kwarning->isVisible():False;
}


void showWarning(const char* text, bool with_button = true){
  XEvent ev;
  if (!kwarning){
    kwarning = new KWarning(0, 0, Qt::WStyle_Customize | Qt::WStyle_NoBorder | Qt::WStyle_Tool);
  }
  kwarning->setText(text, with_button);
  manager->timeStamp();
  while (XCheckMaskEvent(qt_xdisplay(), EnterWindowMask, &ev));
  myapp->processEvents();
  while (!kwarning->do_grabbing());
}

static void setStringProperty(const char* atomname, const char* value){
  Atom a = XInternAtom(qt_xdisplay(), atomname, False);
  QString str = value;
  XChangeProperty(qt_xdisplay(), qt_xrootwin(), a, XA_STRING, 8,
		  PropModeReplace, (unsigned char *)(str.data()),
		  str.length()+1);
}


// show the modal logout dialog
static void showLogout(){
  manager->raiseSoundEvent("Logout Message");
  if (!klogout){
    klogout = new Klogout(0, 0, Qt::WStyle_Customize | Qt::WStyle_NoBorder | Qt::WStyle_Tool);
    // next is a dirty hack to fix a qt-1.2 bug
    // (should be unnecessary with 1.3)
     unsigned long data[2];
     data[0] = (unsigned long) NormalState;
     data[1] = (unsigned long) None;
     Atom wm_state = XInternAtom(qt_xdisplay(), "WM_STATE", False);
     XChangeProperty(qt_xdisplay(), klogout->winId(), wm_state, wm_state, 32,
 		    PropModeReplace, (unsigned char *)data, 2);
    myapp->connect(klogout, SIGNAL(doLogout()), myapp, SLOT(doLogout()));
    myapp->connect(klogout, SIGNAL(changeToClient(QString)),
		   myapp, SLOT(changeToClient(QString)));
  }
  QStrList* no_session = manager->getNoSessionClients();
  QStrList* pseudo_session = manager->getPseudoSessionClients();
  QStrList* unsaved_data = manager->getUnsavedDataClients();
  klogout->prepareToShow(unsaved_data, pseudo_session, no_session);
  delete no_session;
  delete unsaved_data;
  while (!klogout->do_grabbing());
}

QRect &stringToRect(QString s, QRect *q){
  int x = 0;
  int y = 0;
  int w = 0;
  int h = 0;

  int a;
  a = s.find('+', 0);
  if (a>0){
    x = s.left(a).toInt();
    s = s.remove(0, a+1);
  }
  a = s.find('+', 0);
  if (a>0){
    y = s.left(a).toInt();
    s = s.remove(0, a+1);
  }
 a = s.find('+', 0);
  if (a>0){
    w = s.left(a).toInt();
    s = s.remove(0, a+1);
  }
  h = s.toInt();
  q->setRect(x, y, w, h);
  return *q;
}



static void grabKey(KeySym keysym, unsigned int mod){
  static int NumLockMask = 0;
  if (!keysym||!XKeysymToKeycode(qt_xdisplay(), keysym)) return;
  if (!NumLockMask){
    XModifierKeymap* xmk = XGetModifierMapping(qt_xdisplay());
    int i;
    for (i=0; i<8; i++){
      if (xmk->modifiermap[xmk->max_keypermod * i] ==
	  XKeysymToKeycode(qt_xdisplay(), XK_Num_Lock))
	NumLockMask = (1<<i);
    }
  }
  XGrabKey(qt_xdisplay(),
	   XKeysymToKeycode(qt_xdisplay(), keysym), mod,
	   qt_xrootwin(), True,
	   GrabModeAsync, GrabModeSync);
  XGrabKey(qt_xdisplay(),
	   XKeysymToKeycode(qt_xdisplay(), keysym), mod | LockMask,
	   qt_xrootwin(), True,
	   GrabModeAsync, GrabModeSync);
  XGrabKey(qt_xdisplay(),
	   XKeysymToKeycode(qt_xdisplay(), keysym), mod | NumLockMask,
	   qt_xrootwin(), True,
	   GrabModeAsync, GrabModeSync);
  XGrabKey(qt_xdisplay(),
	   XKeysymToKeycode(qt_xdisplay(), keysym), mod | LockMask | NumLockMask,
	   qt_xrootwin(), True,
	   GrabModeAsync, GrabModeSync);


}


// Like manager->activateClient but also raises the window and sends a
// sound event.
void switchActivateClient(Client* c, bool do_not_raise){
  if (!c->geometry.intersects(QApplication::desktop()->rect())){
    // window not visible => place it again.
    manager->doPlacement(c);
    manager->sendConfig(c);
  }
  if (!do_not_raise)
    manager->raiseClient(c);
  if (!CLASSIC_FOCUS){
    manager->activateClient(c);
    manager->raiseSoundEvent("Window Activate");
  }
}


void logout(){
  showWarning(("Preparing session ... "), false);
  XUngrabServer(qt_xdisplay());
  XSync(qt_xdisplay(), false);
  qApp->processEvents();
  manager->processSaveYourself();
  kwarning->release();
  showLogout();
}

MyApp::MyApp(int &argc, char **argv , const QString& ):QApplication(argc, argv ){

  DEBUG_EVENTS("RootWindow", qt_xrootwin());

  manager = 0;
  process_events_mode = false;

  int i;
  bool restore_session = false;
  for (i=1; i<argc; i++){
    if (QString("-version") == argv[i]){
      printf(KWM_VERSION);
      printf("\n");
      printf(("Copyright (C) 1997 Matthias Ettrich (ettrich@kde.org)\n"));
      ::exit(0);
    }
    else {
      printf(("Usage: "));
      printf("%s [-version] \n", argv[0]);
      ::exit(1);
    }
  }


  myapp = this;
  initting = true;
  XSetErrorHandler(handler);

  // these should be internationalized!
  setStringProperty("KWM_STRING_MAXIMIZE",   ("&Maximize"));
  setStringProperty("KWM_STRING_UNMAXIMIZE", ("&Restore"));
  setStringProperty("KWM_STRING_ICONIFY",    ("&Iconify"));
  setStringProperty("KWM_STRING_UNICONIFY",  ("&DeIconify"));
  setStringProperty("KWM_STRING_STICKY",     ("&Sticky"));
  setStringProperty("KWM_STRING_UNSTICKY",   ("&UnSticky"));
  setStringProperty("KWM_STRING_MOVE",       ("Mo&ve"));
  setStringProperty("KWM_STRING_RESIZE",     ("Resi&ze"));
  setStringProperty("KWM_STRING_CLOSE",      ("&Close"));
  setStringProperty("KWM_STRING_TODESKTOP",  ("&To desktop"));
  setStringProperty("KWM_STRING_ONTOCURRENTDESKTOP",
		    ("&Onto current desktop"));

  desktopMenu = new QPopupMenu;
  desktopMenu->installEventFilter(this);
  desktopMenu->setMouseTracking(True);
  desktopMenu->setCheckable(true);

  QObject::connect(desktopMenu, SIGNAL(activated(int)), this, SLOT(handleDesktopPopup(int)));


  operations = new QPopupMenu ();
  CHECK_PTR( operations );
  operations->installEventFilter(this);
  operations->setMouseTracking(True);

  QObject::connect(operations, SIGNAL(activated(int)), this, SLOT(handleOperation(int)));


  grabKey(XK_Tab, Mod1Mask);
  grabKey(XK_Tab, Mod1Mask | ShiftMask);
  grabKey(XK_Tab, ControlMask);
  grabKey(XK_Tab, ControlMask | ShiftMask);

  createKeybindings();

  options.titlebarPixmapActive = new QPixmap;
  options.titlebarPixmapInactive = new QPixmap;

  //CT 07mar98 prepare the placement stuff
  options.interactive_trigger = -1;

  readConfiguration();

  int number_of_desktops = 1;
  KWM::setNumberOfDesktops(number_of_desktops);
  KWM::setWindowRegion(1, QApplication::desktop()->geometry());

  KWM::switchToDesktop(1);


  XSync(qt_xdisplay(), False);

  XGrabServer(qt_xdisplay());
  XSelectInput(qt_xdisplay(), qt_xrootwin(),
	       KeyPressMask |
	       // 		 ButtonPressMask | ButtonReleaseMask |
	       PropertyChangeMask |
	       ColormapChangeMask |
	       SubstructureRedirectMask |
	       SubstructureNotifyMask
	       );

  XSync(qt_xdisplay(), False);

  manager = new Manager;
  connect(manager, SIGNAL(reConfigure()), this, SLOT(reConfigure()));
  XUngrabServer(qt_xdisplay());
  initting = false;

  if (restore_session)
    restoreSession();
}



BUTTON_FUNCTIONS getFunctionFromKey(const char* key_arg){
  QString key = key_arg;
  if( key == "Off" )
    return NOFUNC;
  else if( key == "Maximize" )
    return MAXIMIZE;
  else if( key == "Iconify" )
    return ICONIFY;
  else if( key == "Close" )
    return CLOSE;
  else if( key == "Sticky" )
    return STICKY;
  else if( key == "Menu" )
    return MENU;
  else
    return UNDEFINED;
}

const char* default_buttons[6] =
{"Menu", "Off", "Off", "Close", "Maximize", "Off"};

//{"Menu", "Off", "Off", "Close", "Maximize", "Iconify"};

void MyApp::readConfiguration(){
  QString key;
  int i;

  killTimers();

  // this belongs in kapp....
  activeTitleBlend = Qt::blue;
  inactiveTitleBlend = Qt::white;

  activeTitleColor = Qt::darkBlue;
  inactiveTitleColor = palette().normal().background();

  activeTextColor = Qt::white;
  inactiveTextColor = Qt::black;


  //CT 03Nov1998 - a titlebar font
  tFont=font();
  tFont.setBold(true);

  BORDER = 4;

  options.WindowMoveType = OPAQUE;

  options.WindowResizeType = OPAQUE;

  options.FocusPolicy = CLICK_TO_FOCUS;

  options.AltTabMode = KDE_STYLE;


  options.TitlebarLook = PLAIN;

  options.alignTitle = AT_LEFT;

  options.framedActiveTitle = true;

  options.PixmapUnderTitleText = false;

  options.ResizeAnimation = 0;

  options.ControlTab = false;

  options.MaximizeOnlyVertically = false;
  options.TitleAnimation = 50;

  if (options.TitleAnimation)
      startTimer(options.TitleAnimation);

  options.AutoRaise = false;
  options.ClickRaise = true;

  options.AutoRaiseInterval = 0;

  options.ElectricBorder = -1;

  options.ElectricBorderNumberOfPushes = 5;

  options.ElectricBorderPointerWarp = FULL_WARP;

  if(options.ElectricBorder > -1){
    if( manager)
      manager->createBorderWindows();
  }
  else{
    if( manager)
      manager->destroyBorderWindows();
  }
  options.ShapeMode = false;



  options.interactive_trigger = -1;
  options.Placement = SMART_PLACEMENT;

  options.BorderSnapZone = 10;

  options.WindowSnapZone = 10;


  options.rstart = qstrdup("rstart -v");

  options.titlebar_doubleclick_command = Client::operationFromCommand("winShade");

  options.TraverseAll = false;

  options.GimmickMode = false;

  options.CommandActiveTitlebar1 = mouseBinding("Raise");
  options.CommandActiveTitlebar2 = mouseBinding("Lower");
  options.CommandActiveTitlebar3 = mouseBinding("Operations menu");
  options.CommandInactiveTitlebar1 = mouseBinding("Activate and raise");
  options.CommandInactiveTitlebar2 = mouseBinding("Activate and lower");
  options.CommandInactiveTitlebar3 = mouseBinding("Activate");
  options.CommandWindow1 = mouseBinding("Activate, raise and pass click");
  options.CommandWindow2 = mouseBinding("Activate and pass click");
  options.CommandWindow3 = mouseBinding("Activate and pass click");
  options.CommandAll1 = mouseBinding("Move");
  options.CommandAll2 = mouseBinding("Toggle raise and lower");
  options.CommandAll3 = mouseBinding("Resize");
							


  for (i=0; i<6; i++){
    options.buttons[i] = getFunctionFromKey(default_buttons[i]);
  }

}

// returns a mouse binding for a given string
int MyApp::mouseBinding(const char* arg)
{
  QString com = arg;
  if (com == "Raise") return MouseRaise;
  if (com == "Lower") return MouseLower;
  if (com == "Operations menu") return MouseOperationsMenu;
  if (com == "Toggle raise and lower") return MouseToggleRaiseAndLower;
  if (com == "Activate and raise") return MouseActivateAndRaise;
  if (com == "Activate and lower") return MouseActivateAndLower;
  if (com == "Activate") return MouseActivate;
  if (com == "Activate, raise and pass click") return MouseActivateRaiseAndPassClick;
  if (com == "Activate and pass click") return MouseActivateAndPassClick;
  if (com == "Move") return MouseMove;
  if (com == "Resize") return MouseResize;
  if (com == "Nothing") return MouseNothing;
  return MouseNothing;
}


// execute one of the configurable mousebindings. Returns true it the
// binding was bound to something
bool  MyApp::executeMouseBinding(Client* c, int command){
  if (!c)
    return false;
  switch (command){
  case MouseRaise:
    manager->raiseClient(c);
    break;
  case MouseLower:
    manager->lowerClient(c);
    break;
  case MouseOperationsMenu:
    // must be handled by the client itself since it needs the position of the event!
    break;
  case MouseToggleRaiseAndLower:
    if (c == top_client_before_button_press)
      manager->lowerClient(c);
    else
      manager->raiseClient(c);
    break;
  case MouseActivateAndRaise:
    switchActivateClient(c);
    break;
  case MouseActivateAndLower:
    switchActivateClient(c,true);
    manager->lowerClient(c);
    break;
  case MouseActivate:
    switchActivateClient(c,true);
    break;
  case MouseActivateRaiseAndPassClick:
    switchActivateClient(c);
    return false;
    break;
  case MouseActivateAndPassClick:
    switchActivateClient(c,true);
    return false;
    break;
  case MouseMove:
    c->simple_move();
    break;
  case MouseResize:
    c->simple_resize();
    break;
  case MouseNothing:
    return false;
    break;
  default:
    return false;
  }
  return true;
}


void MyApp::writeConfiguration(){
}


void MyApp::saveSession(){
}

void MyApp::restoreSession(){
}

void MyApp::cleanup()
{
  writeConfiguration();
  manager->cleanup();
}

// put the focus on the window with the specified label.  Will switch
// to the appropriate desktop and eventually deiconiy the window
void MyApp::changeToClient(QString label){
  Client* c = manager->findClientByLabel(label);
  if (c){
    if (!c->isOnDesktop(manager->currentDesktop()))
      manager->switchDesktop(c->desktop);
    if (c->isIconified())
      c->unIconify();
    else {
      switchActivateClient(c);
    }
  }
}


 // Same as changeToClient above, but you can also specify names of
 // virtual desktops as label argument. Window names therefore have to
 // start with three blanks. changeToTaskClient is used in the current
 // session manager.
void MyApp::changeToTaskClient(QString label){
  if (label.left(3) == "   "){
    // client
    label.remove(0,3);
    changeToClient(label);
  }
  else {
    // desktop
    int i;
    for (i=1; i <= manager->number_of_desktops; i++){
      if (label == KWM::getDesktopName(i)){
	manager->switchDesktop(i);
	return;
      }
    }
  }
}

// process the logout: save session and exit.
void MyApp::doLogout(){
  manager->raiseSoundEvent("Logout");
  saveSession();
  writeConfiguration();
  manager->cleanup(true);
  exit();
}

// reread the kwm configuration files
void MyApp::reConfigure(){
  readConfiguration();
  manager->readConfiguration();
}

static void freeKeyboard(bool pass){
    XSync(qt_xdisplay(), False);
    if (!pass)
      XAllowEvents(qt_xdisplay(), AsyncKeyboard, CurrentTime);
    else
      XAllowEvents(qt_xdisplay(), ReplayKeyboard, CurrentTime);
    XSync(qt_xdisplay(), False);
}

static bool tab_grab = False;
static bool control_grab = False;


bool MyApp::handleKeyPress(XKeyEvent key){
  int kc = XKeycodeToKeysym(qt_xdisplay(), key.keycode, 0);
  int km = key.state & (ControlMask | Mod1Mask | ShiftMask);

  if (!control_grab){

    if( (kc == XK_Tab)  &&
	( km == (Mod1Mask | ShiftMask)
	  || km == (Mod1Mask)
	  )){
      freeKeyboard(False);
      if (!tab_grab){
 	if (options.AltTabMode == CDE_STYLE){
 	  // CDE style raise / lower
	  Client* c = manager->topClientOnDesktop();
	  Client* nc = c;
	  if (km & ShiftMask){
	    do {
	      nc = manager->previousStaticClient(nc);
	    } while (nc && nc != c &&
		     (!nc->isOnDesktop(manager->currentDesktop()) ||
		      nc->isIconified()));

	  }
	  else
	    do {
	      nc = manager->nextStaticClient(nc);
	    } while (nc && nc != c &&
		     (!nc->isOnDesktop(manager->currentDesktop()) ||
		      nc->isIconified()));
	  if (c && c != nc)
	    manager->lowerClient(c);
	  if (nc)
	    switchActivateClient(nc);
 	  return True;
 	}
	XGrabKeyboard(qt_xdisplay(),
		      qt_xrootwin(), False,
		      GrabModeAsync, GrabModeAsync,
		      CurrentTime);
	tab_grab = True;
	if (manager->current())
	  setInfoBoxWindows(manager->current(), options.TraverseAll);
	else
	  setInfoBoxWindows(manager->nextClient(0), options.TraverseAll);
	infoBoxClient = manager->current();
      }
      Client* sign = infoBoxClient;
      do {
	if (infoBoxClient != sign && !sign)
	  sign = infoBoxClient;
	if (km & ShiftMask)
	  infoBoxClient = manager->nextClient(infoBoxClient);
	else
	  infoBoxClient = manager->previousClient(infoBoxClient);
      } while (infoBoxClient != sign && infoBoxClient &&
	       !options.TraverseAll &&
	       !infoBoxClient->isOnDesktop(manager->currentDesktop()));

      if (!options.TraverseAll && infoBoxClient
	  && !infoBoxClient->isOnDesktop(manager->currentDesktop()))
	infoBoxClient = 0;
      if (infoBoxClient){
	QString s;
	if (!infoBoxClient->isOnDesktop(manager->currentDesktop())){
	  s = KWM::getDesktopName(infoBoxClient->desktop);
	  s.append(": ");
	}
	if (infoBoxClient->isIconified())
	  setInfoBoxText(s + QString("(")+infoBoxClient->label+")",
			 infoBoxClient->window);
	else
	  setInfoBoxText(s + infoBoxClient->label,
			 infoBoxClient->window);
      }
      else
	setInfoBoxText(("*** No Tasks ***"), None);
      return False;
    }
  }

  if (!tab_grab){


    if( (kc == XK_Tab)  &&
	( km == (ControlMask | ShiftMask)
	  || km == (ControlMask)
	  )){
      if (!options.ControlTab){
	freeKeyboard(True);
	return True;
      }
      if (!control_grab){
	XGrabKeyboard(qt_xdisplay(),
		      qt_xrootwin(), False,
		      GrabModeAsync, GrabModeAsync,
		      CurrentTime);
	control_grab = True;
	setInfoBoxWindows(0);
	infoBoxVirtualDesktop = manager->currentDesktop();
      }
      if (km & ShiftMask){
	infoBoxVirtualDesktop--;
	if (infoBoxVirtualDesktop == 0)
	  infoBoxVirtualDesktop = manager->number_of_desktops;
      }
      else{
	infoBoxVirtualDesktop++;
	if (infoBoxVirtualDesktop > manager->number_of_desktops)
	  infoBoxVirtualDesktop = 1;
      }
      setInfoBoxText(KWM::getDesktopName(infoBoxVirtualDesktop), None);
      return False;
    }
  }

  if (control_grab || tab_grab){
    if (kc == XK_Escape){
      XUngrabKeyboard(qt_xdisplay(), CurrentTime);
      hideInfoBox();
      tab_grab = False;
      control_grab = False;
      return True;
    }
    return False;
  }

  freeKeyboard(False);
  return False;
}

void MyApp::handleKeyRelease(XKeyEvent key){
  int i;
  if (tab_grab){
    XModifierKeymap* xmk = XGetModifierMapping(qt_xdisplay());
    for (i=0; i<xmk->max_keypermod; i++)
      if (xmk->modifiermap[xmk->max_keypermod * Mod1MapIndex + i]
	  == key.keycode){
	XUngrabKeyboard(qt_xdisplay(), CurrentTime);
	hideInfoBox();
	tab_grab = false;
	if (infoBoxClient){
	  if (!infoBoxClient->isOnDesktop(manager->currentDesktop()))
	    manager->switchDesktop(infoBoxClient->desktop);
	
	  if (infoBoxClient->state == NormalState){
	    switchActivateClient(infoBoxClient);
	  }
	  else{ // IconicState
	    infoBoxClient->unIconify();
	  }
	}
      }
  }
  if (control_grab){
    XModifierKeymap* xmk = XGetModifierMapping(qt_xdisplay());
    for (i=0; i<xmk->max_keypermod; i++)
      if (xmk->modifiermap[xmk->max_keypermod * ControlMapIndex + i]
	  == key.keycode){
	XUngrabKeyboard(qt_xdisplay(), CurrentTime);
	hideInfoBox();
	control_grab = False;
	manager->switchDesktop(infoBoxVirtualDesktop);
      }
  }
}



void  MyApp::timerEvent( QTimerEvent * ){
  if (manager->current())
    manager->current()->animateTitlebar();
}

 void MyApp::createKeybindings(){
}

// keybinding slots

void MyApp::slotKillWindowMode(){
  static Cursor kill_cursor = 0;
  if (!kill_cursor)
       kill_cursor = XCreateFontCursor(qt_xdisplay(), XC_pirate);
     if (XGrabPointer(qt_xdisplay(), qt_xrootwin(), False,
		      ButtonPressMask | ButtonReleaseMask |
		      PointerMotionMask |
		      EnterWindowMask | LeaveWindowMask,
		      GrabModeAsync, GrabModeAsync, None,
		      kill_cursor, CurrentTime) == GrabSuccess){
       XGrabKeyboard(qt_xdisplay(),
		     qt_xrootwin(), False,
		     GrabModeAsync, GrabModeAsync,
		     CurrentTime);
       killSelect();
       XUngrabKeyboard(qt_xdisplay(), CurrentTime);
       XUngrabPointer(qt_xdisplay(), CurrentTime);
     }
}

void MyApp::slotWindowOperations(){
  if (manager->current())
    manager->current()->showOperations();
}
void MyApp::slotWindowRaise(){
  if (manager->current())
    manager->raiseClient(manager->current());
}
void MyApp::slotWindowLower(){
  if (manager->current())
    manager->lowerClient(manager->current());
}
void MyApp::slotWindowClose(){
  if (klogout && klogout->isVisible()){
    klogout->cleanup();
    return;
  }
  doOperation(OP_CLOSE);
}
void MyApp::slotWindowIconify(){
  doOperation(OP_ICONIFY);
}
void MyApp::slotWindowResize(){
  doOperation(OP_RESIZE);
}
void MyApp::slotWindowMove(){
  doOperation(OP_MOVE);
}
void MyApp::slotWindowToggleSticky(){
  doOperation(OP_STICKY);
}

void MyApp::slotKMenu(){
  KWM::sendKWMCommand("kpanel:system");
}

void MyApp::slotSwitchOneDesktopLeft(){
  manager->moveDesktopInDirection(Manager::Left);
}
void MyApp::slotSwitchOneDesktopRight(){
  manager->moveDesktopInDirection(Manager::Right);
}
void MyApp::slotSwitchOneDesktopUp(){
  manager->moveDesktopInDirection(Manager::Up);
}
void MyApp::slotSwitchOneDesktopDown(){
  manager->moveDesktopInDirection(Manager::Down);
}

void MyApp::slotSwitchDesktop1(){
  manager->switchDesktop(1);
}
void MyApp::slotSwitchDesktop2(){
  manager->switchDesktop(2);
}
void MyApp::slotSwitchDesktop3(){
  manager->switchDesktop(3);
}
void MyApp::slotSwitchDesktop4(){
  manager->switchDesktop(4);
}
void MyApp::slotSwitchDesktop5(){
  manager->switchDesktop(5);
}
void MyApp::slotSwitchDesktop6(){
  manager->switchDesktop(6);
}
void MyApp::slotSwitchDesktop7(){
  manager->switchDesktop(7);
}
void MyApp::slotSwitchDesktop8(){
  manager->switchDesktop(8);
}



bool MyApp::eventFilter( QObject *obj, QEvent * ev){

    if (obj == operations || obj == desktopMenu){
      if (ev->type() == QEvent::MouseButtonRelease){
	// ignore the popup-close in some cases
	if (ignore_release_on_this){
	  QEvent ev(QEvent::Leave);
	  QMouseEvent mev (QEvent::MouseButtonRelease,
			   QCursor::pos(), LeftButton, LeftButton);
	  QApplication::sendEvent(ignore_release_on_this, &ev);
	  QApplication::sendEvent(ignore_release_on_this, &mev);
	  if (ignore_release_on_this->rect().
	      contains(ignore_release_on_this
		       ->mapFromGlobal(QCursor::pos()))){
	    ignore_release_on_this = 0;
	    return True;
	  }
	  ignore_release_on_this = 0;
	}
      }
    }
    return False;
}

bool MyApp::buttonPressEventFilter( XEvent * ev)
{

    top_client_before_button_press = manager->topClientOnDesktop();
    Client *c = manager->getClient(ev->xbutton.window);
    if (c) {
	c->stopAutoraise( (ev->xbutton.state & Mod1Mask) != Mod1Mask);

	if (c->isMenuBar()) {
	    XAllowEvents(qt_xdisplay(), ReplayPointer, CurrentTime);
	    XUngrabPointer(qt_xdisplay(), CurrentTime);
	    XSync(qt_xdisplay(), false);
	    return true;
	}
    }
	

    if (c && ev->xbutton.window == c->window){
	bool no_replay = false;
	bool mod1 = (ev->xbutton.state & Mod1Mask) == Mod1Mask;
	if ( mod1){
	    switch (ev->xbutton.button) {
	    case Button1:
		no_replay = executeMouseBinding(c, options.CommandAll1);
		break;	
	    case Button2: 	
		no_replay = executeMouseBinding(c, options.CommandAll2);
		break;
	    case Button3:
		no_replay = executeMouseBinding(c, options.CommandAll3);
		break;
	    }
	}
	else if (!c->isActive()){
	    switch (ev->xbutton.button) {
	    case Button1:
		no_replay = executeMouseBinding(c, options.CommandWindow1);
		break;
	    case Button2:
		no_replay = executeMouseBinding(c, options.CommandWindow2);
		break;
	    case Button3:
		no_replay = executeMouseBinding(c, options.CommandWindow3);
		break;
	    default:
		switchActivateClient(c,true);
		break;
	    }
	}
 	// unfreeze the passive grab which is active currently
	if (no_replay ) {
	    // wait for the button release, do not ungrab.
	  XAllowEvents(qt_xdisplay(), SyncPointer, CurrentTime);
	}
	else {
	  XAllowEvents(qt_xdisplay(), ReplayPointer, CurrentTime);
	  XUngrabPointer(qt_xdisplay(), CurrentTime);
	  XSync(qt_xdisplay(), false);
	}
	return true;
    }
    else {
	c = manager->getClientFromSizegrip(ev->xbutton.window);
	if (c) {
	    QCursor::setPos(c->mapToGlobal(QPoint(c->width()-1, c->height()-1)));
	    c->simple_resize();
	    return true;
	}
    }
    return false;
}


bool MyApp::x11EventFilter( XEvent * ev){
  static Atom kwm_win_frame_geometry = 0;
  if (! kwm_win_frame_geometry)
    kwm_win_frame_geometry = XInternAtom(qt_xdisplay(),"KWM_WIN_FRAME_GEOMETRY",False);

    if (process_events_mode) {
	switch (ev->type) {
	case EnterNotify:
	    return TRUE; //ignore these
	case LeaveNotify:
	    //ignore these if for a client
	    if (manager->getClient(ev->xcrossing.window))
		return TRUE;
	    break;
	case PropertyNotify:
	  {
	    // ignore kwm_win_frame_geometry (we generate lots of the
	    // when doing opaque resizing, so the event stack would
	    // overflow
	    if (ev->xproperty.atom == kwm_win_frame_geometry)
	      return TRUE;
	  }
	case ButtonPress:
	case ButtonRelease:
	case KeyPress:
	case KeyRelease:
	case ClientMessage:
	    // process these later
	    if (events_count == 49) {
		events_count--;
	    }
	    events[events_count++] = *ev;
 	    // debug("events: %d", events_count);
	    return TRUE;
	    break ;
	}
    }




  switch (ev->type) {
  case KeyPress:
    DEBUG_EVENTS("KeyPress", ev->xkey.window)
    return handleKeyPress(ev->xkey);
    break;
  case KeyRelease:
    DEBUG_EVENTS("KeyRelease", ev->xkey.window)
    handleKeyRelease(ev->xkey);
    return false;
    break;
  case ButtonPress:
    DEBUG_EVENTS("ButtonPress", ev->xbutton.window)
    return buttonPressEventFilter(ev);
  break;
  case ButtonRelease:
    DEBUG_EVENTS("ButtonRelease", ev->xbutton.window)
    break;
  case CreateNotify:
    DEBUG_EVENTS("CreationNotify", ev->xcreatewindow.window)
    return true;
    break;
  case MapRequest:
    DEBUG_EVENTS("MapRequest", ev->xmaprequest.window)
    manager->mapRequest(&ev->xmaprequest);
    break;
  case ConfigureRequest:
    DEBUG_EVENTS("ConfigureRequest", ev->xconfigurerequest.window)
    manager->configureRequest(&ev->xconfigurerequest);
    break;
  case CirculateRequest:
    DEBUG_EVENTS("CirculateRequest", ev->xcirculaterequest.window)
    manager->circulateRequest(&ev->xcirculaterequest);
    break;
  case UnmapNotify:
    DEBUG_EVENTS("UnmapNotify", ev->xunmap.window)
    DEBUG_EVENTS("UnmapNotifyEvent", ev->xunmap.event)
    manager->unmapNotify(&ev->xunmap);
    if (ev->xunmap.window != ev->xunmap.event){
      return true;
    }
    break;
  case DestroyNotify:
    DEBUG_EVENTS("DestroyNotify", ev->xdestroywindow.window)
    manager->destroyNotify(&ev->xdestroywindow);
    if (ev->xdestroywindow.window != ev->xdestroywindow.event){
      return true;
    }
    break;
  case ClientMessage:
    DEBUG_EVENTS("ClientMessage", ev->xclient.window)
    manager->clientMessage(ev);
    break;
  case ColormapNotify:
    DEBUG_EVENTS("ColormapNotify", ev->xcolormap.window)
    manager->colormapNotify(&ev->xcolormap);
    break;
  case PropertyNotify:
    DEBUG_EVENTS("PropertyNotify", ev->xproperty.window)
    manager->propertyNotify(&ev->xproperty);
    break;
  case SelectionClear:
    break;
  case SelectionNotify:
    break;
  case SelectionRequest:
    break;
  case EnterNotify:
    //DEBUG_EVENTS("EnterNotify", ev->xcrossing.window)
    if (!operations->isVisible())
      manager->enterNotify(&ev->xcrossing);
    break;
  case LeaveNotify:
    //DEBUG_EVENTS("LeaveNotify", ev->xcrossing.window)
    manager->leaveNotify(&ev->xcrossing);
    break;
  case ReparentNotify:
    DEBUG_EVENTS("ReparentNotify", ev->xreparent.window)
    DEBUG_EVENTS("ReparentNotifyParent:", ev->xreparent.parent)
    DEBUG_EVENTS("ReparentNotifyEvent:", ev->xreparent.event)
    manager->reparentNotify(&ev->xreparent);
    return true; //do not confuse Qt with these events...
    break;
  case MotionNotify:
    //DEBUG_EVENTS("MotionNotify", ev->xmotion.window)
    manager->motionNotify(&ev->xmotion);
    break;
  case ConfigureNotify:
    DEBUG_EVENTS("ConfigureNotify", ev->xconfigure.window)
    // this is because Qt cannot handle (usually does not need to)

	// SubstructureNotify events.
    if (ev->xconfigure.window != ev->xconfigure.event){
      return true;
    }
    break;
  case MapNotify:
    DEBUG_EVENTS("MapNotify", ev->xmap.window)
    if (ev->xmap.window != ev->xmap.event){
      return true;
    }
    break;
  case Expose:
    break;
  case FocusIn:
    break;
  case FocusOut:
      {
 	  // handle multi screen displays. The other window manager
 	  // might take the focus away.
  	  Client *c = manager->getClient(ev->xfocus.window);
  	  if (ev->xfocus.mode == NotifyNormal &&
	      ev->xfocus.detail != NotifyPointer &&
	      c && c == manager->current()){
	      c->setactive(False);
  	  }
      }

    break;
  case MappingNotify:
    break;
  default:
    if (manager->shape && ev->type == manager->shape_event)
      manager->shapeNotify((XShapeEvent *)ev);
    break;
  }

  return false;
}


void MyApp::myProcessEvents() {
    static bool inHere = FALSE;
    if (inHere)
	return;
    inHere = TRUE;
    events_count = 0;
    process_events_mode = true;
    processEvents();
    process_events_mode = false;
    int i;
    for (i = events_count - 1; i >= 0; i--) {
	XPutBackEvent(qt_xdisplay(), &events[i]);
    }
    events_count = 0;
    inHere = FALSE;
}

void MyApp::handleOperation(int itemId){
  if (Client::operation_client)
    Client::operation_client->handleOperation(itemId);
}

void MyApp::handleDesktopPopup(int itemId){
  if (Client::operation_client)
    Client::operation_client->ontoDesktop(itemId);
}

void MyApp::doOperation(int itemId){
  if (manager->current())
    manager->current()->handleOperation(itemId);
}



int main( int argc, char ** argv ){
  QApplication::setColorSpec( QApplication::ManyColor );
  //  QApplication::useXResourceManager( FALSE );
  MyApp a(argc, argv,"kwm");

  if (signal(SIGTERM, sighandler) == SIG_IGN)
    signal(SIGTERM, SIG_IGN);
  if (signal(SIGINT, sighandler) == SIG_IGN)
    signal(SIGINT, SIG_IGN);
  if (signal(SIGHUP, sighandler) == SIG_IGN)
    signal(SIGHUP, SIG_IGN);

  fcntl(ConnectionNumber(qt_xdisplay()), F_SETFD, 1);
  return a.exec();
}


